# --------------------------------------------------------
# Konvertierungen zwischen verschiedenen Formaten
# --------------------------------------------------------
# Folgende Konvertierungen sind vorhanden
#   GIF        -> PNG           gif_to_png()
#                 PostScript    gif_to_ps()
#   GnuPlot    -> GIF           gnuplot_to_gif()
#                 PostScript    gnuplot_to_ps()
#   PostScript -> GIF           ps_to_gif()
#   PSTeX      -> GIF           pstex_to_gif()
#   TIFF       -> GIF           tiff_to_gif()
#                 PostScript    tiff_to_ps()
#   XFig       -> GIF           fig_to_gif()
#                 PostScript    fig_to_ps()
#   XFigTex    -> GIF           figtex_to_gif()
#                 PSTeX         figtex_to_pstex()
#   Text       -> HTML          txt_to_html()
#
# Die Gre von Pixeldateien kann mit dem Paket Image::Size
# berechnet werden, das hier mit verwendet wird.

# Die Ausmae eines Bildes knnen mit folgendem Befehl ermittelt
# werden (Image::Size Paket):
#    ($width, $height) = imgsize("filename");
#

# -------------------------------------------------------------------
# Postscript nach GIF
# -------------------------------------------------------------------
sub ps_to_gif {
  my $name = shift;
  my $target = shift;
  my $trans = shift;
  my $scale = shift;
  my $angle = shift;
  my $antialias = shift;
  my $crop = shift;
  my $bname;
  my $ftype;        # eps oder ps
  my $res;
  my $location = cwd;

  # testen, ob PsToImg installiert ist
  &my_exit("No pstoimg detected, can't make image.\n") if ($PSTOIMG eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name");
  $ftype=&get_file_type("$name");
  $pfad=&get_abs_pfad("$name");

  if ($trans) { $trans = "-transparent"; }
  else { $trans = ""; }
  if ($scale) { $scale = "-scale $scale"; }
  else { $scale = ""; }
  if ($angle) { $angle = "-flip r$angle"; }
  else { $angle = ""; }
  if ($antialias) { $antialias = "-antialias"; }
  else { $antialias = ""; }
  if ($crop) { $crop = "-crop $crop"; }
  else { $crop = ""; }

  &my_print("convert $name to $OUTPUT_DIR/$target/$bname.gif\n");
  chdir "$pfad" or &my_exit("Error in chdir $pfad\n");

  $res = 0;
  if( (&my_system("$PSTOIMG -type gif $trans $scale $angle $antialias $crop -out tmp.gif \"$bname.$ftype\"") != 0) ||
      (move("tmp.gif","$WORK_DIR/$OUTPUT_DIR/$target/$bname.gif") == 0) ) {
    &my_exit("Convert $name to $OUTPUT_DIR/$target/$bname.gif failed ($!)\n");
  }

  chdir "$location" or &my_exit("Error in chdir $location\n");

  return "$res";
}

# -------------------------------------------------------------------
# Tiff nach GIF
# -------------------------------------------------------------------
sub tiff_to_gif {
  my $name = shift;
  my $target = shift;
  my $trans = shift;
  my $scale = shift;
  my $angle = shift;
  my $antialias = shift;
  my $bname;
  my $ftype;        # tif oder tiff
  my $res;
  my $location = cwd;

  # testen, ob TiffToPnm installiert ist
  &my_exit("No tifftopnm detected, can't make image.\n") if ($TIFFTOPNM eq "");
  # testen, ob PnmToPs installiert ist
  &my_exit("No pnmtops detected, can't make image.\n") if ($PNMTOPS eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name");
  #$ftype=&get_file_type("$name");
  #$pfad=&get_abs_pfad("$name");

  &my_print("convert $name to $OUTPUT_DIR/$target/$bname.gif\n");
  #chdir "$pfad" or &my_exit("Error in chdir $pfad\n");
  $res=(&main::my_system("$TIFFTOPNM \"$name\" > \"$TEMP_DIR/$bname.pgm\"")||
        &main::my_system("$PNMTOPS -noturn -center \"$TEMP_DIR/$bname.pgm\" > \"$TEMP_DIR/$bname.ps\"")||
  &ps_to_gif("$TEMP_DIR/$bname.ps","$target","$trans","$scale","$angle","$antialias"))&&
  &my_exit("Convert $name to $OUTPUT_DIR/$target/$bname.gif failed\n");
        
  unlink "$TEMP_DIR/$bname.ps","$TEMP_DIR/$bname.pgm";
  #chdir "$location" or &my_exit("Error in chdir $location\n");

  return "$res";
}

# -------------------------------------------------------------------
# Gre eines Fig-Bildes (in cm) berechnen
#    Basiert auf getFigMeasure, Version 0.91, 5.4.1998
# -------------------------------------------------------------------
sub fig_size {
  my $fn = shift;

  my $dpi=1200;
  my $dpcm;
  my $xmin=999999;
  my $xmax=-999999;
  my $ymin=999999;
  my $ymax=-999999;
  my @tmp;
  my @line;

  open (FIG,"$fn") || &my_exit("Can't read file: $fn\n");

  while(!(($_=<FIG>)=~/^Inches|^Metric/)){}

  $dpcm=450 if /Metric/;
  $dpcm=472 if /Inches/;

  while(!(($_=<FIG>)=~/^$dpi/)){}
  while(<FIG>){
    @tmp=();
    @line=();
    if (/^5/){                #ARC
      @line=split(/\s+/);
      @line=@line[16..21];
    } elsif (/^6/){                #Compound
      @line=split(/\s+/);
      @line=@line[1..4];
      $count=1;
      while($count){
        $_=<FIG>;
        ++$count if /^6/;
        --$count if /^-6/;
      }
    } elsif (/^1/){                #Ellipse
      @line=split(/\s+/);
      my $xcenter=$line[12];
      my $ycenter=$line[13];
      my $xradius=$line[14];
      my $yradius=$line[15];
      push(@tmp,$xcenter+$xradius);
      push(@tmp,$ycenter+$yradius);
      push(@tmp,$xcenter-$xradius);
      push(@tmp,$ycenter-$yradius);
      @line=@tmp;
    } elsif (/^2 5/){            #Picture Object
      $_=<FIG>;$_=<FIG>;
      @line=split(/\s+/);
      @line=@line[1..8];
    }elsif (/^[23]/){                #Polyline,Spline
      if (/^2/){$offset=2;}else{$offset=0;}
      @line=split(/\s+/);
      my $farrow=$line[11+$offset];
      my $barrow=$line[12+$offset];
      my $points=$line[13+$offset];
      foreach $i (1..$farrow+$barrow){
        $_=<FIG>;            #Zeilen mit Arrowdaten ueberlesen
      }
      my $lines=int(($points-1)/6+1);
      my $line="";
      foreach $i (1..$lines){
        $line .= <FIG>;
      }
      $line=~s/^\s+//;
      @line=split(/\s+/,$line);
    }elsif (/^4/){                #Text
      @line=split(/\s+/,$_,14);
      my $justification=$line[1];
      my $angle=$line[7];
      my $special=$line[8];
      my $height=$line[9];
      my $length=$line[10];
      my $x=$line[11];
      my $y=$line[12];
      my $chars=$line[13];
      if ($special & 2  && !($chars=~/\$/) || !($special & 2)){
        if ($justification==0){                #links
          push(@tmp,$x);
          push(@tmp,$y);
          push(@tmp,$x+$length);
          push(@tmp,$y-$height);
          push(@tmp,$x);
          push(@tmp,$y-$height);
          push(@tmp,$x+$length);
          push(@tmp,$y);
        }elsif($justification==1){            #zentriert
          $length=$length/2;
          push(@tmp,$x-$length);
          push(@tmp,$y);
          push(@tmp,$x+$length);
          push(@tmp,$y-$height);
          push(@tmp,$x-$length);
          push(@tmp,$y-$height);
          push(@tmp,$x+$length);
          push(@tmp,$y);
        }elsif($justification==2){            #rechts
          push(@tmp,$x);
          push(@tmp,$y);
          push(@tmp,$x-$length);
          push(@tmp,$y-$height);
          push(@tmp,$x);
          push(@tmp,$y-$height);
          push(@tmp,$x-$length);
          push(@tmp,$y);
        }
      } else {
        push(@tmp,$x);
        push(@tmp,$y);
        push(@tmp,$x);
        push(@tmp,$y-$height);
      }
      if ($angle){
        my $i;
        for($i=0;$i<=$#tmp;$i+=2){
          $tmpx=$tmp[$i]-$x;
          $tmpy=$tmp[$i+1]-$y;
          $tmp[$i]=($tmpx * cos($angle) - $tmpy * sin($angle))+$x;
          $tmp[$i+1]=($tmpx * sin($angle) + $tmpy * cos($angle))+$y;
        }
      }
      @line=@tmp;
    }
    while ($#line>0){
      my $tmpx=shift(@line);
      my $tmpy=shift(@line);
      $xmin=$tmpx if $tmpx<$xmin;
      $ymin=$tmpy if $tmpy<$ymin;
      $xmax=$tmpx if $tmpx>$xmax;
      $ymax=$tmpy if $tmpy>$ymax;
    }
  }
  close FIG;

  my $xcm=int(($xmax-$xmin)/$dpcm*100)/100;
  my $ycm=int(($ymax-$ymin)/$dpcm*100)/100;
  return ($xcm, $ycm);
}

# -------------------------------------------------------------------
# Fig nach GIF
# -------------------------------------------------------------------
sub fig_to_gif {
  my $name = shift;
  my $target = shift;
  my $trans = shift;
  my $scale = shift;
  my $angle = shift;
  my $antialias = shift;
  my ($real_width, $real_height);
  my $faktor="";
  my $faktor_h="";
  my $bname,$pfad;
  my $tmp;
  my $res;
  my $location = cwd;
        
  # testen, ob Fig2Dev installiert ist
  &my_exit("No fig2dev detected, can't make image.\n") if ($FIG2DEV eq "");
  # testen, ob PsToImg installiert ist
  &my_exit("No pstoimg detected, can't make image.\n") if ($PSTOIMG eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  ($real_width,$real_height) = &fig_size($name);

  # Skaliere Bild so, das es auf den DIN A4-Blatt passt,
  # sonst fehlt ein Stueck im GIF-Bild
  if ($real_width>14) {$faktor = 14/$real_width;} 
  if ($real_height>22) {
    $faktor_h = 22/$real_height;
    if ($faktor) {
      $faktor = $faktor_h if ($faktor > $faktor_h);
    } else {
      $faktor = $faktor_h;
    }
  }
        
  $bname=&get_base_name("$name");
  $tmp ="$bname.ps";
        
  if ($trans) { $trans = "-transparent"; }
  else { $trans = ""; }
  if ($scale) { $scale = "-scale $scale"; }
  else { $scale = ""; }
  if ($faktor) { $faktor ="-m $faktor"; } 
  if ($angle) { $angle = "-flip r$angle"; }
  else { $angle = ""; }
  if ($antialias) { $antialias = "-antialias"; }
  else { $antialias = ""; }

  &my_print("convert $name to $OUTPUT_DIR/$target/$bname.gif\n");
  chdir "$TEMP_DIR" or &my_exit("Error in chdir $TEMP_DIR\n");

  $res = 0;
  if ( (&my_system("$FIG2DEV -L ps -p Portrait $faktor \"$name\" > \"$TEMP_DIR/$tmp\"") != 0) ||
       (&my_system("$PSTOIMG -type gif $trans $scale $angle $antialias -out \"$bname.gif\" \"$tmp\"") != 0) ||
       (move("$bname.gif","$WORK_DIR/$OUTPUT_DIR/$target/$bname.gif") == 0) ) {
    &my_exit("Convert $name to $OUTPUT_DIR/$target/$bname.gif failed\n");
  }
      
  unlink "$tmp";
  chdir "$location" or &my_exit("Error in chdir $WORK_DIR\n");
  return "$res";
}

# -------------------------------------------------------------------
# FigTeX nach GIF
# -------------------------------------------------------------------
sub figtex_to_gif {
  my $name = shift;
  my $target = shift;
  my $trans = shift;
  my $scale = shift;
  my $angle = shift;
  my $antialias = shift;
  my $bname;
  my ($real_width, $real_height);
  my $faktor="";
  my $faktor_h="";
  my $res;
        
  # testen, ob Fig2Dev installiert ist
  &my_exit("No fig2dev detected, can't make image.\n") if ($FIG2DEV eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  ($real_width,$real_height) = &fig_size($name);

  # Skaliere Bild so, das es auf den DIN A4-Blatt passt,
  # sonst fehlt ein Stueck im GIF-Bild
  if ($real_width>14) {$faktor = 14/$real_width;}
  if ($real_height>22) {
    $faktor_h = 22/$real_height;
    if ($faktor) {
      $faktor = $faktor_h if ($faktor > $faktor_h);
    } else {
        $faktor = $faktor_h;
    }
  }
        
  $bname=&get_base_name("$name");
        
  if ($faktor) { $faktor ="-m $faktor"; } 
        
  &my_print("convert $name (figtex) to $OUTPUT_DIR/$target/$bname.gif\n");
  $res=(&my_system("$FIG2DEV -L pstex -p Portrait $faktor \"$name\" > \"$TEMP_DIR/$bname.pstex\"")|| 
  &my_system("$FIG2DEV -L pstex_t -p \"$TEMP_DIR/$bname.pstex\" $faktor \"$name\" > \"$TEMP_DIR/$bname.pstex_t\"")||
  &pstex_to_gif("$TEMP_DIR/$bname.pstex_t","$target","$trans","$scale","$angle","$antialias"))&&
  &my_exit("Convert $name (figtex) to $OUTPUT_DIR/$target/$bname.gif failed\n");
  unlink "$TEMP_DIR/$bname.pstex","$TEMP_DIR/$bname.pstex_t";
        
  return "$res";
}

# -------------------------------------------------------------------
# Gnu-Plot-Grafiken nach GIF
# -------------------------------------------------------------------
sub gnuplot_to_gif {
  my $name=shift;
  my $target = shift;
  my $trans = shift;
  my $scale = shift;
  my $angle = shift;
  my $antialias = shift;
  my $bname;
  my $line;
  my $res;
  my $tmp_file;
        
  # testen, ob Gnuplot installiert ist
  &my_exit("No gnuplot detected, can't make image.\n") if ($GNUPLOT eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name");
  $tmp_file = "$TEMP_DIR/$bname.plt";

  &my_print("convert $name to $OUTPUT_DIR/$target/$bname.gif ...\n");
  #Output-Datei zum schreiben oeffnen
  open(PLOT,">$tmp_file") or &my_exit("Error by open : $tmp_file\n");
  print PLOT "set terminal postscript portrait\n";
  print PLOT "set output \"$TEMP_DIR/$bname.ps\"\n";
  open(QUELLE,"$name") or &my_exit("Error by open : $name\n");
  while ($line=<QUELLE>) {     #lese die Datei Zeilenweise
    print PLOT $line;
  }
  close QUELLE;
  close PLOT;
        
  $res = (&my_system("$GNUPLOT \"$TEMP_DIR/$bname.plt\"") ||
          &ps_to_gif("$TEMP_DIR/$bname.ps","$target","$trans","$scale","$angle","$antialias","a"))&&
          &my_exit("Convert $name (gnuplot) to $OUTPUT_DIR/$target/$bname.gif failed\n");

  unlink "$tmp_file","$TEMP_DIR/$bname.ps";

  return $res;
}

# -------------------------------------------------------------------
# PsTeX nach GIF
# -------------------------------------------------------------------
sub pstex_to_gif {
  my $name = shift;
  my $target = shift;
  my $trans = shift;
  my $scale = shift;
  my $angle = shift;
  my $antialias = shift;
  my $bname;
  my $out = "";
  my $tex_file;
  my $aux_file;
  my $log_file;
  my $dvi_file;
  my $ps_file;
  my $res;

  # testen, ob Latex installiert ist
  &my_exit("No latex detected, can't make image.\n") if ($LATEX eq "");
  # testen, ob DviPs installiert ist
  &my_exit("No dvips detected, can't make image.\n") if ($DVIPS eq "");
  # testen, ob PsToImg installiert ist
  &my_exit("No pstoimg detected, can't make image.\n") if ($PSTOIMG eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name");

  $tex_file="$bname"."_tmp.tex";
  $aux_file="$bname"."_tmp.aux";
  $log_file="$bname"."_tmp.log";
  $dvi_file="$bname"."_tmp.dvi";
  $ps_file="$bname"."_tmp.ps";

  &my_print("convert $name to $OUTPUT_DIR/$target/$bname.gif ...\n");
  #Output-Datei zum schreiben oeffnen
  open(P_TEX,">$tex_file") or &my_exit("Error by open : $TEXFILE\n");
  print P_TEX "\\documentclass[11pt]{article}\n";
  print P_TEX "\\usepackage{epsfig, german, a4wide, amsfonts, pictex}\n";
  print P_TEX "\\thispagestyle{empty}\n";
  print P_TEX "\\begin{document}\n";
  print P_TEX "\\input{$name}\n";
  print P_TEX "\\end{document}\n";
  close(P_TEX);

  if ($trans) { $trans = "-transparent"; }
  else { $trans = ""; }
  if ($scale) { $scale = "-scale $scale"; }
  else { $scale = ""; }
  if ($angle) { $angle = "-flip r$angle"; }
  else { $angle = ""; }
  if ($antialias) { $antialias = "-antialias"; }
  else { $antialias = ""; }

  if (!($DEBUG)) {$out = '> /dev/null';}

  $res = 0;
  if ( (&my_system("$LATEX \"$tex_file\" $out") != 0) ||
       (&my_system("$DVIPS \"$dvi_file\"") != 0) ||
       (&my_system("$PSTOIMG -type gif $trans $scale $angle $antialias -crop a -out \"$bname.gif\" \"$ps_file\"") != 0) ||
       (move("$bname.gif","$WORK_DIR/$OUTPUT_DIR/$target/$bname.gif") == 0) ) {
    &my_exit("Convert $name to $OUTPUT_DIR/$target/$bname.gif failed\n");
  }
  unlink "$tex_file","$dvi_file","$ps_file","$aux_file","$log_file";

  return "$res";
}

# -------------------------------------------------------------------
# Fig nach Postscript
# -------------------------------------------------------------------
sub fig_to_ps {
  my $name = shift;
  my $target = shift;
  my $width = shift;
  my $height = shift;
  my $bname;
  my ($real_width, $real_height);
  my ($faktor,$parameter);
  my $res;

  # testen, ob Fig2Dev installiert ist
  &my_exit("No fig2dev detected, can't make image.\n") if ($FIG2DEV eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  ($real_width,$real_height) = &fig_size($name);
  $width =~ s/(\D+)$//g;
  $faktor = $width/$real_width if ($real_width);
      
  if ($faktor) {$parameter ="-m $faktor";}

  $bname=&get_base_name("$name")."_fig";

  &my_print("convert $name to $target/$bname.ps\n");
  $res=&main::my_system("$FIG2DEV -L ps -p Portrait $parameter \"$name\" > \"$WORK_DIR/$target/$bname.ps\"")&&
  &my_exit("Convert $name to $target/$bname.ps failed\n");
      
  return "$res";
}

# -------------------------------------------------------------------
# FigTeX nach PsTeX
# -------------------------------------------------------------------
sub figtex_to_pstex {
  my $name = shift;
  my $target = shift;
  my $width = shift;
  my $height = shift;
  my $scale = shift;
  my ($real_width, $real_height);
  my ($faktor,$parameter);
  my $res;
        
  # testen, ob Fig2Dev installiert ist
  &my_exit("No fig2dev detected, can't make image.\n") if ($FIG2DEV eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  ($real_width,$real_height) = &fig_size($name);
  $width =~ s/(\D+)$//g if ($width);
  $height =~ s/(\D+)$//g if ($height);
  if ($scale) {
    $faktor = $scale;
  } else {
    if ($width) {
      $faktor = $width/$real_width if ($real_width);
    } else {
      $faktor = $height/$real_height if (($height)&&($real_height));
    }
  }
        
  if ($faktor) {$parameter ="-m $faktor";} 

  $bname=&get_base_name("$name");

  &my_print("convert $name to $target/$bname\[.pstex, .pstex_t\]\n");
  $res=(&my_system("$FIG2DEV -L pstex -p Portrait $parameter \"$name\" > \"$WORK_DIR/$target/$bname.pstex\"")|| 
  &my_system("$FIG2DEV -L pstex_t -p $target/$bname.pstex $parameter \"$name\" > \"$WORK_DIR/$target/$bname.pstex_t\"")) &&
  &my_exit("Convert $name to $target/$bname\[.pstex, .pstex_t\] failed\n");
    
  return "$res";
}

# -------------------------------------------------------------------
# Gnu-Plot-Grafiken nach Postscript
# -------------------------------------------------------------------
sub gnuplot_to_ps {
  my $name=shift;
  my $target = shift;
  my $bname;
  my $line;
  my $res;
  my $tmp_file;
        
  # testen, ob Gnuplot installiert ist
  &my_exit("No gnuplot detected, can't make image.\n") if ($GNUPLOT eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name")."_plot";
        
  $tmp_file = "$TEMP_DIR/$bname.plt";

  &my_print("convert $name to $target/$bname.ps ...\n");

  #Output-Datei zum schreiben oeffnen
  open(PLOT,">$tmp_file") or &my_exit("Error by open : $tmp_file\n");
  print PLOT "set terminal postscript portrait\n";
  print PLOT "set output \"$TEMP_DIR/$bname.ps\"\n";
  open(QUELLE,"$name") or &my_exit("Error by open : $name\n");
  while ($line=<QUELLE>) {     #lese die Datei Zeilenweise
      print PLOT $line;
  }
  close QUELLE;
  close PLOT;
        
  $res = 0;
  if ( (&my_system("$GNUPLOT \"$TEMP_DIR/$bname.plt\"") != 0) ||
       (move("$TEMP_DIR/$bname.ps","$WORK_DIR/$OUTPUT_DIR") == 0) ) {
    &my_exit("Convert $name (gnuplot) to $OUTPUT_DIR/$target/$bname.ps failed\n");
  }

  unlink "$tmp_file";
  return $res;
}

# -------------------------------------------------------------------
# GIF nach PNG
# -------------------------------------------------------------------
sub gif_to_png {
  my $name = shift;
  my $target = shift;
  my $image = shift;
  my $tmp_pnm;
  my $res;

  # testen, ob Gif2Pnm installiert ist
  &my_exit("No giftopnm detected, can't make image.\n") if ($GIFTOPNM eq "");
  # testen, ob PnmToPng installiert ist
  &my_exit("No pnmtopng detected, can't make image.\n") if ($PNMTOPNG eq "");


  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name")."_gif";
  $tmp_pnm ="$TEMP_DIR/$bname.pnm";
  if ($image) { $image = "-image \"$image\""; }

  &my_print("convert $name to $target/$bname.png\n");
  $res=(&main::my_system("$GIFTOPNM -verbose $image \"$name\" >\"$tmp_pnm\"") ||
  &main::my_system("$PNMTOPNG \"$tmp_pnm\" >\"$WORK_DIR/$target/$bname.png\""))&&
  &my_exit("Convert $name to $target/$bname.png failed\n");
  unlink "$tmp_pnm";
  
  return "$res";
}

# -------------------------------------------------------------------
# GIF nach Postscript
# -------------------------------------------------------------------
sub gif_to_ps {
  my $name = shift;
  my $target = shift;
  my $image = shift;
  my $tmp_pnm;
  my $res;

  # testen, ob Gif2Pnm installiert ist
  &my_exit("No giftopnm detected, can't make image.\n") if ($GIFTOPNM eq "");
  # testen, ob PnmToPs installiert ist
  &my_exit("No pnmtops detected, can't make image.\n") if ($PNMTOPS eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name")."_gif";
  $tmp_pnm ="$TEMP_DIR/$bname.pnm";
  if ($image) { $image = "-image $image"; }

  &my_print("convert $name to $target/$bname.ps\n");
  $res=(&main::my_system("$GIFTOPNM $image \"$name\" > \"$tmp_pnm\"") ||
  &main::my_system("$PNMTOPS -noturn \"$tmp_pnm\" > \"$WORK_DIR/$target/$bname.ps\""))&&
  &my_exit("Convert $name to $target/$bname.ps failed\n");
  unlink "$tmp_pnm";
  
  return "$res";
}

# -------------------------------------------------------------------
# Tiff nach Postscript
# -------------------------------------------------------------------
sub tiff_to_ps {
  my $name = shift;
  my $target = shift;
  my $res;

  # testen, ob TiffToPnm installiert ist
  &my_exit("No tifftopnm detected, can't make image.\n") if ($TIFFTOPNM eq "");
  # testen, ob PnmToPs installiert ist
  &my_exit("No pnmtops detected, can't make image.\n") if ($PNMTOPS eq "");

  # es hat die Konvertierung stattgefunden
  $SOURCE_IS_CHANGE=1;

  $bname=&get_base_name("$name")."_tiff";

  &my_print("convert $name to $target/$bname.ps\n");
  $res=(&main::my_system("$TIFFTOPNM \"$name\" > \"$TEMP_DIR/$bname.pgm\"")||
        &main::my_system("$PNMTOPS -noturn -center \"$TEMP_DIR/$bname.pgm\" > \"$WORK_DIR/$target/$bname.ps\""))&&
  &my_exit("Convert $name to $target/$bname.ps failed\n");
      
  unlink "$TEMP_DIR/$bname.pgm";
  return "$res";
}

# -------------------------------------------------------------------
# Text nach HTML
# -------------------------------------------------------------------
sub txt_to_html {
  my $textfile = shift;
  my $htmlbody = shift;
  my $res = "";
  my $line;

  if ($htmlbody) {
    $res .= "<!Diese Datei wurde von Master-Generator erzeugt>\n";
    $res .= "<HTML>\n";
    $res .= "<HEAD>\n";
    $res .= "<TITLE> Source of $textfile </TITLE>\n";
    $res .= "</HEAD>\n";
    $res .= "<BODY BGCOLOR=$main::BGCOLOR TEXT=$main::TEXTCOLOR BACKGROUND=$main::BACKGROUND>\n";
    $res .= "<PRE>\n";
  }

  open (CODE,"$textfile")
    or &main::my_exit("Error by open : $textfile\n");
  while ($line=<CODE>) { 
        # Alle enthaltenen & schtzen
        $line =~ s/\&/&amp;/g;
        # Einige Sonderzeichen ersetzen: Tab, <, >, "
        $line =~ s/\t/      /g;
        $line =~ s/</&lt;/g;
        $line =~ s/>/&gt;/g;
        $line =~ s/\"/&quot;/g;
        # Umlaute ersetzen
        $line =~ s//&auml;/g;
        $line =~ s//&ouml;/g;
        $line =~ s//&uuml;/g;
        $line =~ s//&Auml;/g;
        $line =~ s//&Ouml;/g;
        $line =~ s//&Uuml;/g;
        $line =~ s//&szlig;/g;
        $res .= "$line"; 
  }
  close CODE;

  if ($htmlbody) {
    $res .= "</PRE>\n";
    $res .= "</BODY>\n";
    $res .= "</HTML>\n";
  }

  return $res;
}

1;
