#!/usr/local/gnu/bin/perl
#
# getFigMeasure.pl Version 0.91
#
# Karsten Moranz + Frank M. Thiesing
#
# getFigMeasure.pl <Fig-File>
#
# Liefert die Groesse des uebergebenen xfig-Bildes in cm.
# Es werden zwei Werte auf einer Zeile ausgegeben:
# <Xsize> <Ysize>
#
#
# Version 0.9, 4.4.1998
#
# - beruecksichtigt werden die Formate bis xfig 3.2
# - es werden 1200 dpi bzw. 450 dpcm angenommen
# - beruecksichtigt werden:
#   ARC, Compound, Ellipse, Picture Object, Polyline, Spline, Text
#   Text inklusive Adjustment und Rotation
# - In jedem Fall wird der Aufpunkt eines Textes und seine Hoehe beruecksichtigt.
# - Die Laenge eines Textes wird NICHT beruecksichtigt, wenn es sich
#   um einen "special" Text handelt, der "$" enthaelt, da dies eine
#   Formel in TeX wird, die hier nicht richtig vermessen werden kann.
#   Um diese Formel bei der Messung zu beruecksichtigen sollte in xfig
#   eine unsichtbare Box in den Ausmassen der spaeteren Formel gezeichnet werden.
#
# Version 0.91, 5.4.1998
#
# - korrekte Verarbeitung geschachtelter Compound-Umgebungen
#

$usage="getFigMeasure.pl <Fig-File>\n";

die $usage if $#ARGV<0;

$dpi=1200;
$xmin=999999;$xmax=-999999;$ymin=999999;$ymax=-999999;
$fn=$ARGV[0];
open (FIG,"$fn") || die "Konnte '$fn' nicht oeffnen!\n";

while(!(($_=<FIG>)=~/^Inches|^Metric/)){}
$dpcm=450 if /Metric/;
$dpcm=472 if /Inches/;

while(!(($_=<FIG>)=~/^$dpi/)){}
while(<FIG>){
	@tmp=();@line=();
	if (/^5/){				#ARC
		@line=split(/\s+/);
		@line=@line[16..21];
	}elsif (/^6/){				#Compound
		@line=split(/\s+/);
		@line=@line[1..4];
                $count=1;
                while($count){
			$_=<FIG>;
		        ++$count if /^6/;
	        	--$count if /^-6/;
		}
	}elsif (/^1/){				#Ellipse
		@line=split(/\s+/);
		$xcenter=$line[12];
		$ycenter=$line[13];
		$xradius=$line[14];
		$yradius=$line[15];
		push(@tmp,$xcenter+$xradius);
		push(@tmp,$ycenter+$yradius);
		push(@tmp,$xcenter-$xradius);
		push(@tmp,$ycenter-$yradius);
		@line=@tmp;
	}elsif (/^2 5/){			#Picture Object
		$_=<FIG>;$_=<FIG>;
		@line=split(/\s+/);
		@line=@line[1..8];
	}elsif (/^[23]/){				#Polyline,Spline
		if (/^2/){$offset=2;}else{$offset=0;}
		@line=split(/\s+/);
		$farrow=$line[11+$offset];
		$barrow=$line[12+$offset];
		$points=$line[13+$offset];
		foreach $i (1..$farrow+$barrow){
			$_=<FIG>;			#Zeilen mit Arrowdaten ueberlesen
		}
		$lines=int(($points-1)/6+1);
		$line="";
		foreach $i (1..$lines){
			$line .= <FIG>;
		}
		$line=~s/^\s+//;
		@line=split(/\s+/,$line);
	}elsif (/^4/){				#Text
		@line=split(/\s+/,$_,14);
		$justification=$line[1];
		$angle=$line[7];
		$special=$line[8];
		$height=$line[9];
		$length=$line[10];
		$x=$line[11];
		$y=$line[12];
		$chars=$line[13];
		if ($special & 2  && !($chars=~/\$/) || !($special & 2)){
		    if ($justification==0){				#links
			push(@tmp,$x);
			push(@tmp,$y);
			push(@tmp,$x+$length);
			push(@tmp,$y-$height);
			push(@tmp,$x);
			push(@tmp,$y-$height);
			push(@tmp,$x+$length);
			push(@tmp,$y);
		    }elsif($justification==1){			#zentriert
			$length=$length/2;
			push(@tmp,$x-$length);
			push(@tmp,$y);
			push(@tmp,$x+$length);
			push(@tmp,$y-$height);
			push(@tmp,$x-$length);
			push(@tmp,$y-$height);
			push(@tmp,$x+$length);
			push(@tmp,$y);
		    }elsif($justification==2){			#rechts
			push(@tmp,$x);
			push(@tmp,$y);
			push(@tmp,$x-$length);
			push(@tmp,$y-$height);
			push(@tmp,$x);
			push(@tmp,$y-$height);
			push(@tmp,$x-$length);
			push(@tmp,$y);
		    }
		} else {
		    push(@tmp,$x);
		    push(@tmp,$y);
		    push(@tmp,$x);
		    push(@tmp,$y-$height);
		}
		if ($angle){
		    &rotate($angle,$x,$y,*tmp);
		}
		@line=@tmp;
	}
	&comp_components(@line);
#	print "@line\n";
}
close FIG;
#print "$xmin/$xmax $ymin/$ymax\n";
$diffx=$xmax-$xmin;
$diffy=$ymax-$ymin;
$xcm=int($diffx/$dpcm*100)/100;
$ycm=int($diffy/$dpcm*100)/100;
print "$xcm $ycm\n";

sub comp_components{
	local(@components)=@_;

	while ($#components>0){
		$tmpx=shift(@components);
		$tmpy=shift(@components);
		$xmin=$tmpx if $tmpx<$xmin;
		$ymin=$tmpy if $tmpy<$ymin;
		$xmax=$tmpx if $tmpx>$xmax;
		$ymax=$tmpy if $tmpy>$ymax;
	}
}

sub rotate{
	local($angle,$x,$y,*points)=@_;

	for($i=0;$i<=$#points;$i+=2){
		$tmpx=$points[$i]-$x;
		$tmpy=$points[$i+1]-$y;
		$points[$i]=($tmpx * cos($angle) - $tmpy * sin($angle))+$x;
		$points[$i+1]=($tmpx * sin($angle) + $tmpy * cos($angle))+$y;
	}
	
}
