# Grafisches Make fr Mas2TeX
#  bentigt Perl/Tk
#  Version 06.02.2001

# ACHTUNG:
#   * Patch in latex2html ntig, damit auch '+' als Trenner bei
#     Option -html_version erkannt wird
#     (zur Zeit noch nicht ntig, da -math Option verwendet wird)

use File::Basename;
use Tk;
use Tk::Balloon;
use Tk::LabFrame;
use Tk::FileSelect;

#
# Ausgabe-Methoden
#
sub outputClear {
  $GUItext->delete('1.0', end);
}
sub outputLine {
  $GUItext->insert(end, $_[0], $_[1]);
  $GUItext->see(end);
  $GUItext->update();
}
sub setStatus {
  $GUIstatus->configure(-text => $_[0]);
  $GUIstatus->update();
}

#
# Ein Kommando in einer Pipe ausfhren,
#  dabei bleibt das Tk-Programm ansprechbar
#
sub runCommand {
  my $cmd = $_[0];

  # Kommando in Output-Fenster ausgeben
  outputLine(">>> $cmd\n","red");
  
  # Kommando ausfhren und kompletten Output anzeigen
  $output = qx/$cmd 2>&1/;
  outputLine($output);
}

#
# ganzes Verzeichnis mit Inhalt lschen
#
sub unlinkDir {
  my $dir = $_[0];
  my @files;

  opendir(DIR, $dir);
  @files = readdir(DIR);
  closedir(DIR);

  foreach $file (@files) {
    next if ($file eq "." || $file eq "..");
    if (-d "$dir/$file") {
      unlinkDir("$dir/$file");
    } else {
      unlink("$dir/$file");
    }
  }
  rmdir($dir);
}

#
# Info-Nachricht anzeigen
#
sub messageBox {
  $GUItop->messageBox(
      -icon    => $_[0],
      -type    => $_[1],
      -title   => $_[2],
      -message => $_[3]
  );
}

#
# Konfiguration einlesen/speichern
#
sub readConfig {
  # define some defaults
  $mas2tex{"BIN,-copy"}              = 0;
  $mas2tex{"BIN,-debug"}             = 1;
  $mas2tex{"BIN,-htmllocal"}         = 0;
  $mas2tex{"BIN,-local_icons"}       = 0;
  $mas2tex{"BIN,-makefile"}          = 1;
  $l2h{"BIN,-math"}                  = 1;
  $l2h{"TXT,-html_version"}          = '3.2';
  $l2h{"TXT,-split"}                = '+3';
  $l2h{"TXT,-toc_depth"}             = 3;
  $l2h{"TXT,-long_titles"}           = 3;
  $l2h{"BIN,-show_section_numbers"}  = 1;
  $l2h{"TXT,-info"}                  = '0';
  $l2h{"BIN,-local_icons"}           = 0;
  $l2hpost{"BIN,-remove_title"}      = 1;

  # require configuration (if it exists)
  ($CFG = $ENV{HOME}) =~ s/[\\\/]$//;
  $CFG .= "/.m2tmake";
  if (-r $CFG) {
    printf "Read configuration $CFG\n";
    require $CFG;
  }
}
sub writeConfig {
  ($CFG = $ENV{HOME}) =~ s/[\\\/]$//;
  $CFG .= "/.m2tmake";
  if (! open(FILE, ">$CFG")) {
    messageBox(error, ok, "Mas2TeX Make", "Can't write config '$CFG'.");
  } else {
    printf(FILE "\$MAKE_DIR = \"".quotemeta($MAKE_DIR)."\";\n");
    printf(FILE "\$MAKE_FILE = \"".quotemeta($MAKE_FILE)."\";\n");
    printf(FILE "\$MAKE_TARGET = \"".quotemeta($MAKE_TARGET)."\";\n");
    foreach $key (keys %mas2tex) {
      printf(FILE "\$mas2tex{\"$key\"} = \"".quotemeta($mas2tex{$key})."\";\n");
    }
    foreach $key (keys %l2h) {
      printf(FILE "\$l2h{\"$key\"} = \"".quotemeta($l2h{$key})."\";\n");
    }
    foreach $key (keys %l2hpost) {
      printf(FILE "\$l2hpost{\"$key\"} = \"".quotemeta($l2hpost{$key})."\";\n");
    }
    printf(FILE "1;\n");
    close(FILE);
  }
}

sub endProgram {
  writeConfig;
  exit;
}

#
# Starten des Make-Vorgangs
#
sub appendOpts {
  my ($cmd, %opts) = @_;
  my ($key, $type);
  foreach $key (keys %opts) {
    $type = substr($key, 0, 3);
    $opt  = substr($key, 4);
    if ($type eq "BIN") {
      if ($opts{$key}) { $cmd .= " $opt"; }
    } elsif ($type eq "TXT") {
      if ($opts{$key} ne "") { $cmd .= " $opt $opts{$key}"; }
    } else {
      print "Wrong option type - ignores: $type ($key)\n";
    }
  }
  return $cmd;
}
sub runMas2TeX {
  my ($type,$suff) = @_;
  setStatus("Mas2TeX in LaTeX-Dokument umwandeln ...");
  $cmd = "mas2tex -output $type -tex_output ${MAKE_FILE}${suff}.tex -dir ${MAKE_FILE}${suff}";
  $cmd = appendOpts($cmd, %mas2tex);
  runCommand("$cmd -- ${MAKE_FILE}.mas");
}
sub runL2H {
  setStatus("LaTeX-Quelle in HTML umwandeln ...");
  $cmd = "latex2html -dir ${MAKE_FILE} -info \"\" -title ${MAKE_FILE}";
  $cmd = appendOpts($cmd, %l2h);
  runCommand("$cmd ${MAKE_FILE}.tex");
}
sub runL2HPost {
  setStatus("HTML-Dokumente nachbehandeln ...");
  $cmd = "l2hpost -dir ${MAKE_FILE}";
  $cmd = appendOpts($cmd, %l2hpost);
  runCommand($cmd);
}
sub make {
  chdir($MAKE_DIR);
  my $cmd;

  changeGUIState("disabled");
  outputClear();
  if ($MAKE_TARGET eq "ps") {
    runMas2TeX("latex", "_latex");
    setStatus("LaTeX-Quelle kompilieren ...");
    runCommand("latex --interaction=nonstopmode ${MAKE_FILE}_latex");
    runCommand("latex --interaction=nonstopmode ${MAKE_FILE}_latex");
    setStatus("PostScript-Dokument erzeugen ...");
    runCommand("dvips -o ${MAKE_FILE}.ps ${MAKE_FILE}_latex");
  } elsif ($MAKE_TARGET eq "pdf") {
    runMas2TeX("pdf", "_pdf");
    setStatus("LaTeX-Quelle kompilieren (fr Inhaltsverzeichnis) ...");
    runCommand("latex --interaction=nonstopmode ${MAKE_FILE}_pdf");
    runCommand("latex --interaction=nonstopmode ${MAKE_FILE}_pdf");
    setStatus("PDF-Dokument erzeugen ...");
    runCommand("dvipdfm -p a4 -o ${MAKE_FILE}.pdf ${MAKE_FILE}_pdf");
  } elsif ($MAKE_TARGET eq "html") {
    runMas2TeX("html", "");
    setStatus("LaTeX-Quelle kompilieren (fr Inhaltsverzeichnis) ...");
    runCommand("latex --interaction=nonstopmode ${MAKE_FILE}");
    runCommand("latex --interaction=nonstopmode ${MAKE_FILE}");
    runL2H();
    runL2HPost();
  } elsif ($MAKE_TARGET eq "clean") {
    outputLine("Delete all created files ...\n");
    #unlink ".latex2html-init";
    unlink "$MAKE_FILE.ps";
    unlink "$MAKE_FILE.pdf";
    unlink "core";
    unlink <$MAKE_FILE*.toc>;
    unlink <$MAKE_FILE*.tex>;
    unlink <*.log>;
    unlink <*.dvi>;
    unlink <*.aux>;
    unlink <*.out>;
    unlink <*.ps.gz>;
    unlinkDir "${MAKE_FILE}_latex";
    unlinkDir "${MAKE_FILE}_pdf";
    unlinkDir "${MAKE_FILE}";
    outputLine("Done.\n");
  } else {
    &messageBox('error','OK','Make',"No actions defined for target $MAKE_TARGET");
  }
  setStatus("Fertig ...");
  changeGUIState("normal");
}

#
# Mas2TeX-Quelle auswhlen
#
sub browseFile {
#  #   Type names              Extension(s)    Mac File Type(s)
#  #
#  #---------------------------------------------------------
  @types =
    (["Mas2TeX Sources", [qw/.mas/]],
     ["All files",       '*']
  );

  my %options = ();
  $options{-filetypes}   = \@types;
  if ($MAKE_DIR ne "") { $options{-initialdir}  = $MAKE_DIR; }
  if ($MAKE_FILE ne "") { $options{-initialfile} = "${MAKE_FILE}.mas"; }
  my $newfile = $GUItop->getOpenFile(%options);
  if ($newfile ne "") {
    ($MAKE_FILE, $MAKE_DIR, $suf) = fileparse($newfile,'\.[^.]*');
    $GUIbtMake->configure(-state => normal);
  }
}

#
# Frame mit Mas2Tex-Optionen ersellen
#   Parameter: Parent-Widget Output-Format
#
sub create_cb_opt {
  my $frame   = $_[0];
  my $row     = $_[1];
  my $opt     = $_[2];
  my $var     = $_[3];
  my $balloon = $_[4];
  my $msg     = $_[5];
  my $cb = $frame->Checkbutton(-text => $opt, -variable => $var);
  $cb->grid(-row => $row, -column => 1, -sticky => 'w');
  $balloon->attach($cb, -state => "balloon", -balloonposition => "mouse", -msg => $msg);
}
sub create_en_opt {
  my $frame   = $_[0];
  my $row     = $_[1];
  my $opt     = $_[2];
  my $var     = $_[3];
  my $balloon = $_[4];
  my $msg     = $_[5];
  $frame->Label(-text => $opt)->grid(-row => $row, -column => 0, -sticky => "e");
  my $en = $frame->Entry(-textvariable => $var);
  $en->grid(-row => $row, -column => 1, -sticky => 'ew');
  $balloon->attach($en, -state => "balloon", -balloonposition => "mouse", -msg => $msg);
}
sub mas2tex_opts {
  my $parent  = $_[0];
  my $balloon = $_[1];
  my $output  = $_[2];

  my $f;

  $f=$parent->LabFrame(
             -label => "Mas2TeX Options",
             -labelside => "acrosstop"
           )->pack(-side => "left", -fill => "both", -expand => 1);
  $f=$f->Frame()->pack(-anchor => "nw");
  create_cb_opt( $f,  0, "-copy",          \$mas2tex{"BIN,-copy"}, $balloon,
      "Copy media files to output directory\n(only used, if HTML is created)");
  create_cb_opt( $f,  1, "-htmllink_copy", \$mas2tex{"BIN,-htmllink_copy"}, $balloon,
      "Also copy local HTML-documents to output directory (only with -copy)\n(only used, if HTML is created)");
  create_cb_opt( $f,  2, "-htmllocal",     \$mas2tex{"BIN,-htmllocal"}, $balloon,
      "?\n(only used, if HTML is created)");
  create_cb_opt( $f,  3, "-local_icons",   \$mas2tex{"BIN,-local_icons"}, $balloon,
      "copy navigation icons and background to output directory\n(only used, if HTML is created)");
  create_cb_opt( $f,  4, "-relative_path", \$mas2tex{"BIN,-relative_path"}, $balloon,
      "use relative paths for sources\n(only used, if HTML is created)");
  create_cb_opt( $f,  5, "-debug",         \$mas2tex{"BIN,-debug"}, $balloon,
      "Show full information while processing master files");
  create_cb_opt( $f,  6, "-statistics",    \$mas2tex{"BIN,-statistics"}, $balloon,
      "Show statistics of processed files");
  create_cb_opt( $f,  7, "-report",        \$mas2tex{"BIN,-report"}, $balloon,
      "Create report using R/RR commands");
  create_cb_opt( $f,  8, "-self_report",   \$mas2tex{"BIN,-self_report"}, $balloon,
      "Create automatic report");
  create_cb_opt( $f,  9, "-not_l2hinit",   \$mas2tex{"BIN,-not_l2hinit"}, $balloon,
      "Don't create standard L2H init file");
  create_cb_opt( $f, 10, "-make_all",      \$mas2tex{"BIN,-make_all"}, $balloon,
      "rebuild all sources");
  create_cb_opt( $f, 11, "-make_show",     \$mas2tex{"BIN,-make_show"}, $balloon,
      "rebuild all SHOW-environments");
  create_cb_opt( $f, 12, "-makefile",      \$mas2tex{"BIN,-makefile"}, $balloon,
      "optimizes processing for use in makefile");
  # [-make_tex]        - ??
}
sub l2h_opts {
  my $parent  = $_[0];
  my $balloon = $_[1];
  my $output  = $_[2];

  my $f;

  $f=$parent->LabFrame(
             -label => "LaTeX2HTML Options",
             -labelside => "acrosstop"
           )->pack(-side => "left", -fill => "both", -expand => 1);
  $f=$f->Frame()->pack(-anchor => "nw");
  create_cb_opt( $f, 0, "-math",               \$l2h{"BIN,-math"}, $balloon,
      "Use MATH extensions in HTML-code (not HTML-3.2 conform)");
  create_cb_opt( $f, 1, "-show_section_numbers", \$l2h{"BIN,-show_section_numbers"}, $balloon,
      "Show LaTeX-section numbers");
  create_cb_opt( $f, 2, "-local_icons", \$l2h{"BIN,-local_icons"}, $balloon,
      "Copy navigation icons into document directory");
  create_en_opt( $f, 3, "-html_version", \$l2h{"TXT,-html_version"}, $balloon,
      "Specify used HTML-version. Possible: 2.0, 3.0, 3.2, 4.0");
  create_en_opt( $f, 4, "-split", \$l2h{"TXT,-split"}, $balloon,
      "Stop making separate files at this depth (say \"-split 0\" for one huge HTML file).");
  create_en_opt( $f, 5, "-toc_depth", \$l2h{"TXT,-toc_depth"}, $balloon,
      "MISSING DESCRIPTION");
  create_en_opt( $f, 6, "-long_titles", \$l2h{"TXT,-long_titles"}, $balloon,
      "MISSING DESCRIPTION");
  create_en_opt( $f, 7, "-info", \$l2h{"TXT,-info"}, $balloon,
      "0=no info page at the end, 1=standard info page");

  #    latex2html [ -split *num* ] [ -link *num* ] [ -toc_depth *num* ]
  #    [ -(no)toc_stars ] [ -(no)short_extn ] [ -iso_language *lang* ]
  #    [ -(no)validate ] [ -(no)latex ] [ -(no)djgpp ] [ -(no)fork ]
  #    [ -(no)external_images ] [ -(no)ascii_mode ] [ -(no)lcase_tags ]
  #    [ -(no)ps_images ] [ -font_size *size* ] [ -(no)tex_defs ]
  #    [ -(no)navigation ] [ -(no)top_navigation ] [ -(no)buttom_navigation ]
  #    [ -(no)auto_navigation ] [ -(no)index_in_navigation ]
  #    [ -(no)contents_in_navigation ] [ -(no)next_page_in_navigation ]
  #    [ -(no)previous_page_in_navigation ] [ -(no)footnode ]
  #    [ -(no)numbered_footnotes ] [ -prefix *output_filename_prefix* ]
  #    [ -(no)auto_prefix ] [ -long_titles *num* ] [ -(no)custom_titles ]
  #    [ -title|-t *top_page_title* ] [ -(no)rooted ]
  #    [ -rootdir *output_directory* ] [ -dir *output_directory* ] [ -mkdir ]
  #    [ -address *author_address* | -noaddress ] [ -(no)subdir ]
  #    [ -info *0* | *1* | *string* ] [ -(no)auto_link ]
  #    [ -reuse *num* | -noreuse ] [ -(no)antialias_text ] [ -(no)antialias ]
  #    [ -(no)transparent ] [ -(no)white ] [ -(no)discard ]
  #    [ -image_type *type* ] [ -(no)images ]
  #    [ -accent_images *type* | -noaccent_images ] [ -style *style* ]
  #    [ -(no)parbox_images ] [ -(no)math ] [ -(no)math_parsing ]
  #    [ -(no)latin ] [ -(no)entities ] [ -(no)local_icons ]
  #    [ -(no)scalable_fonts ] [ -(no)images_only ]
  #    [ -(no)show_section_numbers ] [ -(no)show_init ]
  #    [ -init_file *Perl_file* ] [ -up_url *up_URL* ] [ -up_title *up_title* ]
  #    [ -down_url *down_URL* ] [ -down_title *down_title* ]
  #    [ -prev_url *prev_URL* ] [ -prev_title *prev_title* ]
  #    [ -index *index_URL* ] [ -biblio *biblio_URL* ] [ -contents *toc_URL* ]
  #    [ -external_file *external_aux_file* ] [ -(no)short_index ]
  #    [ -(no)unsegment ] [ -(no)debug ] [ -tmp *path* ] [ -(no)ldump ]
  #    [ -(no)timing ] [ -verbosity *num* ] [ -html_version *num* ]
  #    [ -(no)strict ] *file.tex* [ *file2.tex* ... ]
  #
  #    -link *num*
  #        Stop showing child nodes at this depth.
  #
  #    -(no)toc_stars
  #        MISSING_DESCRIPTION
  #
  #    -(no)short_extn
  #        If this is set all HTML file will have extension `.htm' instead of
  #        `.html'. This is helpful when shipping the document to PC systems.
  #
  #    -iso_language *lang*
  #        MISSING_DESCRIPTION
  #
  #    -(no)validate
  #        When this is set true, the HTML validator specified in l2hconf.pm
  #        will run.
  #
  #    -(no)latex
  #        Pass unknown environments to LaTeX. This is the default.
  #
  #    -(no)external_images
  #        If set, leave the images outside the document.
  #
  #    -(no)ascii_mode
  #        This is different from -noimages. If this is set, LaTeX2HTML will
  #        show textual tags rather than images, both in navigation panel and
  #        text (Eg. `[Up]' instead the up icon). You could use this feature to
  #        create simple text from your document, eg. with 'Save as... Text'
  #        from Netscape or with lynx -dump.
  #
  #    -(no)lcase_tags
  #        writes out HTML tag names using lowercase letters, rather than
  #        uppercase.
  #
  #    -(no)ps_images
  #        If set, use links to external postscript images rather than inlined
  #        bitmaps.
  #
  #    -font_size *size*
  #        To set the point size of LaTeX-generated GIF files, specify the
  #        desired value (i.e., `10pt', `11pt', `12pt', etc.). The default is
  #        to use the point size of the original LaTeX document. This value
  #        will be magnified by *$FIGURE_SCALE_FACTOR* and *$MATH_SCALE_FACTOR*
  #        defined in l2hconf.pm.
  #
  #    -(no)tex_defs
  #        Enable interpretation of raw TeX commands (default). Note: There are
  #        many variations of `\def' that LaTeX2HTML cannot process correctly!
  #
  #    -(no)navigation
  #        Put a navigation panel at the top of each page (default).
  #
  #    -(no)top_navigation
  #        Enables navigation links at the top of each page (default).
  #
  #    -(no)buttom_navigation
  #        Enables navigation links at the buttom of each page.
  #
  #    -(no)auto_navigation
  #        Put navigation links at the top of each page. If the page exceeds
  #        *$WORDS_IN_PAGE* number of words then put one at the bottom of the
  #        page.
  #
  #    -(no)index_in_navigation
  #        Put a link to the index page in the navigation panel.
  #
  #    -(no)contents_in_navigation
  #        Put a link to the table of contents in the navigation panel.
  #
  #    -(no)next_page_in_navigation
  #        Put a link to the next logical page in the navigation panel.
  #
  #    -(no)previous_page_in_navigation
  #        Put a link to the previous logical page in the navigation panel.
  #
  #    -(no)footnode
  #        Puts all footnotes onto a separate HTML page, called footnode.html,
  #        rather than at the bottom of the page where they are referenced.
  #
  #    -(no)numbered_footnotes
  #        If true, you will get every footnote applied with a subsequent
  #        number, else with a generic hyperlink icon.
  #
  #    -prefix *output_filename_prefix*
  #        Set the output file prefix, prepended to all `.html', `.gif' and
  #        `.pl' files. See also -auto_prefix.
  #
  #    -(no)auto_prefix
  #        Set this to automatically insert the equivalent of -prefix
  #        `basename-'", where "basename" is the base name of the file being
  #        translated.
  #
  #    -(no)custom_titles
  #        MISSING_DESCRIPTION
  #
  #    -title|-t *top_page_title*
  #        The title (displayed in the browser's title bar) the document shall
  #        get.
  #
  #    -(no)rooted
  #        MISSING_DESCRIPTION
  #
  #    -rootdir *output_directory*
  #        MISSING_DESCRIPTION
  #
  #    -mkdir
  #        Allow directory specified with -dir to be created if necessary.
  #
  #    -address *author_address* | -noaddress
  #        Supply your own string if you don't like the default "<Name>
  #        <Date>". -noaddress suppresses the generation of an address footer.
  #
  #    -(no)subdir
  #        If set (default), LaTeX2HTML creates (or reuses) another file
  #        directory. When false, the generated HTML files will be placed in
  #        the current directory.
  #
  #    -(no)auto_link
  #        MISSING_DESCRIPTION
  #
  #    -reuse *num* | -noreuse
  #        If false, do not reuse or recycle identical images generated in
  #        previous runs. If the html subdirectory already exists, start the
  #        interactive session. If *num* is nonzero, do recycle them and switch
  #        off the interactive session. If 1, only recycle images generated
  #        from previous runs. If 2, recycle images from the current and
  #        previous runs (default).
  #
  #    -(no)antialias_text
  #        Use anti-aliasing in the generation of images of typeset material;
  #        e.g. mathematics and text, e.g. in tables and {makeimage}
  #        environments.
  #
  #    -(no)antialias
  #        Use anti-aliasing in the generation of images of figures. This
  #        usually results in "sharper" bitmap images.
  #
  #    -(no)transparent
  #        If this is set to false then any inlined images generated from
  #        "figure" environments will NOT be transparent.
  #
  #    -(no)white
  #        This sets the background of generated images to white for
  #        anti-aliasing.
  #
  #    -(no)discard
  #        if true, the PostScript file created for each generated image is
  #        discarded immediately after its image has been rendered and saved in
  #        the required graphics format. This can lead to significant savings
  #        in disk-space, when there are a lot of images, since otherwise these
  #        files are not discarded until the end of all processing.
  #
  #    -image_type *type*
  #        Specify the type of bitmap images to be generated. Depending on your
  #        setup, LaTeX2HTML can generate gif or png images. Note: Gif images
  #        have certain legal restrictions, as their generation involves an
  #        algorithm patented by Unisys.
  #
  #    -(no)images
  #        If false, LaTeX2HTML will not attempt to produce any inlined images.
  #        The missing images can be generated "off-line" by restarting
  #        LaTeX2HTML with -images_only.
  #
  #    -accent_images *type* | -noaccent_images
  #        MISSING_DESCRIPTION
  #
  #    -style *style*
  #        MISSING_DESCRIPTION
  #
  #    -(no)parbox_images
  #        MISSING_DESCRIPTION
  #
  #    -(no)math_parsing
  #        MISSING_DESCRIPTION
  #
  #    -(no)latin
  #        MISSING_DESCRIPTION
  #
  #    -(no)entities
  #        MISSING_DESCRIPTION
  #
  #    -(no)scalable_fonts
  #        MISSING_DESCRIPTION
  #
  #    -(no)images_only
  #        When true, LaTeX2HTML will only try to convert the inlined images in
  #        the file images.tex which should have been generated automatically
  #        during previous runs. This is very useful for correcting "bad LaTeX"
  #        in this file.
  #
  #    -(no)show_init
  #        MISSING_DESCRIPTION
  #
  #    -init_file *Perl_file*
  #        MISSING_DESCRIPTION
  #
  #    -up_url *up_URL*, -up_title *up_title*
  #    -down_url *down_URL*, -down_title *down_title*
  #    -prev_url *prev_URL*, -prev_title *prev_title*
  #    -index *index_URL*,
  #    -contents *toc_URL*
  #    -biblio *biblio_URL*
  #        If both of the listed two options are set then the "Up" ("Previous"
  #        etc.) button of the navigation panel in the first node/page of a
  #        converted document will point to *up_URL* etc. *up_title* should be
  #        set to some text which describes this external link. Similarly you
  #        might use these options to link external documents to your
  #        navigation panel.
  #
  #    -external_file *external_aux_file*
  #        MISSING_DESCRIPTION
  #
  #    -(no)short_index
  #        If this is set then makeidx.perl will construct codified names for
  #        the text of index references.
  #
  #    -(no)unsegment
  #        Use this to translate a segmented document as if it were not
  #        segmented.
  #
  #    -(no)debug
  #        If this is set then intermediate files are left for later inspection
  #        and a lot of diagnostic output is produced. This output may be
  #        useful when searching for problems and/or submitting bug reports to
  #        the developers. Temporary files include $$_images.tex and
  #        $$_images.log created during image conversion. Caution: Intermediate
  #        files can be *enormous*!
  #
  #    -tmp *path*
  #        Path for temporary files. This should be a local, fast filesystem
  #        because it is heavily used during image generation. The default is
  #        set in l2hconf.pm.
  #
  #    -(no)ldump
  #        This will cause LaTeX2HTML to produce a LaTeX dump of images.tex
  #        which is read in on subsequent runs and speeds up startup time of
  #        LaTeX on the images.tex translation. This actually consumes
  #        additional time on the first run, but pays off on subsequent runs.
  #        The dump file will need about 1 Meg of disk space.
  #
  #    -(no)timing
  #        MISSING_DESCRIPTION
  #
  #    -verbosity *num*
  #        The amount of message information printed to the screen during
  #        processing by LaTeX2HTML is controlled by this setting. By
  #        increasing this value, more information is displayed. Here is the
  #        type of extra information that is shown at each level:
  #
  #          0   no extra information
  #          1   section types and titles
  #          2   environment
  #          3   command names
  #          4   links, labels and internal sectioning codes
  #
  #    -html_version *list*
  #        Which HTML version should be generated. Currently available are:
  #        `2.0', `3.0', `3.2', `4.0'. Some additional options that may be
  #        added are: `math' (parse mathematics), `i18n' (?), `table' (generate
  #        tables), `frame' (generate frames), `latin1'...`latin9' (use
  #        ISO-Latin-x encoding), `unicode' (generate unicode characters).
  #        Separate the options with ',', e.g. `4.0,math,frame'.
  #
  #    -(no)strict
  #        MISSING_DESCRIPTION
}
sub l2hpost_opts {
  my $parent  = $_[0];
  my $balloon = $_[1];
  my $output  = $_[2];

  my $f;

  $f=$parent->LabFrame(
             -label => "HTML Postprocessing",
             -labelside => "acrosstop"
           )->pack(-side => "left", -fill => "both", -expand => 1);
  $f=$f->Frame()->pack(-anchor => "nw");
  create_cb_opt( $f, 0, "-remove_title", \$l2hpost{"BIN,-remove_title"}, $balloon,
      "Remove title ahead of links to child-sections");
}
sub centerInParent {
  my $win    = $_[0];
  my $parent = $_[1];

  $win->withdraw;
  $win->update;
  $x = $parent->rootx+int(($parent->width  - $win->width)/2);
  $y = $parent->rooty+int(($parent->height - $win->height)/2);
  $win->geometry("+$x+$y");
  $win->focus;
  $win->deiconify;
}

#
# Optionsfenster
#
sub options {
  my $opt = $GUItop->Toplevel(-title => "Options");
  my $bal = $opt->Balloon();

  $opt->Button(-text => "Ok", -command => [$opt=>destroy])->pack(-side => bottom, -anchor => 'e');
  mas2tex_opts($opt, $bal, $MAKE_TARGET);
  l2h_opts($opt, $bal, $MAKE_TARGET);
  l2hpost_opts($opt, $bal, $MAKE_TARGET);

  $opt->transient($GUItop);
  $opt->title("Options");
  changeGUIState("disabled");
  centerInParent($opt,$GUItop);
  $opt->waitWindow;
  changeGUIState("normal");
}

#
# Erstellen der Oberflche
#
sub createGUI {
  my $t,$f;

  # Angaben zur Mas2TeX-Quelle
  $t=$GUItop->LabFrame(
             -label => "Mas2TeX-Source",
             -labelside => "acrosstop"
           )->pack(-fill => "x");
  $f=$t->Frame()->pack(-fill => "x", -expand => 1);
  $f->Label(-text => "Source directory:")->grid(-row => 0, -column => 0, -sticky => "e");
  $GUIenDir = $f->Entry(-textvariable => \$MAKE_DIR, -width => "0");
  $GUIenDir->grid(-row => 0, -column => 1, -sticky => "ew");
  $f->Label(-text => "Mas2TeX file:")->grid(-row => 1, -column => 0, -sticky => "e");
  $GUIenFile = $f->Entry(-textvariable => \$MAKE_FILE, -width => "10");
  $GUIenFile->grid(-row => 1, -column => 1, -sticky => "ew");
  $GUIbtFile = $f->Button(-text => "Browse ..", -command => \&browseFile);
  $GUIbtFile->grid(-row => 0, -column => 2, -rowspan => 2, -padx => 2);
  $f->gridColumnconfigure(1, -weight => 1);

  # Statuszeile
  $GUIstatus = $GUItop->Label(-relief => sunken, -anchor => w, -fg => blue);
  $GUIstatus->pack(-side => bottom, -fill => both);

  # Ausgabefenster
  $GUItext = $GUItop->Scrolled('Text', -scrollbars => 'se', -wrap => 'none', -height => 10);
  $GUItext->pack(-side => bottom, -fill => both, -expand => 1);
  $GUItext->tagConfigure("red", -foreground => red);
  
  # Auswahl der Ausgabeformate
  $f=$GUItop->LabFrame(
              -label => "What to do",
              -labelside => "acrosstop"
          );
  $f->pack(-side => "left", -anchor => "n");
  $GUIbtOptions = $f->Button(-text => "Options", -command => \&options);
  $GUIbtOptions->pack(-side => right, -anchor => n);
  $GUIrbClean = $f->Radiobutton(
      -text     => "Cleanup",
      -variable => \$MAKE_TARGET,
      -relief   => 'flat',
      -value    => 'clean',
  );
  $GUIrbClean->pack(-side => top, -pady => 2, -anchor => w);
  $GUIrbPS=$f->Radiobutton(
      -text     => "printable: PostScript",
      -variable => \$MAKE_TARGET,
      -relief   => 'flat',
      -value    => 'ps',
  );
  $GUIrbPS->pack(-side => top, -pady => 2, -anchor => w);
  $GUIrbPDF=$f->Radiobutton(
      -text     => "printable/Online: PDF",
      -variable => \$MAKE_TARGET,
      -relief   => 'flat',
      -value    => 'pdf',
  );
  $GUIrbPDF->pack(-side => top, -pady => 2, -anchor => w);
  $GUIrbHTML=$f->Radiobutton(
      -text     => "Online: HTML",
      -variable => \$MAKE_TARGET,
      -relief   => 'flat',
      -value    => 'html',
  );
  $GUIrbHTML->pack(-side => top, -pady => 2, -anchor => w);

  # Knopf zum Beenden
  $GUIbtQuit=$GUItop->Button(
      -text    => 'Quit',
      -command => \&endProgram
  );
  $GUIbtQuit->pack(-side => bottom, -fill => x);

  # Knopf zum Starten des Vorgangs
  $GUIbtMake=$GUItop->Button(
      -text    => 'Make',
      -command => \&make
  );
  $GUIbtMake->pack(-side => bottom, -fill => x);

  if (($MAKE_DIR eq "") || ($MAKE_FILE eq "")) {
    $GUIbtMake->configure(-state => disabled);
  }
}
sub changeCursor {
  my $cursor = $_[0];

  foreach $w ($GUItop, $GUIenDir, $GUIenFile, $GUItext) {
    $w->configure(-cursor => $cursor);
  }
}
sub changeGUIState {
  my $state = $_[0];

  if ($state eq "disabled") { changeCursor("watch"); }
  $GUIenDir->configure(-state => $state);
  $GUIenFile->configure(-state => $state);
  $GUIbtOptions->configure(-state => $state);
  $GUIrbClean->configure(-state => $state);
  $GUIrbPS->configure(-state => $state);
  $GUIrbPDF->configure(-state => $state);
  $GUIrbHTML->configure(-state => $state);
  #$GUItext->configure(-state => $state);
  $GUIbtMake->configure(-state => $state);
  $GUIbtQuit->configure(-state => $state);
  if ($state eq "normal") { changeCursor(""); }
}

$GUItop = MainWindow->new;
$GUItop->withdraw();
if ($ENV{HOME} eq "") {
  messageBox(error, ok, "Mas2TeX Make", "You must define your HOME-variable in environment.");
  exit;
}
&readConfig();
&createGUI();
$GUItop->deiconify();
MainLoop;
