#
# @command DOCBEGIN
# @title   Starten einer mas2tex-Quelle
# @index   Master-Dokument!er\"offnen
# @syntax  \#DOCBEGIN\{optionen\}
# @desc    Mit dem {\tt \#DOCBEGIN}-Befehl mu"s jedes mas2tex-Dokument
#          beginnen. Dieser Befehl erzeugt die n"otige Einleitung, die
#          vom Ausgabemedium abh"angt. Es ist zwar m"oglich, die mas2tex-Quelle
#          wie eine normale \LaTeXe-Quelle zu beginnen, allerdings werden
#          dann nicht alle ausgabespezifischen Funktionen unterst"utzt
#          (wie z.B. interaktive Querverweise oder Lesezeichen in PDF-Dokumenten).
# @example_begin
#   #DOCBEGIN{class=article,option="1pp,draft"}
#     Hello, \LaTeX !          % Hier ist der Textteil
#   #DOCEND{}
# @example_end
#
#   @optname class
#   @optargs <Dokumentklasse> 
#     Mit dieser Option wird eine \LaTeXe-Dokumentklasse
#     eingegeben.\\
#     Defaultwert: {\tt article}
#
#   @optname package
#   @optargs \dq <Modul-Namensliste>\dq
#     Die Makromodule werden aus der Modul-Namensliste
#     in das Dokument importiert.\\
#     Defaultwert: \dq german,a4wide,amsfonts,epsfig,fancyheadings,pictex,latexsym\dq
#
#   @optname leader
#   @optargs \dq <praeembel>\dq
#     Die Pr"aambel setzen. 
#
#   @optname option
#   @optargs \dq <Optionsliste>\dq
#     Erweiterte Optionen f"ur das Dokument.\\
#     Defaultwert: \dq 11pt\dq
#
#   @optname input
#   @optargs \dq <Inputliste>\dq
#     Es werden die Dateien aus der Liste Inputliste eingebunden. 
#
#   @optname parindent
#   @optargs \dq Einzugsl"ange\dq
#     Steuerung der Einzugsl"ange f"ur gesamtes Dokument. Damit
#     wird der Einzug der ersten Zeilen der Abs"atzen beeinflu"st.\\
#     Defaultwert: \dq 0pt\dq
#
#   @optname parskip
#   @optargs \dq Vertikaler Abstand\dq
#     Mit der Option stellt man den Abstand zwischen den Abs"atzen
#     f"ur das gesamte Dokument ein.\\
#     Defaultwert: \dq 5pt\dq
#
# @command DOCEND
# @title   Beenden einer mas2tex-Quelle
# @index   Master-Dokument!beenden
# @syntax  \#DOCEND\{\}
# @desc    Der {\tt \#DOCEND}-Befehl beendet die mas2tex-Quelle. Text,
#          der nach dem Befehl folgt, wird ignoriert. Dies kann n"utzlich
#          sein, um noch nicht freigegebene Teile des Dokumentes zu
#          verstecken.
# @example_begin
#   #DOCBEGIN{class=article,option="1pp,draft"}
#     Hello, \LaTeX !          % Hier ist der Textteil
#   #DOCEND{}
# @example_end
#
# @end

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{DOCBEGIN}  = \&rumpf_work;
$MANAGEFUNKTION->{DOCEND}    = \&rumpf_work;

# DOCBEGIN/DOCEND
sub rumpf_work {
  package rumpf;

  my $COMMAND  = shift @_;
  my (@PARAMS) = @_;

  my ($wert,$flag);
  my $use_package   = "";
  my $use_leader    = "";
  my $use_class     = "";
  my $use_parskip   = "5pt";
  my $use_parindent = "0pt";
  my $use_font      = "";
  my $use_input     = "";
  my @inputs;

  if ($COMMAND eq "DOCBEGIN") {
    # Flags setzen
    foreach $term (@PARAMS) {    
      ($flag,$wert)=&main::my_quotewords("=",0,$term);
      $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
      SWITCH: {
        if ($flag eq "package")   {$use_package=$wert; last SWITCH;}
        if ($flag eq "leader")    {$use_leader=$wert; last SWITCH;}
        if ($flag eq "class")     {$use_class=$wert; last SWITCH;}
        if ($flag eq "parskip")   {$use_parskip=$wert; last SWITCH;}
        if ($flag eq "parindent") {$use_parindent=$wert; last SWITCH;}
        if ($flag eq "option")    {$use_font=$wert; last SWITCH;}
        if ($flag eq "input")     {$use_input=$wert; last SWITCH;}
        &main::my_exit("Unknown flag in $COMMAND : $flag\n");
      }
    }

    # Default-Werte setzen
    unless ($use_class) {
      $use_class="article";
    }
    unless ($use_font) { 
      if ($main::REPORT) { $use_font="12pt"; } 
      else { $use_font="11pt"; } 
    }
    unless ($use_package) {
      if ($main::OUTPUTFORMAT eq "html") {
        # fancyheadings & pictex entfernt, da unter MikTeX nicht verfgbar
        $use_package = "multicol,german,a4wide,amsfonts,epsfig";
        $use_package .= ",verbatim,html,latexsym,fancyheadings";
        $extra_packages = "";
      } elsif ($main::OUTPUTFORMAT eq "latex") {
        # fancyheadings & pictex entfernt, da unter MikTeX nicht verfgbar
        $use_package = "multicol,german,a4wide,amsfonts,epsfig";
        $use_package .= ",verbatim,latexsym,fancyheadings";
        if ($main::USE_PDF_FONTS) {
          $use_package .= ",times,mathptm";
        }
        $extra_packages = "";
      } else {
        # fancyheadings & pictex entfernt, da unter MikTeX nicht verfgbar
        $use_package  = "multicol,german,a4wide,amsfonts,epsfig";
        $use_package .= ",verbatim,latexsym,fancyheadings";
        $use_package .= ",times,mathptm,graphicx";
        $extra_packages  = "\\usepackage\[dvipdfm,a4paper=true,linkcolor=blue";
        $extra_packages .= ",colorlinks=true,bookmarks=true,bookmarksopen=true\]{hyperref}\n";
      }
    }

    &main::my_print("$COMMAND : $main::MASTERFILE.$main::EXT\n");
    &main::master_write(
      "% *** Master-Generator: Begin of Document $main::TEXFILE ***\n".
      "\\documentclass[$use_font]{$use_class}\n".
      "\\usepackage{$use_package}\n".
      "$extra_packages".
       "\\sloppy\n"
    );
    if ($use_leader) {
      &main::master_write($use_leader."\n");
    }
    if ($use_input) {
      @inputs = split /,/, $use_input;
      foreach $use_input (@inputs) {
        &main::master_write("\\input $use_input\n");
      }
    }
    if ($use_parindent) {
      &main::master_write("\\parindent$use_parindent\n");
    }
    if ($use_parskip) {
      &main::master_write("\\parskip$use_parskip\n");
    }
    &main::master_write(
      "\\begin{document}\n".
      "\\thispagestyle{empty}\n"
    );
    if ($main::REPORT || $main::SELF_REPORT) {
      $main::REPORT_WRITE = "";
    }
  } else {                    # dann DOCEND{}
    &main::master_write("% *** Master-Generator: End of Document ***\n");
    &main::master_write("\\end{document}\n");
    &main::my_print("$COMMAND : $main::MASTERFILE.$main::EXT\n");
    &main::my_end;
  }
}

1;
