# @command HTMLLINK
# @title   HTML-Links
# @index   
# @syntax  \#HTMLLINK\{optionen\}
# @desc    Der Befehl {\tt \#HTMLLINK} erm"oglicht es, eine HTML-Verkn"upfung in ein Master-Dokument einzubinden.
#          In der \LaTeXe-Version wird statt dieses Links entweder der Linkname als Text oder
#          alternatives Bild dargestellt. Wie eine Verkn"upfung in der HTML-Version erscheint, 
#          h"angt von der Kommandozeilenoption {\tt -htmllocal} (siehe Seite \pageref{htmllocal}) ab. 
#          Eine HTML-Adresse im Dokument wird in der HTML-Version zu einem Link. Dies kann man 
#          mit der Kommandozeilenoption {\tt -htmllocal} oder mit einem Backslash
#          vor der Adresse unterdr"ucken.
# @example_begin
#  #HTMLLINK{ url=http://www.leo.org,         % HTML-Adresse
#      alt=../Gif/leo.gif,                    % alternative Datei
#      sign="Software Archiv aus M\"unchen",  % Bildunterschrift
#      center,                                % Bild zentriert
#      imglink                                % sensitives Bild in HTML
#  }
# @example_end
#
#   @optname url
#   @optargs <htmladresse>
#       HTML-Adresse
#       \\Beispiel : {\tt url=\http://www.yahoo.de} 
#
#   @optname text
#   @optargs \dq verkn"upfungsname\dq
#       Name f"ur den Link. Erscheint in der \LaTeXe-Version als Text.
#       \\Defaultwert: {\tt url}
#
#   @optname alt
#   @optargs <alternativbilddatei>
#       Erscheint in der \LaTeXe-Version als alternative Darstellung. 
#
#   @optname (no)center
#       Siehe Seite \pageref{cmd:IMAGE:(no)center} ({\tt \#IMAGE}-Optionen). 
#
#   @optname (no)figure
#       Siehe Seite \pageref{cmd:IMAGE:(no)figure} ({\tt \#IMAGE}-Optionen). 
#
#   @optname height
#   @optargs <bildh"ohe>
#       Siehe Seite \pageref{cmd:IMAGE:height} ({\tt \#IMAGE}-Optionen). 
#
#   @optname hheight
#   @optargs <html-h"ohe>
#       Siehe Seite \pageref{cmd:IMAGE:hheight} ({\tt \#IMAGE}-Optionen). 
#
#   @optname hsign
#   @optargs \dq html-bildunterschrift\dq
#       Bildunterschrift f"ur die HTML-Version.
#
#   @optname htmlonly
#       Link erscheint nur in der HTML-Version. 
#
#   @optname hwidth
#   @optargs <html-breite>
#       Siehe Seite \pageref{cmd:IMAGE:hwidth} ({\tt \#IMAGE}-Optionen). 
#
#   @optname imglink
#       Die {\tt alternativbilddatei} wird in der HTML-Version mit der Adresse {\tt htmladresse}
#       verkn"upft.  
#
#   @optname label
#   @optargs <labelbezeichner>
#       Siehe Seite \pageref{cmd:IMAGE:label} ({\tt \#IMAGE}-Optionen). 
#
#   @optname nobox
#       Dieser Schalter unterdr"uckt die Umrahmung der Source-Datei in der 
#       \LaTeXe-Version. 
#
#   @optname nolocal
#       Bei diesem Schalter bleibt die Verkn"upfung auch bei der Kommandozeilenoption {\tt -htmllocal}
#       erhalten.
#
#   @optname scale
#   @optargs <qualit"atsfaktor>
#       Siehe Seite \pageref{cmd:IMAGE:scale} ({\tt \#IMAGE}-Optionen). 
#
#   @optname sign
#   @optargs \dq bildunterschrift\dq
#       Siehe Seite \pageref{cmd:IMAGE:sign} ({\tt \#IMAGE}-Optionen). 
#
#   @optname source
#   @optargs <source-datei>
#       Die HTML-Quelle {\tt source-datei} anzeigen. In der HTML-Version wird ein zweiter Link mit dem 
#       Namen {\tt HTML-Source} generiert. Dieser Link zeigt auf die vom Master-Konverter
#       erzeugte Datei, die die HTML-Quelle {\tt source-datei} enth"alt. 
#       Falls {\tt source-datei} nicht angegeben wird, wird {\tt url} als solche Datei betrachtet.\\  
#       In der \LaTeXe-Version wird die Datei {\tt source-datei}, falls angegeben,
#       mit {\tt verbatiminput} eingebunden.
#
#   @optname (no)trans
#       Siehe Seite \pageref{cmd:IMAGE:(no)trans} ({\tt \#IMAGE}-Optionen). 
#
#   @optname width
#   @optargs <bildbreite>
#       Siehe Seite \pageref{cmd:IMAGE:width} ({\tt \#IMAGE}-Optionen). 
#
#
# @end

$HTMLLOCAL;        # wenn gesetzt, dann keine globale Links 
# HTMLLINK-Work
# HTMLLINK_INTERNAL = URL wird nicht mehr parsiert (kein ../)
sub htmllink_work {
  package htmllink;

  my ($wert,$flag);
  local $use_htmlonly;
  local ($use_target,$use_text,$use_alt_name,$use_imglink,$use_nolocal);
  local $use_sign, $use_hsign;
  local $use_nobox;
  local $use_center;
  local ($use_source,$use_source_flag);
  local $COMMAND = shift @_;
  local (@PARAMS) = @_;     # uebergebene Liste
  local (@TO_IMAGE);        # Liste zur Bildbearbeitung
  
  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}
  
  # Flags setzen
  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    #$wert =~ s/"//g;
    SWITCH: {
      if (($flag eq "target")||($flag eq "url")) {$use_target=$wert; last SWITCH;}
      if ($flag eq "text") {$use_text=$wert; last SWITCH;}
      if ($flag eq "alt") { $use_alt_name=$wert; push @TO_IMAGE,"file=$wert"; last SWITCH;}
      if ($flag eq "source") {$use_source=$wert; $use_source_flag=1; last SWITCH;}
      if ($flag eq "imglink") {$use_imglink=1; last SWITCH;}
      if ($flag eq "nolocal") {$use_nolocal=1; last SWITCH;}
      if ($flag eq "sign") { $use_sign=$wert; last SWITCH;}
      if ($flag eq "hsign") { $use_hsign=$wert; $hsign_flag=1; last SWITCH;}
      if ($flag eq "center") {$use_center=1; push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nocenter") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "trans") {push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "notrans") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "figure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nofigure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nospace") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "hwidth") { push @TO_IMAGE,"hwidth=$wert"; last SWITCH;}
      if ($flag eq "hheight") { push @TO_IMAGE,"hheight=$wert"; last SWITCH;}
      if ($flag eq "label") { push @TO_IMAGE,"label=$wert"; last SWITCH;}
      if ($flag eq "width") { push @TO_IMAGE,"width=$wert"; last SWITCH;}
      if ($flag eq "height") { push @TO_IMAGE,"height=$wert"; last SWITCH;}
      if ($flag eq "scale") { push @TO_IMAGE,"scale=$wert"; last SWITCH;}
      if ($flag eq "htmlonly") {$use_htmlonly=1; last SWITCH;}
      if ($flag eq "nobox") {$use_nobox=1; last SWITCH;}
      &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }

  # wenn ein Link nur fuer HTML elevant ist
  if (($main::OUTPUTFORMAT ne "html")&&($use_htmlonly)) { return; }
  
  # wenn es nur um das Bild geht
  if (($use_alt_name)&&($main::OUTPUTFORMAT eq "latex")) {
    push @TO_IMAGE,"sign=".$use_sign; 
    unshift @TO_IMAGE,"$COMMAND";
    &main::image_work(@TO_IMAGE);
    return;
  }
  
  &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : $use_target\n");

  # Htmllink aus dem Sicht des Master-Dokuments
  if ($COMMAND eq "HTMLLINK") {
    if (!($use_target =~ /\/\//)) {
      if (!($use_target =~ /^\//)) {
        $use_target="../".$use_target;
      }
    }
  }

  &to_flag_check;    # die Flags pruefen
  
  # Entsprechende Mechanismen aufrufen
  if ($main::OUTPUTFORMAT eq "html") {
    &html_work;
  } elsif ($main::OUTPUTFORMAT eq "latex") {
    &latex_work;
  } else {
    &pdf_work;
  }

  return;

  # die Flags pruefen
  sub to_flag_check {
    if ($use_source_flag) {
      if ($use_source) {
        unless (-f "$use_source") {
          &main::my_msg("HTML-Source-file $use_source not found.\n");
          $use_source="";
        }
      } else {
        my $location=main::cwd();
        chdir "$main::WORK_DIR/$main::LAST_HTML_DIR" 
          or &main::my_exit("Error by chdir (htmllink.pl): $main::WORK_DIR/$main::LAST_HTML_DIR\n");
        if (-f "$use_target") {
          $use_source=&main::get_abs_pfad($use_target)."/".
                &main::get_local_name($use_target);
        } else {
          if (($use_source_flag)&&($main::OUTPUTFORMAT eq "latex")) {
            &main::my_msg("File \'$use_target\' not found.\n");
          } 
        }
        chdir "$location"
          or &main::my_exit("Error by chdir (htmllink.pl): $location\n");
      }
    }
  }
  
  # prueft, ob es notwendig ist, Code von der HTML-Datei 
  # neu zu generieren
  sub ok_to_work {
    my $file = shift;
    my $noticy_key = &main::build_key("$main::OUTPUTFORMAT","$file");
    my @stat_liste = stat $file;
    my $mtime = $stat_liste[9];

    if ($main::MAKE_ALL) {return 1;}
    if ($main::NOT_NTCFILE) {return 0;}
    if (((exists $main::NOTICE_OLD{$noticy_key})
      and($main::NOTICE_OLD{$noticy_key} == $mtime))
      or (exists $main::NOTICE_NEW{$noticy_key})) {return 0;} 
    return 1;
  }

  # Diese Subroutine konvertiert Master-Kode in HTML-Kode        #
  sub html_work {
    my $file;
    my ($font_begin,$font_end); 
  
    # Falls '-copy'-Schalter, dann kopiere den Verzeichnis mit
    # dem html-Link lokal in das Outputverzeichnis
    if (($main::COPY_SOURCE)&&($main::HTMLLINK_COPY)) {
      $use_target = &copy_of_htmltree;
    }
  
    if ($main::HTML_SIGN_FONT) {
      $font_begin = "<$main::HTML_SIGN_FONT>";
      $font_end = "<\/$main::HTML_SIGN_FONT>";
    }
    
    # wenn kein Text, dann zeige die Adresse
    unless ($use_text) {
      $use_text = "{\\tt ".&main::link_to_latex($use_target)."}";
    }
  
    &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer HTML ***\n")
        if ($COMMAND eq "HTMLLINK");

    if (($use_nolocal)||(!($main::HTMLLOCAL))) {
      if (($use_imglink)&&($use_alt_name)) {
        &main::write_to_texfile(
          "\\begin{rawhtml}".
          "<A HREF=\"$use_target\">".
          "\\end{rawhtml}"
        );
        push @TO_IMAGE,"NO_BOT_SPACE"; 
        unshift @TO_IMAGE,"$COMMAND";
        &main::image_work(@TO_IMAGE);
        &main::write_to_texfile(
          "\\begin{rawhtml}".
          "<\/A>".
          "$font_begin".
          "\\end{rawhtml}\n"
        );
        &main::write_to_texfile("\\begin{center}\n");
        if ($hsign_flag) {
          &main::write_to_texfile($use_hsign."\n");
        } else {
          &main::write_to_texfile($use_sign."\n");
        }
        &main::write_to_texfile("\\end{center}\n");
        &main::write_to_texfile(
          "\\begin{rawhtml}".
          "$font_end".
          "\\end{rawhtml}"
        );
      } else {
        if ($use_center) { &main::write_to_texfile("\\begin{center}\n"); }
        &main::write_to_texfile(
          "\\begin{rawhtml}".
          "<A HREF=\"$use_target\">".
          "\\end{rawhtml}"
        );
        &main::write_to_texfile($use_text);
        &main::write_to_texfile(
          "\\begin{rawhtml}".
          "<\/A>".
          "\\end{rawhtml}"
        );
        if ($use_center) { &main::write_to_texfile("\\end{center}\n"); }
      }
      if ($use_source) {
        $file=&generate_html_of_source($use_source);
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t \&nbsp;\&nbsp;\&nbsp;\&nbsp;(<A HREF=\"$file\">HTML-Source<\/A>)\n".
          "\\end{rawhtml}\n"
        );
          
      }
    } else {
      if (($use_imglink)&&($use_alt_name)) {
        if ($hsign_flag) { push @TO_IMAGE,"sign=$use_hsign"; }
        else { push @TO_IMAGE,"sign=$use_sign"; } 
        unshift @TO_IMAGE,"$COMMAND";
        &main::image_work(@TO_IMAGE);
      } else {
        $use_text=&main::link_to_latex($use_target);
        &main::write_to_texfile($use_text);
      }
    }
    &main::write_to_texfile("% END : $COMMAND fuer HTML\n")
        if ($COMMAND eq "HTMLLINK");
  }
  
  # Generiert HTML-Datei mit Code von VRML-Datei
  sub generate_html_of_source {
    my ($file) = shift;          #Dateiname als Argument
    my $local_name=&main::get_base_name($file);
    my $type=&main::get_file_type($file);
    my $TARGET="Sources";
    my $code_file;
    my $line;
    my $noticy_key = &main::build_key("$main::OUTPUTFORMAT","$file");
    my @stat_liste = stat $file;
    my $mtime = $stat_liste[9];
    my $htmlout;
    
    unless (-f "$file") {
      &main::my_msg("Generate of \'codelink\' failed.");
      return;
    }
    unless (-d "$main::WORK_DIR/$main::OUTPUT_DIR/$TARGET") {
      &main::get_underdir("$TARGET");
    }
  
    $code_file="$TARGET/code-$local_name.$type.html";
    if ((&ok_to_work($file))||(!(-f "$main::OUTPUT_DIR/$code_file"))) {
      &main::my_print("Generate of $main::OUTPUT_DIR/$code_file ...\n") if ($main::DEBUG);

      $htmlout = &main::txt_to_html( $file, 1 );

      open (HTML, "> $main::WORK_DIR/$main::OUTPUT_DIR/$code_file")
          or &main::my_exit("Error by open : $main::WORK_DIR/$main::OUTPUT_DIR/$code_file");
      print HTML $htmlout;
      close HTML;

      # Aenderungen haben stattgefunden
      $SOURCE_IS_CHANGE=1;
    }
  
    # merke die Datei in ntc-File
    $main::NOTICE_NEW{$noticy_key}="$mtime";
  
    return $code_file;
  }
  
  # Kopiere den HTML-Baum falls '-copy'-Schalter und HTML-Link ist lokal
  sub copy_of_htmltree {
    my $location=main::cwd();
    my $DIR_TARGET = "Html";
    my $NEW_TARGET = "$use_target";
    my ($dir_name,$basename);
    my ($dir_key,$mtime);
    my @stat_liste;
  
  
    chdir "$main::WORK_DIR/$main::LAST_HTML_DIR" 
      or &main::my_exit("Error by chdir (htmllink.pl): $main::WORK_DIR/$main::LAST_HTML_DIR\n");
        
    if ((-f "$use_target")||(-d "$use_target")) {
      if (-f "$use_target") {
        $basename = &main::get_base_name($NEW_TARGET).".".&main::get_file_type($NEW_TARGET);
        $NEW_TARGET =~ s/\/$basename$//;
      } else {
        $basename="";
        $NEW_TARGET =~ s/\/$//;
      }
      $NEW_TARGET =~ /(.*)\/(.*)$/;
      $dir_name = $2;
              
      @stat_liste = stat $NEW_TARGET;
      $mtime = $stat_liste[9];
      $dir_key = &main::build_key("$main::OUTPUTFORMAT","$NEW_TARGET");
  
      &main::get_underdir("$DIR_TARGET");
  
      if ((!(-d "$DIR_TARGET/$dir_name"))||(&ok_to_work($NEW_TARGET))) {
        unless (&main::my_system("cp -r $NEW_TARGET $DIR_TARGET")) {
    
          $NEW_TARGET = "$DIR_TARGET/$dir_name/$basename";
          &main::my_print("Can not copy of $NEW_TARGET.\n");
              
          # merke die Datei in ntc-File
          $main::NOTICE_NEW{$dir_key}="$mtime";
  
        } else {
          &main::my_msg("Can not copy of $NEW_TARGET.\n");
          $NEW_TARGET = "$use_target";
        }
      } else {
        if ((-f "$DIR_TARGET/$dir_name/$basename")||(-d "$DIR_TARGET/$dir_name")) {
          # merke die Datei in ntc-File
          $main::NOTICE_NEW{$dir_key}="$mtime";
  
          $NEW_TARGET = "$DIR_TARGET/$dir_name/$basename";
        }
      }  
    } else {
      unless ($use_target =~ /\/\//) {
        &main::my_msg("Local HTML-Link \'$use_target\' not found.\n");
      } 
    }
   
    chdir "$location" 
      or &main::my_exit("Error by chdir (htmllink.pl): $location\n");
  
    # Nachfolgende Zeile wegen // im Dateinamen
    $NEW_TARGET =~ s/\/\//\//;
  
    return $NEW_TARGET;
  }
  
  # Diese Subroutine konvertiert Master-Kode in Latex-Kode        #
  # Statt eines Links wird das alternative Text oder Bild dargestellt   #
  sub latex_work {
    # wenn kein Text, dann erscheint die HTML-Adresse
    # ersetze die Sonderzeichen wie _,~ ... auf die entsprechenden LaTeX
    # Symbole
  
    &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer LaTeX ***\n")
        if ($COMMAND eq "HTMLLINK");

    if ($use_source) {
      &main::write_to_texfile(
        "\n".
        "\\fbox{\\parbox{16cm}{\n"
      ) unless ($use_nobox);
      &main::write_to_texfile("{\\footnotesize \\verbatiminput{$use_source}}\n");
      unless ($use_nobox) {
       &main::write_to_texfile("}}\n");
       &main::write_to_texfile("\\begin{center}{\\it $use_text}\\end{center}\n") if ($use_text);
       &main::write_to_texfile("\\vspace{3mm}\n\n");
      }
    } else {
      unless ($use_text) {
        $use_text=&main::link_to_latex($use_target);
        $use_text = "{\\tt ".$use_text."}";
      }
      &main::write_to_texfile($use_text);
    } 
    &main::write_to_texfile("% END : $COMMAND fuer LaTeX\n")
        if ($COMMAND eq "HTMLLINK");
  }
  
  # Diese Subroutine konvertiert Master-Kode in PDF-Kode        #
  sub pdf_work {
    my $file;
  
    # wenn kein Text, dann zeige die Adresse
    unless ($use_text) {
      $use_text=&main::link_to_latex($use_target);
      $use_text = "{\\tt ".$use_text."}";
    }
  
    if ($use_source) {
      &main::write_to_texfile(
        "\n".
        "\\fbox{\\parbox{16cm}{\n"
      ) unless ($use_nobox);
      &main::write_to_texfile("{\\footnotesize \\verbatiminput{$use_source}}\n");
      unless ($use_nobox) {
       &main::write_to_texfile("}}\n");
       &main::write_to_texfile("\\begin{center}{\\it $use_text}\\end{center}\n") if ($use_text);
       &main::write_to_texfile("\\vspace{3mm}\n\n");
      }
    } else {
      &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer PDF ***\n")
          if ($COMMAND eq "HTMLLINK");

      if (($use_imglink)&&($use_alt_name)) {
        if ($use_center) { &main::write_to_texfile("\\begin{center}\n"); }
        &main::write_to_texfile("\\href{$use_target}{");

        # IMAGE darf kein center ausfhren:
        my (@tmp) = ();
        foreach $i (@TO_IMAGE) {
            next if ($i eq "center");
            push @tmp, $i;
        }
        printf "TO_IMAGE = @TO_IMAGE\n";
        printf "tmp = @tmp\n";
        push @tmp,"NO_BOT_SPACE"; 
        push @tmp,"nocenter"; 
        push @tmp,"nofigure"; 
        unshift @tmp,"$COMMAND";
        &main::image_work(@tmp);
        &main::write_to_texfile("}");
        if ($use_center) { &main::write_to_texfile("\\end{center}\n"); }
        &main::write_to_texfile("\\begin{center}\n");
        if ($hsign_flag) {
          &main::write_to_texfile($use_hsign."\n");
        } else {
          &main::write_to_texfile($use_sign."\n");
        }
        &main::write_to_texfile("\\end{center}\n");
      } else {
        if ($use_center) { &main::write_to_texfile("\\begin{center}\n"); }
        &main::write_to_texfile("\\href{$use_target}{$use_text}");
        if ($use_center) { &main::write_to_texfile("\\end{center}\n"); }
      }
      &main::write_to_texfile("% END : $COMMAND fuer PDF\n")
          if ($COMMAND eq "HTMLLINK");
    }
  }
}

# Alle Funktionen dieses Macros bekannt machen
$MANAGEFUNKTION->{HTMLLINK} = \&htmllink_work;

1;
