#
# @command IMAGE2
# @title   Einbinden von Doppelbildern
# @index
# @syntax  \#IMAGE2\{optionen\}
# @desc Der Befehl {\tt \#IMAGE2} bietet wie der Befehl {\tt \#IMAGE}
#       die M"oglichkeit, Bilder und Grafiken in Dokumente zu
#       integrieren. Der Befehl {\tt \#IMAGE2} kann allerdings zwei
#       Grafiken nebeneinander stellen. Die Optionen sind analog zum
#       Befehl {\tt \#IMAGE}, nur dass jede Option zweimal vorhanden
#       ist (1 entspricht dem linken, 2 dem rechten Bild).
# @example_begin
#  #IMAGE2{ 
#      file1=Ps/buch.ps,             % linkes Bild
#      trans1,                       % Transparenz (nur HTML-Version)
#      width1=7cm,                   % LaTeX-Breite
#      sign1="Papier-Version",       % linke Unterschrift
#      file2=Gif/html.gif,           % rechtes Bild
#      width2=7cm,                   % LaTeX-Breite
#      sign2="HTML-Version",         % rechte Unterschrift
#      figure,                       % beide Bilder als Gleitobjekt
#      sign="Master-Dokument"        % gemeinsame Unterschrift
#  }
# @example_end
#
#   @optname file1
#   @optargs <bilddatei1> 
#      Linkes Bild. Eine notwendige Option. {\tt bilddatei1} mu"s den gesamten
#      (relativen oder absoluten) Pfad enthalten. 
#
#   @optname file2
#   @optargs <bilddatei2> 
#      Rechtes Bild. Eine notwendige Option. {\tt bilddatei2} mu"s den gesamten
#      (relativen oder absoluten) Pfad enthalten. 
#
#   @optname alt1 / alt2
#   @optargs <alternativbilddatei> 
#      Mit diesem Schalter kann das alternative Bild in die \LaTeXe-Version eingef"ugt werden.
#      Wird bei der alternativen Darstellung einigen Bildformaten (JPEG,PNG) in der
#      \LaTeXe-Version benutzt. 
#
#   @optname angle1 / angle2
#   @optargs <winkel> 
#      Mit dieser Option wird der Drehwinkel in der \LaTeXe-Version bestimmt (in Graden
#      gegen den Uhrzeigersinn). Dieser Parameter wird f"ur die FigTeX-Bilder ignoriert. 
#
#   @optname factor1 / faktor 2
#   @optargs <skalierungsfaktor> 
#      Skalierungsfaktor des FigTeX-Bildes in \LaTeXe-Version. Erlaubt sind die reellen
#      Werte gr"o"ser 0. Bei diesem Faktor werden die Optionen width und height ignoriert.
#      \\Beispiel: {\tt faktor=0.5}
#      \\Defaultwert: {\tt 1}
#
#   @optname figtex1 / figtex2
#      Es handelt sich um ein FigTeX-Bild. 
#
#   @optname (no)figure 
#      Bilder (nicht) als Gleitobjekt setzen. 
#
#   @optname gnuplot1 / gnuplot2
#      Es handelt sich um eine Gnuplot-Grafik (falls Dateierweiterung nicht .plt). 
#
#   @optname hangle1 / hangle2
#   @optargs <winkel> 
#      Mit dieser Option wird der Drehwinkel in der HTML-Version drehwinkel bestimmt
#      (in Graden gegen den Uhrzeigersinn). Erlaubte Werte sind 90, 180 und 270.
#      Diese Option hat keinen Einflu"s auf die GIF-Bilder. 
#
#   @optname height1 / height2
#   @optargs <h"ohe> 
#      H"ohe des Bildes in \LaTeXe-Version. 
#      \\Beispiel: {\tt height=12cm}
#      \\Defaultwert: Originalh"ohe. 
#
#   @optname hheight1 / hheight2
#   @optargs <html-h"ohe> 
#      H"ohe des Bildes in HTML-Version (in Pixel). 
#      \\Beispiel: {\tt height=300}
#      \\Defaultwert: Originalh"ohe. 
#
#   @optname htmlonly
#      Bilder werden nur in HTML-Version sichtbar. 
#
#   @optname hwidth1 / hwidth2
#   @optargs <html-breite> 
#      Breite des Bildes in HTML-Version (in Pixel). 
#      \\Beispiel: {\tt hwidth=230}
#      \\Defaultwert: Originalbreite. 
#
#   @optname image1 / image2
#   @optargs <bildnummer> 
#      Bild mit der Nummer bildnummer aus einem Animated-GIF-Bild in
#      \LaTeXe-Version einf"ugen. 
#      \\Defaultwert: {\tt 1}
#
#   @optname latexonly 
#      Bild wird nur in \LaTeXe-Version sichtbar. 
#
#   @optname label
#   @optargs <labelbezeichner> 
#      Doppelbild wird im Dokument mit labelbezeichner markiert. Mit dem Master-Befehl
#      #IMAGEREF{<labelbezeichner>} kann man auf das Label eine Referenz anlegen. 
#
#   @optname scale1 / scale2
#   @optargs <qualit"atsfaktor> 
#      Qualit"atsfaktor f"ur die generierten GIF-Bilder. Es sind die reellen
#      Werte zwischen 1 und 4 erlaubt. 
#      \\Defaultwert: {\tt 1}
#
#   @optname sign
#   @optargs \dq bildunterschrift\dq
#      Gemeinsame Bildunterschrift angeben. Alle \LaTeXe-Befehle sind erlaubt. Die
#      Sonderzeichen ($\backslash$, \dq , , , \$, =, \{, \}) m"ussen mit $\backslash$
#      (backslash) gesch"utzt werden. 
#
#   @optname sign1 / sign2
#   @optargs \dq bildunterschrift\dq
#      Bildunterschrift f"ur einzelnes Bild, ansonsten analog zu {\tt sign}.
#
#   @optname (no)trans1 / (no)trans2
#      Bild wird in HTML-Version (nicht) transparent. 
#
#   @optname width1 / width2
#   @optargs <breite> 
#      Breite des Bildes in \LaTeXe-Version. 
#      \\Beispiel: {\tt width=9cm}
#      \\Defaultwert: Originalbreite. 
# @end

# Benoetigt image-Modul fuer die einzelne Bilder
sub image2_work {
  package image2;

  my ($wert,$flag);
  local ($use_lname,$use_rname);
  local ($use_alt_lname,$use_alt_rname);
  local $use_figure = $main::USE_FIGURE;
  local $figure_options = $main::FIGURE_OPTIONS;
  local ($use_lsign,$use_rsign,$use_sign);
  local ($use_lwidth);
  local ($use_rwidth);
  local ($use_label);
  my $report_state = $main::REPORT_WRITE; # merkt altes Report-Zustand
                      # wird in die NTC-Datei eingetragen
  local $COMMAND = shift @_;
  local (@PARAMS) = @_;      # uebergebene Liste
  local @TO_LIMAGE;
  local @TO_RIMAGE;

  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}

  # Flags setzen
  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    SWITCH: {
      if ($flag eq "file1") {$use_lname=$wert; push @TO_LIMAGE, "file=$wert"; last SWITCH;}
      if ($flag eq "file2") {$use_rname=$wert; push @TO_RIMAGE, "file=$wert"; last SWITCH;}
      if ($flag eq "alt1") { $use_alt_lname=$wert; push @TO_LIMAGE, "alt=$wert"; last SWITCH; }
      if ($flag eq "alt2") { $use_alt_rname=$wert; push @TO_RIMAGE, "alt=$wert"; last SWITCH; }
      if ($flag eq "sign1") {$use_lsign=$wert; last SWITCH;}
      if ($flag eq "sign2") {$use_rsign=$wert; last SWITCH;}
      if ($flag eq "sign") {$use_sign=$wert; last SWITCH;}
      if ($flag eq "figure") {$use_figure="1"; last SWITCH;}
      if ($flag eq "nofigure") {$use_figure=""; last SWITCH;}
      if ($flag eq "label") {$use_label=$wert; $use_figure="1"; last SWITCH;}
      if ($flag eq "trans1") { push @TO_LIMAGE, "trans"; last SWITCH;}
      if ($flag eq "trans2") { push @TO_RIMAGE, "trans"; last SWITCH;}
      if ($flag eq "notrans1") { push @TO_LIMAGE, "notrans"; last SWITCH;}
      if ($flag eq "notrans2") { push @TO_RIMAGE, "notrans"; last SWITCH;}
      if ($flag eq "figtex1") { push @TO_LIMAGE, "figtex"; last SWITCH;}
      if ($flag eq "figtex2") { push @TO_RIMAGE, "figtex"; last SWITCH;}
      if ($flag eq "gnuplot1") { push @TO_LIMAGE, "gnuplot"; last SWITCH;}
      if ($flag eq "gnuplot2") { push @TO_RIMAGE, "gnuplot"; last SWITCH;}
      if ($flag eq "width1") {$use_lwidth=$wert; push @TO_LIMAGE, "width=$wert"; last SWITCH;}
      if ($flag eq "width2") {$use_rwidth=$wert; push @TO_RIMAGE, "width=$wert"; last SWITCH;}
      if ($flag eq "height1") { push @TO_LIMAGE, "height=$wert"; last SWITCH;}
      if ($flag eq "height2") { push @TO_RIMAGE, "height=$wert"; last SWITCH;}
      if ($flag eq "hwidth1") { push @TO_LIMAGE, "hwidth=$wert"; last SWITCH;}
      if ($flag eq "hwidth2") { push @TO_RIMAGE, "hwidth=$wert"; last SWITCH;}
      if ($flag eq "hheight1") { push @TO_LIMAGE, "hheight=$wert"; last SWITCH;}
      if ($flag eq "hheight2") { push @TO_RIMAGE, "hheight=$wert"; last SWITCH;}
      if ($flag eq "angle1") { push @TO_LIMAGE, "angle=$wert"; last SWITCH;}
      if ($flag eq "angle2") { push @TO_RIMAGE, "angle=$wert"; last SWITCH;}
      if ($flag eq "hangle1") { push @TO_LIMAGE, "hangle=$wert"; last SWITCH;}
      if ($flag eq "hangle2") { push @TO_RIMAGE, "hangle=$wert"; last SWITCH;}
      if ($flag eq "image1") { push @TO_LIMAGE, "image=$wert"; last SWITCH;}
      if ($flag eq "image2") { push @TO_RIMAGE, "image=$wert"; last SWITCH;}
      if ($flag eq "scale1") { push @TO_LIMAGE, "scale=$wert"; last SWITCH;}
      if ($flag eq "scale2") { push @TO_RIMAGE, "scale=$wert"; last SWITCH;}
      if ($flag eq "factor1") { push @TO_LIMAGE, "factor=$wert"; last SWITCH;}
      if ($flag eq "factor2") { push @TO_RIMAGE, "factor=$wert"; last SWITCH;}
      if ($flag eq "latexonly") { return if ($main::OUTPUTFORMAT eq "html"); 
                    last SWITCH;
                    }
      if ($flag eq "htmlonly") { return if ($main::OUTPUTFORMAT eq "latex"); 
                    last SWITCH;
                    }
      &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }

  # alternatives Bild in LaTeX
  if ($main::OUTPUTFORMAT eq "latex") {
    if ($use_alt_lname) { $use_lname = $use_alt_lname; }
    if ($use_alt_rname) { $use_rname = $use_alt_rname; }
  }

  &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : file1=$use_lname, file2=$use_rname\n");

  &to_flag_check;    # Syntax pruefen und wenn noetig die
            # default-Werte setzen

  &build_bild_list;  # Die IMAGE-Liste vervollstaendigen   
  
  # Schreiben in die Report-Version erlauben
  $main::REPORT_WRITE = 1;
  # Entsprechende Mechanismen aufrufen
  if ($main::OUTPUTFORMAT eq "html") {&html_work;}
  else {&latex_work;}
  # Schreiben in die Report-Version zuruecksetzen
  $main::REPORT_WRITE = $report_state;

  return;

  # die Flags ueberpruefen
  sub to_flag_check {
    unless ($use_lname) {&main::my_exit("File1 is not statement.\n");}
    unless ($use_rname) {&main::my_exit("File2 is not statement.\n");}
    $figure_options="htb" unless ($figure_options); 
  }
  
  # Diese Subroutine vervollstaendigt die Liste, die nach IMAGE-Work geschickt wird
  sub build_bild_list {
    unshift @TO_LIMAGE,"IMAGE2=1";
    unshift @TO_RIMAGE,"IMAGE2=2";
    unshift @TO_LIMAGE,"NO_BOT_SPACE";
    unshift @TO_RIMAGE,"NO_BOT_SPACE";
    unshift @TO_LIMAGE,"nocenter";
    unshift @TO_RIMAGE,"nocenter";
    unshift @TO_LIMAGE,"nofigure";
    unshift @TO_RIMAGE,"nofigure";
    unshift @TO_LIMAGE,"$COMMAND";
    unshift @TO_RIMAGE,"$COMMAND";
  }
  
  # Diese Subroutine konvertiert Master-Kode in HTML-Kode        #
  # Die Grafiken werden dabei automatisch ins GIF-Format umgewandelt  #
  sub html_work {
    my $label;
    my ($font_begin,$font_end); 
  
    if ($main::HTML_SIGN_FONT) {
      $font_begin = "<$main::HTML_SIGN_FONT>";
      $font_end = "<\/$main::HTML_SIGN_FONT>";
    }
    if ($use_label) { $label="\\label{$use_label}"; }
  
    &main::write_to_texfile(
      "% *** Master-Generator - $COMMAND fuer HTML ***\n".
      "% BEGIN : $use_lname, $use_rname.$use_rtype\n");
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\t<CENTER>\n".
      "\\end{rawhtml}\n");
  
    # Dieses Teil braucht man fuer die Zaehlung von
    # Abbildungen in der HTML-Version.
    # Nur dann kann man die Bilder in der HTML-Version referenzieren.
    &main::write_to_texfile(
      "\\begin{latexonly}\n".
      "\\begin{figure}[$figure_options]\n") if ($use_figure);
    &main::write_to_texfile("\t\\caption{\\label{$use_label}}\n") if ($use_label);
    &main::write_to_texfile(
      "\\end{figure}\n".
      "\\end{latexonly}\n") if ($use_figure);
  
    &main::write_to_texfile("\\begin{rawhtml}\n");
    &main::write_to_texfile("\t<A NAME=$use_label>\n") if ($use_label);
    &main::write_to_texfile(
      "\t<TABLE BORDER=0 CELLSPACING=5 CELLPADDING=5>\n".
      "\t<TR>\n".
      "\t<TD>\n".
      "\\end{rawhtml}\n"
    );
    &main::image_work(@TO_LIMAGE);
    if ($use_lsign) {
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<CENTER>$font_begin\n".
        "\\end{rawhtml}\n".
        $use_lsign."\n".
        "\\begin{rawhtml}\n".
        "\t$font_end<\/CENTER>\n".
        "\\end{rawhtml}\n");
    } else {
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<BR>\n".
        "\\end{rawhtml}\n");
    }
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t<\/TD>\n".
      "\t<TD>\n".
      "\\end{rawhtml}\n"
    );
    &main::image_work(@TO_RIMAGE);
    if ($use_rsign) {
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<CENTER>$font_begin\n".
        "\\end{rawhtml}\n".
        $use_rsign."\n".
        "\\begin{rawhtml}\n".
        "\t$font_end<\/CENTER>\n".
        "\\end{rawhtml}\n");
    } else {
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<BR>\n".
        "\\end{rawhtml}\n");
    }
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t<\/TD>\n".
      "\t<CAPTION ALIGN=BOTTOM>$font_begin\n".
      "\\end{rawhtml}\n"
    );
    &main::write_to_texfile($use_sign."\n") if ($use_sign);
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t$font_end<\/CAPTION>\n".
      "\t<\/TABLE>\n"
    );
    &main::write_to_texfile("<A\/>\n") if ($use_label);
    &main::write_to_texfile(
      "\t<\/CENTER>\n".
      "\\end{rawhtml}\n"
    );
    &main::write_to_texfile("\\label{$use_label}\n") if ($use_label);
    &main::write_to_texfile("% END : $use_lname und $use_rname fuer HTML\n");
  }
  
  
  # Diese Subroutine konvertiert Master-Kode in Latex-Kode        #
  # Die Grafiken werden dabei automatisch ins PS-Format umgewandelt    #
  # Ausnahme : die FigTeX-Bilder
  sub latex_work {
    my ($label);
    my $pwidth;
    if ($use_label) {
      $label="\\label{$use_label}"; 
    }
    &main::write_to_texfile(
      "% *** Master-Generator - IMAGE2 fuer LaTeX ***\n".
      "% BEGIN : $use_lname, $use_rname\n"
    );
    &main::write_to_texfile("\\begin{figure}[$figure_options]\n") if ($use_figure);
    &main::write_to_texfile("\\begin{center}\n");
    if ($use_lwidth) {$pwidth=$use_lwidth;}
    else {$pwidth="6cm"};
  
    &main::write_to_texfile("\\parbox[b]{$pwidth}{\n");
    &main::write_to_texfile("\t\\begin{center}\n");
    &main::image_work(@TO_LIMAGE);
    if ($use_lsign) {
      &main::write_to_texfile("\t\\begin{center}\n") unless ($use_rsign);
      &main::write_to_texfile("\n\t{\\small ".$use_lsign."}\n");
      &main::write_to_texfile("\t\\end{center}\n") unless ($use_rsign);
    } else {
      &main::write_to_texfile("\t\\begin{center}\n\n\\ ");
      &main::write_to_texfile("\t\\end{center}\n");
    }
    &main::write_to_texfile("\t\\end{center}\n");
    &main::write_to_texfile("}\n");
  
    #&main::write_to_texfile("\\hfill\n");
    &main::write_to_texfile("\\hspace{0.4cm}\n");
  
    if ($use_rwidth) {$pwidth=$use_rwidth;}
    else {$pwidth="6cm"};
  
    &main::write_to_texfile("\\parbox[b]{$pwidth}{\n");
    &main::write_to_texfile("\t\\begin{center}\n");
    &main::image_work(@TO_RIMAGE);
    if ($use_rsign) {
      &main::write_to_texfile("\t\\begin{center}\n") unless ($use_lsign);
      &main::write_to_texfile("\n\t{\\small ".$use_rsign."}\n");
      &main::write_to_texfile("\t\\end{center}\n") unless ($use_lsign);
    } else {
      &main::write_to_texfile("\t\\begin{center}\n\n");
      &main::write_to_texfile("\t\\end{center}\n");
    }
    &main::write_to_texfile("\t\\end{center}\n");
    &main::write_to_texfile("}\n");
  
    if ($use_figure) {
      &main::write_to_texfile("\\caption{$label\\small ".$use_sign."}\n") if ($use_sign);
    } else {
      if ($use_sign) {
        &main::write_to_texfile(
          #"\\begin{center}\n".
          "\n\t{\\rm ".$use_sign."}\n"
          #"\\end{center}\n"
        );
      }
    }
    &main::write_to_texfile("\\end{center}\n");
    &main::write_to_texfile("\\end{figure}\n") if ($use_figure);
    &main::write_to_texfile("% END : $use_lname und $use_rname fuer LaTeX\n");
  }
  
}

$MANAGEFUNKTION->{IMAGE2} = \&image2_work;

1;
