# @command JAVA
# @title   Java-Applets
# @index   
# @syntax  \#JAVA\{optionen\}
# @desc    Der Master-Befehl {\tt \#JAVA} bietet die M"oglichkeit, Java-Applets in
#          das Dokument einzuf"ugen. Es ist auch m"oglich, die Dokumentation ({\tt javadoc}) und
#          den Programmcode einzubinden. Statt eines Applets kann in der \LaTeXe-Version
#          ein alternatives Bild dargestellt werden.\\
#          Der Befehl {\tt \#JAVA} hat im Gegensatz zu anderen Master-Befehlen zwei verschiedenen
#          Modi, die von der Dateierweiterung der Option {\tt file} abh"angen.
# @example_begin
#   #JAVA{
#       file=Applets/Live.class,     % class-Datei
#       embed,                       % eingebettet
#       hwidth=500,                  % HTML-Breite
#       hheight=280,                 % HTML-Hoehe
#       sign="Berechnung des Lebensverlaufs eines Menschen \\\\ 
#             ausgehend vom Geburtsdatum",
#       center,                      % zentriert
#       alt=Gif/live.gif             % alternatives Bild
#   }		
#
#   #JAVA{
#       file=Evo.java,                   % java-Datei
#       java_path=../Applets,            % Pfad mit der Java-Quelle
#       applet,                          % Link auf Applet setzen
#       applet_path=../Applets,          % Pfad mit dem Applet
#       applet_image=../icons/duke.gif   % Bild als Link auf Applet
#   }		
# @example_end
#
#   @optname file
#   @optargs <dateiname>.<class\textbar java>
#       \begin{enumerate}
#         \item Dateierweiterung ist {\tt class}. Die Datei wird als Applet in die HTML-Version eingebunden.
#               In der \LaTeXe-Version erscheint ein alternatives Bild.
#         \item Dateierweiterung ist {\tt java}. In die \LaTeXe-Version wird der Programmquelltext von
#               {\tt dateiname.java} eingebunden. In der HTML-Version erscheinen je nach weiteren Optionen die 
#               Links auf den Programmquelltext ({\tt dateiname.java}), die Dokumentation (HTML-Datei) und das Applet
#               (HTML-Datei). 
#       \end{enumerate} 
#
#   @optname alt
#   @optargs <alternativbilddatei>
#       Die Bilddatei wird in der \LaTeXe-Version eingef"ugt.
#       Die {\tt alternativbilddatei} mu"s den gesamten (relativen oder absoluten) Pfad enthalten. 
#       {\bf (nur im class-Modus)}
#
#   @optname (no)center
#       Siehe Seite \pageref{cmd:IMAGE:(no)center} ({\tt \#IMAGE}-Optionen). 
#       {\bf (nur im class-Modus)}
#
#   @optname codelink
#   @optargs \dq linkname\dq
#       Es wird ein Link mit dem Namen {\tt linkname} auf eine HTML-Datei,
#       die den Quellkode von dem Applet enth"alt, erzeugt. 
#       Diese HTML-Datei wird automatisch vom Master-Konverter generiert und hat das gleiche Layout wie
#       das ganze Dokument. Die Datei wird in dem Unterverzeichnis {\tt \$OUTPUT\_DIR/Sources} abgelegt.
#       Die Datei {\tt dateiname.java} mu"s dabei im gleichen Verzeichnis liegen wie {\tt dateiname.class}.
#       {\bf (nur im class-Modus)}
#       \\Defaultwert : {\tt Source} 
#
#   @optname (no)embed
#       Das Applet (nicht) einbetten. Bei der Option {\tt noembed} wird eine Verkn"upfung  
#       mit dem Namen {\tt hsign} auf die Datei erstellt. 
#       {\bf (nur im class-Modus)}
#
#   @optname (no)figure
#       Siehe Seite \pageref{cmd:IMAGE:(no)figure} ({\tt \#IMAGE}-Optionen). 
#       {\bf (nur im class-Modus)}
#
#   @optname height
#   @optargs <bildh"ohe>
#       Siehe Seite \pageref{cmd:IMAGE:height} ({\tt \#IMAGE}-Optionen). 
#       {\bf (nur im class-Modus)}
#
#   @optname hheight
#   @optargs <html-h"ohe>
#       HTML-H"ohe in Pixel oder Prozent.
#       {\bf (nur im class-Modus)}
#       \\Defaultwert: 80% 
#
#   @optname hsign
#   @optargs \dq html-unterschrift\dq
#       Unterschrift bei eingebettetem Applet, sonst Verkn"upfungsname.
#       Diese Option kann auch die HTML-Tags enthalten.
#       {\bf (nur im class-Modus)}
#       \\Beispiel: {\tt hsign= \dq Verkn\&uuml;pfungsname\dq}
#
#   @optname hwidth
#   @optargs <html-breite>
#       HTML-Breite in Pixel oder Prozent.
#       {\bf (nur im class-Modus)}
#       \\Defaultwert: 80% 
#
#   @optname imglink
#       Es erscheint in der HTML-Version ein sensitives Bild ({\tt alternativbilddatei}), das mit der 
#       Applet-Datei verkn"upft wird. Bei dieser Option
#       wird der {\tt embed}-Schalter ignoriert.
#       {\bf (nur im class-Modus)}
#
#   @optname import
#   @optargs \dq importliste\dq
#       In der Liste {\tt importliste} gibt man alle Dateien oder Verzeichnisse an, die
#       in die Applet-Klassendatei importiert werden. Mit der Kommandozeilenoption {\tt -copy}
#       werden diese Dateien lokal mitkopiert.
#       {\bf (nur im class-Modus)}
#
#   @optname label
#   @optargs <labelbezeichner>
#       Siehe Seite \pageref{cmd:IMAGE:label} ({\tt \#IMAGE}-Optionen). 
#       {\bf (nur im class-Modus)}
#
#   @optname sign
#   @optargs \dq bildunterschrift\dq
#       Siehe Seite \pageref{cmd:IMAGE:sign} ({\tt \#IMAGE}-Optionen). 
#       {\bf (nur im class-Modus)}
#
#   @optname width
#   @optargs <bildbreite>
#       Siehe Seite \pageref{cmd:IMAGE:width} ({\tt \#IMAGE}-Optionen). 
#       {\bf (nur im class-Modus)}
#
#   @optname applet
#   @optargs <applet-file>
#       Link auf die Applet-Datei setzen. Die Datei {\tt applet-file}
#       wird im {\tt java\_applet\_path}-Verzeichnis gesucht.
#       Falls die HTML-Datei {\tt applet-file} nicht angegeben ist, 
#       wird als Dokumentation die Datei {\tt dateiname.html}
#       aus dem {\tt java\_applet\_path}-Verzeichnis genommen.
#       {\bf (nur im java-Modus)}
#
#   @optname applet\_image
#   @optlabel applet_image
#   @optargs <bilddatei>
#       Die Datei {\tt bilddatei} erscheint als sensitives Bild mit Link auf die Applet-Datei. 
#       {\bf (nur im java-Modus)}
#
#   @optname applet\_path
#   @optlabel applet_path
#   @optargs <pfadname>
#       Pfad mit dem Applet (s. {\tt applet}-Option).
#       {\bf (nur im java-Modus)}
#
#   @optname doc
#   @optargs <doc-file>
#       Link auf die Dokumentation setzen. Die Datei {\tt doc-file} wird im {\tt java\_doc\_path}-Verzeichnis gesucht.
#       Ist die HTML-Datei {\tt doc-file} nicht angegeben, dann
#       wird als Dokumentation die {\tt dateiname.html}
#       aus dem {\tt java\_doc\_path}-Verzeichnis genommen.
#       {\bf (nur im java-Modus)}
#
#   @optname java\_doc\_path
#   @optlabel java_doc_path
#   @optargs <pfadname>
#       Pfad mit der Dokumentation (s. {\tt doc}-Option). 
#       {\bf (nur im java-Modus)}
#
#   @optname java\_path
#   @optlabel java_path
#   @optargs <pfadname>
#       Pfad mit dem Programmquelltext ({\tt dateiname.java}). 
#       {\bf (nur im java-Modus)}
#
#   @optname htmlonly
#       Der Befehl ist nur f"ur die HTML-Version relevant.
#       {\bf (nur im java-Modus)}
#
# @end

# JAVA-Modul
sub java_work {
  package java;

  my ($wert,$flag);
  local ($use_name,$use_alt_name,$use_sign,$use_imglink);
  local ($use_archive);
  local ($use_local_name,$use_type);
  local ($use_relativ_path);
  local ($use_dir);
  local ($use_hheight,$use_hwidth);
  local $use_codelink;
  local $use_import;
  local ($use_label);
  local ($noticy_key);
  local $status;  # gleich null, wenn die
                  # Konvertierung fehlerfrei
  local $use_param;
  local $use_param_list;
  local ($use_embed,$use_hsign);
  local $use_embed = $main::USE_APPLET_EMBED;
  local $use_center = $main::USE_CENTER;
  local $use_java_path = $main::USE_JAVA_PATH;
  local $use_java_doc_path = $main::USE_JAVADOC_PATH;
  local $use_java_doc_file;
  local $use_codebase;
  local $use_applet_path = $main::USE_APPLET_PATH;
  local $use_applet_file;
  local $use_applet_image = $main::USE_APPLET_IMAGE;
  local ($use_doc,$use_applet);
  local $use_name_value;
  local ($doc_file,$applet_file);
  local ($hsign_flag,$codelink_flag);
  local $use_htmlonly;
  local ($mtime);

  my (@stat_liste);
  my $file_name;          # Dateiname, der angegeben ist
                  # wird in die NTC-Datei eingetragen
  my $report_state = $main::REPORT_WRITE; # merkt altes Report-Zustand

  local $COMMAND=shift @_;
  local (@TO_IMAGE);        # Liste zur Bildbearbeitung
  local (@PARAMS) = @_;      # uebergebene Liste
  
  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}
  
  # Flags setzen
  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    #$wert =~ s/"//g;
    SWITCH: {
      if ($flag eq "file") {$use_name=$wert; $file_name=$wert; last SWITCH;}
      if ($flag eq "embed") {$use_embed="1"; last SWITCH;}
      if ($flag eq "imglink") {$use_imglink="1"; last SWITCH;}
      if ($flag eq "noembed") {$use_embed=""; last SWITCH;}
      if ($flag eq "hsign") { $use_hsign=$wert; $hsign_flag="1"; last SWITCH;}
      if ($flag eq "hwidth") { $use_hwidth=$wert; last SWITCH;}
      if ($flag eq "hheight") { $use_hheight=$wert; last SWITCH;}
      if ($flag eq "sign") { $use_sign=$wert; last SWITCH;}
      if ($flag eq "codelink") { $use_codelink=$wert; $codelink_flag=1; last SWITCH;}
      if ($flag eq "archive") { $use_archive=$wert; last SWITCH;}
      if ($flag eq "import") { $use_import=$wert; last SWITCH;}
      if ($flag eq "java_path") { $use_java_path=$wert; last SWITCH;}
      if ($flag eq "java_doc_path") { $use_java_doc_path=$wert; last SWITCH;}
      if ($flag eq "applet_path") { $use_applet_path=$wert; last SWITCH;}
      if ($flag eq "codebase") { $use_codebase=$wert; last SWITCH;}
      if ($flag eq "applet_image") { $use_applet_image=$wert; last SWITCH;}
      if ($flag eq "doc") { $use_doc=1; $use_java_doc_file=$wert; last SWITCH;}
      if ($flag eq "applet") { $use_applet=1; $use_applet_file=$wert; last SWITCH;}
      if ($flag eq "param") { $use_param=1; $use_param_list=$wert; last SWITCH;}

      if ($flag eq "alt") { $use_alt_name=$wert; push @TO_IMAGE,"file=$wert"; last SWITCH;}
      if ($flag eq "label") { $use_label=$wert; push @TO_IMAGE,"label=$wert"; last SWITCH;}
      if ($flag eq "center") { $use_center=1; push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nocenter") { $use_center=""; push @TO_IMAGE,"$flag"; last SWITCH;}

      if ($flag eq "figure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nofigure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nospace") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "width") { push @TO_IMAGE,"width=$wert"; last SWITCH;}
      if ($flag eq "height") { push @TO_IMAGE,"height=$wert"; last SWITCH;}
      if ($flag eq "htmlonly") { $use_htmlonly=1; last SWITCH;}

      &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }
  
  if (($main::OUTPUTFORMAT ne "html")&&($use_htmlonly)) {
    return;
  } 

  $use_java_path =~ s/\/$//;
  $use_java_doc_path =~ s/\/$//;
  $use_applet_path =~ s/\/$//;

  # Uebergebenen Wert merken
  if ($use_name =~ /\.java$/) {
    $use_name_value = $use_name;
    $use_name_value =~ s/\.java$//;
  }

  # Falls CODEBASE angegeben
  if ($use_codebase) {
    $use_codebase =~ s/\/$//;
    $use_name=$use_codebase."/".$use_name;
  }

  # Java-Pfad dem Namen voreinstellen, falls kein globaler Pfad
  unless ($use_name =~ /^\//) { 
    $use_name = "$use_java_path/$use_name" if ($use_name =~ /\.java$/);
  }

  
  if ($main::OUTPUTFORMAT eq "html") {
    &to_flag_check;    # Syntax pruefen und wenn noetig die
              # default-Werte setzen
  }

  $use_local_name = &main::get_base_name($use_name);  # Basisname
  $use_type = &main::get_file_type($use_name);    # Dateierweiterung
  &main::statistic_count($use_type);          # fuer Statistik
  # Relativen Pfad erkuendigen
  $use_relativ_path = "..\/".$use_name;
  $use_relativ_path =~ s/$use_local_name\.$use_type$//;

  # Absoluten Pfad erkuendigen
  if ($use_name =~ /\.class$/) {
    $use_dir=&main::get_abs_pfad($use_name);
    $use_name = "$use_dir/$use_local_name.$use_type";
  }

  # fuer Statistik
  &main::statistic_count("java");

  # Dokumentation- und Applet-Dateien setzen
  if ($main::OUTPUTFORMAT eq "html") {
    if ($use_doc) {
      if ($use_java_doc_file) {
        $doc_file="$use_java_doc_path/$use_java_doc_file";
      } else {
        $doc_file="$use_java_doc_path/$use_name_value.html";
      }
    }
    if ($use_applet) {
      if ($use_applet_file) {
        $applet_file="$use_applet_path/$use_applet_file";
      } else {
        $applet_file="$use_applet_path/$use_name_value.html";
      }
      #&main::my_msg("JavaApplet-file $applet_file not found.\n") unless (-f "$applet_file");
      #unless ($applet_file =~ /^\//) { $applet_file="../$applet_file"; }
    }
  }
  
  # wenn es nur um das Bild geht
  if (($main::OUTPUTFORMAT eq "latex") || ($main::OUTPUTFORMAT eq "pdf")) {
    if ($use_type eq "class") {
      if ($use_alt_name) {
        push @TO_IMAGE,"sign=$use_sign"; 
        unshift @TO_IMAGE,"$COMMAND";
        &main::image_work(@TO_IMAGE);
      } else { &main::my_print("$COMMAND : Not Alt-File\n"); }
    } else {
      &java_latex;
    }
    return;
  } else {
    if ($use_type eq "class") {
      &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : File=$file_name\n");
    } else {
      &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : File=$use_name\n");
    }
  }

  # den Notice-Schluessel bilden
  if ($use_type eq "class") {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name);
  } else {
    $noticy_key=&main::build_key($main::OUTPUTFORMAT,$use_name);
  }

  # Dateieigenschaften erkundigen (Dateigroesse und die letzte
  # Modifikationszeit)
  @stat_liste = stat $use_name;
  $mtime = $stat_liste[9];

  # Schreiben in die Report-Version erlauben
  $main::REPORT_WRITE = 1;
  # Entsprechende Mechanismen aufrufen
  &html_work;
  # Schreiben in die Report-Version zuruecksetzen
  $main::REPORT_WRITE = $report_state;
  
  # wenn alles geklappt, dann Notice-Eintrag machen
  if (!($status)) { #wenn erfolgreich dann eintragen
    if ($use_type eq "class") {
      $main::NOTICE_NEW{$noticy_key}="$mtime";
    } else {
      $main::NOTICE_NEW{$noticy_key}="<>";
    }
  }

  return;

  # Prueft Syntax und Existenz von Quellen vom Master-Befehl 
  sub to_flag_check {
    unless ($use_name) {
      &main::my_exit("Java-File is not statement.\n");
    }
    
    if (($use_name =~ /\.class$/)&&((!(-f "$use_name")))) {
      &main::my_msg("File '$use_name' not found.\n");
    }
  
    # Applet-Image
    if ($use_applet_image) {
      $use_applet_image =~ s/(\s+)//g;
      if (-f "$use_applet_image") {
        if (($main::COPY_SOURCE)||($main::LOCAL_ICONS)) {
          my $img_local=&main::get_base_name($use_applet_image).".".&main::get_file_type($use_applet_image);
          unless (-f "$main::WORK_DIR/$main::OUTPUT_DIR/$img_local") {
            &main::copy($use_applet_image,"$main::WORK_DIR/$main::OUTPUT_DIR");
          }
          $use_applet_image=$img_local;
        } else {
          unless ($use_applet_image =~ /^\//) {
            $use_applet_image = "../".&main::get_relativ_name($use_applet_image);
          }
        }
      } else {
        &main::my_msg("Applet-image $use_applet_image not found.\n");
      }
    }
  
    # Pruefe die HTML-Groessen
    $use_hwidth =~ s/(\s+)//g;
    if ($use_hwidth) { 
      unless ($use_hwidth =~ /^((\d+)|(\d+%))$/) {
        &main::my_msg("Not allowed value in $COMMAND for hwidth : $use_hwidth\n");
        $use_hwidth="80%";
      }
    } else {
      $use_hwidth="80%";
    }
    $use_hheight =~ s/(\s+)//g;
    if ($use_hheight) { 
      unless ($use_hheight =~ /^((\d+)|(\d+%))$/) {
        &main::my_msg("Not allowed value in $COMMAND for hheight : $use_hheight\n");
        $use_hheight="80%";
      }
    } else {
      $use_hheight="80%";
    }
  
    # Wenn keine HTML-Unterschrift, dann nehme die LaTeX-Unterschrift
    unless ($hsign_flag) { $use_hsign=$use_sign; }
    unless ($use_hsign) { $use_hsign=$use_name; }
  
  
    if (($use_imglink)&&($use_alt_name)) {
      &main::my_msg("Flags \'embed\' and \'imglink\' in $COMMAND. Ignore \'embed\'.\n");
      $use_embed="";
    }
    if (($codelink_flag)&&(!($use_codelink))) { $use_codelink = "Source"; }
  }
  
  # Prueft, ob die Konvertierungsroutine noetig ist
  sub ok_to_work {
    my $ntc_key;
    my $mod_time;
    
    $ntc_key="$noticy_key";
    $mod_time="$mtime";
  
    if ($main::MAKE_ALL) {return 1;}
    if ($main::NOT_NTCFILE) {return 0;}
    if (((exists $main::NOTICE_OLD{$ntc_key})
      and($main::NOTICE_OLD{$ntc_key} == $mod_time))
      or (exists $main::NOTICE_NEW{$ntc_key})) {return 0;} 
    return 1;
  }
  
  # Diese Subroutine konvertiert Master-Kode in HTML-Kode        #
  sub html_work {
    my ($codebase);
  
    SWITCH: {
      if ($use_type eq "class") {$codebase = &java_to_html; &applet_html($codebase); last SWITCH;}
      if ($use_type eq "java") {&java_html; last SWITCH;}
      &main::my_exit("Unkrown expansion '.$use_type'\n");
    }
  }
  
  # setzt eine Java-Applikation als Applet oder
  # als ein Link auf ein Applet
  sub applet_html {
    my ($codebase)=shift;
    my $width="WIDTH=$use_hwidth";
    my $height="HEIGHT=$use_hheight";
    my $archive;
    my ($font_begin,$font_end);
    my @myparams;
    my ($myvparam, $myparam, $myvalue); 
  
    if ($main::HTML_SIGN_FONT) {
      $font_begin = "<$main::HTML_SIGN_FONT>";
      $font_end = "<\/$main::HTML_SIGN_FONT>";
    }
  
    &main::write_to_texfile(
      "% *** Master-Generator - $COMMAND fuer HTML ***\n".
      "% BEGIN : $use_local_name.$use_type\n".
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\t<P>\n"
    );
    &main::write_to_texfile("<A NAME=$use_label>\n") if ($use_label);
    $use_hsign = &main::sign_to_html($use_hsign);
    if ($use_embed) {
      if ($use_archive) { $archive="ARCHIVE=\"".$use_archive."\""; }
      &main::write_to_texfile(
        "\t<CENTER>\n".
        "\t<APPLET CODEBASE=\"$codebase\" CODE=\"$use_local_name\" $width $height $archive>\n"
      );  
      if ($use_param) {
          @myparams = split /;/, $use_param_list;
          foreach $myvparam (@myparams) {
              ($myparam, $myvalue) = split /:/, $myvparam, 2;
              &main::write_to_texfile("\t\t<PARAM NAME=$myparam VALUE=$myvalue>\n");
          }    
      }    
      &main::write_to_texfile(
        "\t<\/APPLET><BR>\n".
        "\t$font_begin".$use_hsign."$font_end\n".
        "\t<\/CENTER>\n"
      );
      &main::write_to_texfile("\\end{rawhtml}\n");
    } elsif (($use_imglink)&&($use_alt_name)) {
      &main::write_to_texfile(
        "\t<A HREF = \"Sources/applet-$use_local_name-$use_type.html\">\n".
        "\\end{rawhtml}\n"
      );
      push @TO_IMAGE,"NO_BOT_SPACE"; 
      unshift @TO_IMAGE,"$COMMAND";
      &main::image_work(@TO_IMAGE);
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<\/A>\n".
        "\t$font_begin\n".
        "\\end{rawhtml}\n"
      );
      &main::write_to_texfile("\\begin{center}\n") if ($use_center);
        if ($hsign_flag) {
          &main::write_to_texfile("$use_hsign\n");
        } else {
          &main::write_to_texfile("$use_sign\n");
        }
        &main::write_to_texfile("\\end{center}\n") if ($use_center);
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t$font_end\n".
          "\\end{rawhtml}\n"
        );
    } else {
      $use_hsign = &main::sign_to_html($use_hsign);
      # &main::write_to_texfile("\\begin{rawhtml}\n");
      &main::write_to_texfile("<A HREF=\"Sources/applet-$use_local_name-$use_type.html\">$use_hsign<\/A>\n"); 
      &main::write_to_texfile("\\end{rawhtml}\n");
    }
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "<\/A>\n".
      "\\end{rawhtml}\n"
    ) if ($use_label);
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\t<P>\n".
      "\\end{rawhtml}\n"
    );
  
    if ($use_codelink) {
      if (-f "$main::WORK_DIR/$main::OUTPUT_DIR/Sources/code-$use_local_name-$use_type.html") {
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t<A HREF = \"Sources/code-$use_local_name-$use_type.html\"> $use_codelink </A>\n".
          "\t<P>\n".
          "\\end{rawhtml}\n"
        );
      }
    }
    &main::write_to_texfile("% END : $use_name fuer HTML\n");
  
    # Import-Classen bearbeiten
    &make_import;
  }
  
  # macht die notwendigen Links in HTML zwischen
  # schon existierenden Applet, Java-Source und Java-Doc
  sub java_html {
    my $code;
  
    $code = $use_name;
  
    &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer HTML ***\n");
    &main::write_to_texfile(
      "Source:\n". 
      "\\begin{rawhtml}\n".
      "\t<A HREF=\"$code\">\n".
      "\t\t$use_name_value.java\n".
      "\t</A>\n".
      "\t&nbsp;\n".
      "\t&nbsp;\n".
      "\\end{rawhtml}\n"
    );
    &main::write_to_texfile(
      "JavaDoc:\n".
      "\\begin{rawhtml}\n".
      "\t<A HREF=\"$doc_file#_top_\">\n".
      "\t\t$use_name_value.html\n".
      "\t</A>\n".
      "\t&nbsp;\n".
      "\t&nbsp;\n".
      "\\end{rawhtml}\n"
    ) if ($use_doc);
    &main::write_to_texfile(
      "Applet:\n".
      "\\begin{rawhtml}\n".
      "\t<A HREF=\"$applet_file\">\n". 
      "\t\t<IMG ALIGN=MIDDLE BORDER=0 SRC=\"$use_applet_image\">\n". 
      "\t</A>\n".
      "\\end{rawhtml}\n"
    ) if ($use_applet);
    &main::write_to_texfile("% END : $use_name fuer HTML\n");
  }
  
  sub java_latex {
    my $local_name=&main::link_to_latex("$use_local_name.java");
    my $java_file = "$use_name";
  
    unless ($java_file =~ /^\//) {
      $java_file =~ s/^\.\.\///;
    }
  
    $java_file = &main::get_relativ_name($java_file);
  
    unless (-f "$java_file") {
      &main::my_msg("Java-file $java_file not found.\n");
    }
    &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer LaTeX ***\n");
    &main::write_to_texfile("\\addcontentsline{lot}{section}{\\tt $local_name}\n");
    &main::write_to_texfile("{\\normalsize\n");
    &main::write_to_texfile("\\verbatiminput{$java_file}\n");
    &main::write_to_texfile("}\n");
    &main::write_to_texfile("% END : $use_name fuer LaTeX\n");
  }
  
  sub java_to_html {
    my ($DIR_TARGET,$CODEBASE);
    
    my $DIR_TARGET="$main::OUTPUT_DIR/Applets/";
    my $DIRNAME = &main::get_base_name($use_dir);
    my $CODEBASE="Applets";
    
    if (($main::COPY_SOURCE)&&((&ok_to_work)||
        (!(-f "$main::WORK_DIR/$main::OUTPUT_DIR/$CODEBASE/$DIRNAME/$use_local_name.$use_type")))) {
      &main::get_underdir("$CODEBASE");
      &main::my_print("Copy $use_dir to $main::OUTPUT_DIR/$CODEBASE\n");
      $status = &main::my_system("cp -r $use_dir $main::WORK_DIR/$main::OUTPUT_DIR/$CODEBASE");
      
      # es hat sich was geaendert 
      $main::SOURCE_IS_CHANGE=1;
    }
     if (($use_codelink)&&((&ok_to_work)||
         (!(-f "$main::WORK_DIR/$main::OUTPUT_DIR/Sources/code-$use_local_name-$use_type.html")))) {
      &generate_html_of_source("$use_dir/$use_local_name".".java");
    }
    if ((!($use_embed))&&((&ok_to_work)||
        (!(-f "$main::WORK_DIR/$main::OUTPUT_DIR/Sources/applet-$use_local_name-$use_type.html")))) {
      &generate_applet_of_source("$use_name");
    }
    if ($main::COPY_SOURCE) { return "./"."$CODEBASE/$DIRNAME"; }
    else {
      if ($main::RELATIVE_PFAD) {
        return $use_relativ_path;
      }
      else { return $use_dir; }
    }
  }
  
  # Kopiert falls notwendig die zu dem Java-Projekt
  # gehoerigen Klassen in das output-Verzeichnis
  sub make_import {
    my @import_list;
    my $item;
    if (($main::COPY_SOURCE)&&($use_import)&&(&ok_to_work)) {
      &main::my_print("Working of import-list ...\n");
      @import_list=split /,/,$use_import;
      foreach $item (@import_list) {
        if (-f "$item") {
          &main::copy($item,"$main::WORK_DIR/$main::OUTPUT_DIR/Applets");
        } elsif (-d "$item") {  
          &main::my_system("cp -r $item $main::WORK_DIR/$main::OUTPUT_DIR/Applets");
        } else {
          &main::my_msg("File or directory $item not found.\n");
        }
      }
    }
  }
  
  # Generiert HTML-Datei mit Code von Java-Datei
  sub generate_html_of_source {
    my ($file) = shift;          #Dateiname als Argument
    my $TARGET="Sources";
    my $line;
    
    unless (-f $file) {
      &main::my_msg("Generate of \'codelink\' failed.");
      return;
    }
    unless (-d "$main::WORK_DIR/$main::OUTPUT_DIR/$TARGET") {
      &main::get_underdir("$TARGET");
    }
  
    open (CODE,"$file") or &main::my_exit("Error by open : $file\n");
    &main::my_print("Generate of $main::OUTPUT_DIR/code-$use_local_name-$use_type.html ...\n") if ($main::DEBUG);
    open (HTML,"> $main::WORK_DIR/$main::OUTPUT_DIR/$TARGET/code-$use_local_name-$use_type.html") 
      or &main::my_exit("Error by write : $main::OUTPUT_DIR/$TARGET/code-$use_local_name-$use_type.html\n");
    print HTML "<!Diese Datei wurde von Master-Generator erzeugt>\n";
    print HTML "<HTML>\n";
    print HTML "<HEAD>\n";
    print HTML "<TITLE> Source of $use_local_name.$use_type </TITLE>\n";
    print HTML "</HEAD>\n";
    print HTML "<BODY BGCOLOR=$main::BGCOLOR TEXT=$main::TEXTCOLOR BACKGROUND=$main::BACKGROUND>\n";
    print HTML "<PRE>\n";
    while ($line=<CODE>) { 
      $line =~ s/\t/      /g;      # ersetze die Tabs durch die Leerzeichen
      print HTML "$line"; 
    }
    print HTML "</PRE>\n";
    print HTML "</BODY>\n";
    print HTML "</HTML>\n";
  
    close HTML;
    close CODE;
      
    # es hat sich was geaendert 
    $main::SOURCE_IS_CHANGE=1;
  }
  
  # Generiert HTML-Datei mit einem Applet von der Java-Datei
  sub generate_applet_of_source {
    my ($file) = shift;          #Dateiname als Argument
    my $TARGET="Sources";
    my $width="WIDTH=$use_hwidth";
    my $height="HEIGHT=$use_hheight";
    my $archive;
    my $DIRNAME = &main::get_base_name($use_dir);
    
    unless (-f $file) {
      &main::my_msg("Generate of \'applet-html\' failed.");
      return;
    }
    unless (-d "$main::WORK_DIR/$main::OUTPUT_DIR/$TARGET") {
      &main::get_underdir("$TARGET");
    }
  
    &main::my_print("Generate of $main::OUTPUT_DIR/$TARGET/applet-$use_local_name-$use_type.html ...\n") if ($main::DEBUG);
    open (HTML,"> $main::WORK_DIR/$main::OUTPUT_DIR/$TARGET/applet-$use_local_name-$use_type.html") 
      or &main::my_exit("Error by write : $main::OUTPUT_DIR/$TARGET/applet-$use_local_name-$use_type.html\n");
      
    if ($use_archive) { $archive="archive=".$use_archive; }
    print HTML "<!Diese Datei wurde von Master-Generator erzeugt>\n";
    print HTML "<HTML>\n";
    print HTML "<HEAD>\n";
    print HTML "<TITLE> Applet of $use_local_name.$use_type </TITLE>\n";
    print HTML "</HEAD>\n";
    print HTML "<BODY BGCOLOR=$main::BGCOLOR TEXT=$main::TEXTCOLOR BACKGROUND=$main::BACKGROUND>\n";
    print HTML "<CENTER>\n";
    if ($main::COPY_SOURCE) {
      print HTML "<APPLET CODEBASE=\"../Applets/$DIRNAME\" CODE=\"$use_local_name\" $width $height $archive>\n";
    } else {
      print HTML "<APPLET CODEBASE=\"$use_dir\" CODE=\"$use_local_name\" $width $height>\n";
    }  
    print HTML "<\/APPLET><BR>\n";
    print HTML "<\/CENTER>\n";
    print HTML "</BODY>\n";
    print HTML "</HTML>\n";
  
    close HTML;
      
    # es hat sich was geaendert 
    $main::SOURCE_IS_CHANGE=1;
  }
  
}

$MANAGEFUNKTION->{JAVA} = \&java_work;

1;
