#
# @command NEWCOMMAND
# @title   Makro definieren
# @index   
# @syntax  \#NEWCOMMAND\{name=args\}\\\%Definition\\\#NEWCOMMAND{end}
# @desc    Durch die {\tt \#NEWCOMMAND}-Umgebung wird ein neuer Makrobefehl mit
#          dem Namen {\tt Macroname} deklariert, der den in {\tt Makro} definierten
#          Text setzt bzw. dort enthaltene Befehle ausf"uhrt.\\
#          Innerhalb der Definition {\tt Makro} k"onnen weitere Master- oder 
#          \LaTeXe-Befehle aufgerufen werden. Das optionale Argument
#          {\tt args} gibt die Anzahl der Argumente des neuen durch {\tt Makroname}
#          bezeichneten Befehls an. Die Parameter der Befehle werden in der Reihenfolge
#          numeriert, in der sie beim Befehlsaufruf angegeben werden. Die Argumente
#          werden intern in {\tt Makro} als {\tt \#argnum} referiert, wobei {\tt argnum}
#          die Nummer des Argumentes ist.\\
#          Der Aufruf von {\tt \#NEWCOMMAND\{Makroname\}} f"uhrt zu einer 
#          Fehlermeldung, wenn der Makroname {\tt Makroname} zuvor bereits bekannt war.
#          In diesem Fall kann die bestehende Makrodefinition durch
#          {\tt \#RENEWCOMMAND} redefiniert, also mit einem neuen {\tt Makro} versehen,
# @example_begin
#   #NEWCOMMAND{MY_IMAGE=2}      % neuer Befehl MY_IMAGE
#                                % mit 2 Argumenten
#      #IMAGE{                   % setze ein Bild
#             file=#1,           % Erstes Argument ist Dateiname 
#             sign="Datei \{\\tt #1\}",
#             center,
#             width=#2           % zweites Argument 
#                                % ist die Bildbreite
#      }
#   #NEWCOMMAND{end}
# @example_end
#
#   @optname name
#   @optargs args
#     Definiert einen neuen Befehl, der im folgenden unter {\tt \#name} verwendbar ist.
#     Der Befehl bekommt {\tt args} Argumente.
#
#
# @command RENEWCOMMAND
# @title   Makro umdefinieren
# @index   
# @syntax  \#RENEWCOMMAND\{name=args\}\\\%Definition\\\#RENEWCOMMAND{end}
# @desc    Durch die {\tt \#RENEWCOMMAND}-Umgebung wird ein vorher in einer
#          {\tt \#NEWCOMMAND}-Umgebung definiertes Makro umdefiniert, der
#          alte Wert des Makros geht verloren.
#          Innerhalb der Definition {\tt Makro} k"onnen weitere Master- oder 
#          \LaTeXe-Befehle aufgerufen werden. Das optionale Argument
#          {\tt args} gibt die Anzahl der Argumente des neuen durch {\tt Makroname}
#          bezeichneten Befehls an. Die Parameter der Befehle werden in der Reihenfolge
#          numeriert, in der sie beim Befehlsaufruf angegeben werden. Die Argumente
#          werden intern in {\tt Makro} als {\tt \#argnum} referiert, wobei {\tt argnum}
#          die Nummer des Argumentes ist.\\
#          Der Aufruf von {\tt \#RENEWCOMMAND\{Makroname\}} f"uhrt zu einer 
#          Fehlermeldung, wenn der Makroname {\tt Makroname} nicht bekannt ist.
# @example_begin
#   #RENEWCOMMAND{MY_IMAGE=2}    % bestehenden Befehl MY_IMAGE umdefinieren
#                                % mit 2 Argumenten

#      Hier sollte das Bild {\tt #1 } zu sehen sein.
#   #RENEWCOMMAND{end}
# @example_end
#
#   @optname name
#   @optargs args
#     Definiert einen bestehenden Befehl des Namens {\tt name} um,
#     der Befehl bekommt {\tt args} Argumente.
#
# @end

sub new_work {
  package new;

  my $COMMAND=shift @_;    # Master-Befehl
  my (@PARAMS) = @_;       # uebergebene Liste
  my $term;
  my ($cmd,$args);
  my $DK=$main::SHOWDESKRIPTOR{$main::INPUTFILE};  # Deskriptor der Master-Datei
  my $end;
  my $line;
  my $rest;

  $term = shift @PARAMS;    
  ($cmd,$args)=&main::my_quotewords("=",1,$term);
  $cmd =~ s/(\s*)//g;      # Trennzeichen loeschen
  $args =~ s/"//g;

  &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : (re)define Macro \'$cmd\' ...\n");

  # altes Makro beim Redefinieren loeschen  
  if ($COMMAND eq "RENEWCOMMAND") {
    unless ($main::MACROS{$cmd}) {
      &main::my_exit("Makro '$cmd' is undefined.\n");
    }
    $main::MACROS{$cmd} = "";
  }  

  if ($cmd eq "end") {
    &main::my_exit("\#$COMMAND\{end\} withhout \#$COMMAND\{newname\}\n");
  } else {

    # wenn Makro bereits existiert, dann Fehlermeldung und Exit
    if ($main::MACROS{$cmd}) {
      &main::my_exit("Macro \'$cmd\' already defined.\n");
    }
  
    $main::MACROS_ARGS{$cmd}=$args;  # Argumentenzahl merken
    
    package main;
    while (!($end)&&($line=<$DK>)) {   #lese die Datei Zeilenweise
      if ($line =~ /^(\s*)#$COMMAND(\s*){(\s*)(.*)(\s*)}(.*)$/) {
        if ($4 eq "end") {
          # Definition ist zu Ende
          $end=1;
          $rest="$5";
        } else {
          # rekursiver Aufruf, falls
          # Makro-Definition bei einer Makro-Definition
          &main::new_work($COMMAND,$4);
        }
      # Makro merken
      } else {
        # loesche Kommentar
        $line =~ s/([^\\])%(\s*)(.*)$/$1/;
        $main::MACROS{$cmd} .= $line; # Makro merken;
      }
    }
    package new;
    &main::write_to_texfile("$rest\n") if ($rest);
  }
}

# Macro bearbeiten (evaluieren)
sub macro_work {
  package macro;

  my ($wert,$flag);
  my $COMMAND=shift @_;
  my (@PARAMS) = @_;  # uebergebene Liste
  my $arg;            # Argument
  my $arg_num;        # Nummer des Argumentes
  my $macro;          # Makro-Text
  
  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}
  
  &main::my_print("$COMMAND (macro): (l.$main::INPUTFILE_LINE)\n");
  $macro = $main::MACROS{$COMMAND};
  foreach $arg (@PARAMS) {
    $arg =~ s/\\(.)/$1/g;
    $arg_num++;
    $macro =~ s/\#$arg_num/$arg/g;
  }

  if ($main::MACROS_ARGS{$COMMAND}) {
    if ($arg_num > $main::MACROS_ARGS{$COMMAND}) {
      &main::my_msg("Too many arguments in macro \#$COMMAND.\n");
    } elsif ($arg_num < $main::MACROS_ARGS{$COMMAND}) {
      &main::my_msg("Too little arguments in macro \#$COMMAND.\n");
    }
  }

  # Ersetze alle verbliebene (nicht uebergebene)
  # Argumente durch ""
  # Falls in der Makro-Definition ein Argument steht,
  # der nich uebergeben wurde, dann eine Meldung 
  if ($macro =~ s/\#(\d+)//g) {
    &main::my_msg("Argument \#$1 in macro \#$COMMAND is not response.\n");
  }
     
  # Makro evaluieren
  &main::parse_of_commands($main::INPUTFILE,"$macro",1);

  return;
}

$MANAGEFUNKTION->{MACRO}        = \&macro_work;
$MANAGEFUNKTION->{NEWCOMMAND}   = \&new_work;
$MANAGEFUNKTION->{RENEWCOMMAND} = \&new_work;

1;
