# @command MEDIA
# @title   Einbinden von multimedialen Quellen
# @index   
# @syntax  \#MEDIA\{optionen\}
# @desc    Der Befehl {\tt \#MEDIA} erm"oglicht es, eine multimediale Datei
#          in ein Master-Dokument einzubinden. In der \LaTeXe-Version wird
#          statt so einer Quelle das alternative Bild dargestellt. Das Format
#          der multimedialen Datei ist beliebig ( {\tt .avi}, {\tt .mov},
#          {\tt .wav}, {\tt .mid}, ...). F"ur das alternative Bild werden
#          alle Formate vom {\tt \#IMAGE}-Befehl unterst"utzt.
# @example_begin
#  #MEDIA{ 
#      file=Media/bach.mid,                 % MIDI-Datei
#      imglink,                             % sensitives Bild
#      noembed,                             % nicht eingebettet
#      alt=../Ps/bach.ps,                   % Bild fuer LaTeX
#      width=7cm,                           % LaTeX-Breite
#      center,                              % zentrieren
#      sign="J. S. Bach"                    % LaTeX-Unterschrift
#  }
# @example_end
#
#   @optname file
#   @optargs <mediadatei>
#      Diese Option ist notwendig. Die {\tt mediadatei} wird in die HTML-Version integriert.\\
#      Beispiel: {\tt file=Quellen/music.mid} 
#
#   @optname alt
#   @optargs <alternativbilddatei>
#      Die Bilddatei wird in der \LaTeXe-Version eingef"ugt.
#      Die {\tt alternativbilddatei} mu"s den gesamten Pfad enthalten. 
#
#   @optname (no)center
#      Die Quelle und das alternative Bild (nicht) zentrieren.
#
#   @optname (no)embed
#      Die Quelle (nicht) einbetten. Bei der Option {\tt noembed} wird eine Verkn"upfung mit 
#      dem Namen {\tt hsign} auf die Datei erstellt. 
#
#   @optname (no)figure
#      Siehe Seite \pageref{cmd:IMAGE:(no)figure} ({\tt \#IMAGE}-Optionen). 
#
#   @optname height
#   @optargs <bildh"ohe>
#      Siehe Seite \pageref{cmd:IMAGE:height} ({\tt \#IMAGE}-Optionen). 
#
#   @optname hheight
#   @optargs <html-h"ohe>
#      HTML-H"ohe in Pixel oder Prozent.\\ 
#      Defaultwert: 0 
#
#   @optname hsign
#   @optargs \dq html-unterschrift\dq
#      Bei eingebetteten Quellen Media-Unterschrift, sonst Verkn"upfungsname.
#      Diese Option kann auch die HTML-Tags enthalten.\\
#      Beispiel: {\tt hsign= \dq Verkn\&uuml;pfungsname\dq}
#
#   @optname hwidth
#   @optargs <html-breite>
#      HTML-Breite in Pixel oder Prozent.\\
#      Defaultwert: 0 
#
#   @optname imglink
#      Es erscheint in der HTML-Version ein sensitives Bild ({\tt alternativbilddatei}), das mit 
#      der Media-Datei verkn"upft wird. Bei dieser Option
#      wird der {\tt embed}-Schalter ignoriert.
#
#   @optname label
#   @optargs <labelbezeichner>
#      Siehe Seite \pageref{cmd:IMAGE:label} ({\tt \#IMAGE}-Optionen). 
#
#   @optname (no)loop
#      Die Mediadatei wird (nicht) endlos abgespielt ({\tt LOOP= \dq(FALSE) TRUE\dq }). 
#
#   @optname (no)autostart
#      Die Mediadatei wird (nicht) automatisch abgespielt ({\tt AUTOSTART= \dq (FALSE) TRUE\dq }). 
#
#   @optname options
#   @optargs \dq EMBED-Optionen\dq
#      Hier kann man einige Optionen f"ur den {\tt <EMBED>}-Tag
#      eingeben.
#
#   @optname sign
#   @optargs \dq bildunterschrift\dq
#      Siehe Seite \pageref{cmd:IMAGE:sign} ({\tt \#IMAGE}-Optionen). 
#
#   @optname width
#   @optargs <bildbreite>
#      Siehe Seite \pageref{cmd:IMAGE:width} ({\tt \#IMAGE}-Optionen). 
#
# @end

# MEDIA-Work
sub media_work {
  package media;

  my ($wert,$flag);
  local ($use_name,$use_alt_name,$use_sign,$use_imglink);
  local ($use_local_name,$use_type);
  local $use_embed = $main::USE_EMBED;
  local $use_center = $main::USE_CENTER;
  local ($use_options);
  local ($use_hsign,$hsign_flag);
  local ($use_play,$use_loop);
  local ($use_relativ_name);          # Relativer Dateiname;
  local ($use_dir);
  local (@copy_additional_files);
  local ($noticy_key);
  local ($mtime);
  local $status;          # gleich null, wenn die
                  # Konvertierung fehlerfrei
  my (@stat_liste);
  my $file_name;          # Dateiname, der angegeben ist
                  # wird in die NTC-Datei eingetragen
  my $report_state = $main::REPORT_WRITE; # merkt altes Report-Zustand
                      # wird in die NTC-Datei eingetragen
  local $COMMAND = shift @_;
  local (@PARAMS) = @_;      # uebergebene Liste
  local (@TO_IMAGE);        # Liste zur Bildbearbeitung
  
  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}
  
  # Flags setzen
  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    #$wert =~ s/"//g;
    SWITCH: {
      if ($flag eq "file") {$use_name=$wert; $file_name=$wert; last SWITCH;}
      if ($flag eq "copy") { push (@copy_additional_files, $wert); last SWITCH;}
      if ($flag eq "embed") {$use_embed="1"; last SWITCH;}
      if ($flag eq "imglink") {$use_imglink="1"; last SWITCH;}
      if ($flag eq "noembed") {$use_embed=""; last SWITCH;}
      if ($flag eq "hsign") { $use_hsign=$wert; $hsign_flag="1"; last SWITCH;}
      if ($flag eq "hwidth") { $use_hwidth=$wert; last SWITCH;}
      if ($flag eq "hheight") { $use_hheight=$wert; last SWITCH;}
      if ($flag eq "play") {$use_play="TRUE"; last SWITCH;}
      if ($flag eq "noplay") {$use_play="FALSE"; last SWITCH;}
      if ($flag eq "autostart") {$use_play="TRUE"; last SWITCH;}
      if ($flag eq "noautostart") {$use_play="FALSE"; last SWITCH;}
      if ($flag eq "loop") {$use_loop="TRUE"; last SWITCH;}
      if ($flag eq "noloop") {$use_loop="FALSE"; last SWITCH;}
      if ($flag eq "sign") { $use_sign=$wert; last SWITCH;}

      if ($flag eq "alt") { $use_alt_name=$wert; push @TO_IMAGE,"file=$wert"; last SWITCH;}
      if ($flag eq "label") { $use_label=$wert; push @TO_IMAGE,"label=$wert"; last SWITCH;}
      if ($flag eq "center") { $use_center=1; push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nocenter") { $use_center=""; push @TO_IMAGE,"$flag"; last SWITCH;}

      if ($flag eq "figure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nofigure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nospace") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "width") { push @TO_IMAGE,"width=$wert"; last SWITCH;}
      if ($flag eq "height") { push @TO_IMAGE,"height=$wert"; last SWITCH;}
      if ($flag eq "options") { $use_options=$wert; last SWITCH;}

      &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }
  # wenn es nur um das Bild geht
  if (($main::OUTPUTFORMAT eq "latex") || ($main::OUTPUTFORMAT eq "pdf")) {
    if ($use_alt_name) {
      push @TO_IMAGE,"sign=".$use_sign; 
      unshift @TO_IMAGE,"$COMMAND";
      &main::image_work(@TO_IMAGE);
    } else { &main::my_print("$COMMAND : Not Alt-File\n"); }
    return;
  } else {
    &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : File=$use_name\n");
  }

  &to_flag_check;    # Syntax pruefen und wenn noetig die
            # default-Werte setzen
  
  
  $use_relativ_name = &main::get_relativ_name($use_name);  # relativer Pfadname
  $use_local_name = &main::get_base_name($use_name);    # Basisname
  $use_type = &main::get_file_type($use_name);      # Dateierweiterung
  &main::statistic_count($use_type);            # fuer Statistik
  # Absoluten Pfad erkuendigen
  $use_dir=&main::get_abs_pfad($use_name);
  $use_name = "$use_dir/$use_local_name.$use_type";


  # den Notice-Schluessel bilden
  $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name);

  # Dateieigenschaften erkundigen (Dateigroesse und die letzte
  # Modifikationszeit)
  @stat_liste = stat $use_name;
  $mtime = $stat_liste[9];

  # Schreiben in die Report-Version erlauben
  $main::REPORT_WRITE = 1;
  # Entsprechende Mechanismen aufrufen
  &html_work;
  # Schreiben in die Report-Version zuruecksetzen
  $main::REPORT_WRITE = $report_state;

  # wenn alles geklappt, dann Notice-Eintrag machen
  if (!($status)) { #wenn erfolgreich dann eintragen
    $main::NOTICE_NEW{$noticy_key}="$mtime";
  }

  return;

  # Prueft Syntax und Existenz von Quellen vom Master-Befehl 
  sub to_flag_check {
    unless ($use_name) {&main::my_exit("File is not statement.\n");}
    unless (-f $use_name) {&main::my_msg("File '$use_name' not found.\n");}
    
    # Pruefe die HTML-Groessen
    $use_hwidth =~ s/(\s+)//g;
    if ($use_hwidth) { 
      unless ($use_hwidth =~ /^((\d+)|(\d+%))$/) {
        &main::my_msg("Not allowed value in $COMMAND for hwidth : $use_hwidth\n");
        $use_hwidth="";
      }
    }
    $use_hheight =~ s/(\s+)//g;
    if ($use_hheight) { 
      unless ($use_hheight =~ /^((\d+)|(\d+%))$/) {
        &main::my_msg("Not allowed value in $COMMAND for hheight : $use_hheight\n");
        $use_hheight="";
      }
    }
  
    if ((!($use_hsign))&&(!($hsign_flag))) {$use_hsign=$use_sign;}
    if (($use_imglink)&&($use_embed)&&(!($main::USE_EMBED))) {
      &main::my_msg("Flags \'embed\' and \'imglink\' in $COMMAND. Ignore \'embed\'.\n");
      $use_embed="";
    }
  }
  
  # Prueft, ob die Konvertierungsroutine noetig ist
  sub ok_to_work {
    my $ntc_key;
    my $mod_time;
    
    $ntc_key="$noticy_key";
    $mod_time="$mtime";
  
    if ($main::MAKE_ALL) {return 1;}
    if ($main::NOT_NTCFILE) {return 0;}
    if (((exists $main::NOTICE_OLD{$ntc_key})
      and($main::NOTICE_OLD{$ntc_key} == $mod_time))
      or (exists $main::NOTICE_NEW{$ntc_key})) {return 0;} 
    return 1;
  }
  
  # Diese Subroutine konvertiert Master-Kode in HTML-Kode        #
  sub html_work {
    my ($new_name);
    my ($width,$height,$play,$loop);
    my ($font_begin,$font_end); 
  
    $new_name = &media_to_html;
  
    if ($main::HTML_SIGN_FONT) {
      $font_begin = "<$main::HTML_SIGN_FONT>";
      $font_end = "<\/$main::HTML_SIGN_FONT>";
    }
    if ($use_hwidth) { $width="WIDTH=$use_hwidth"; }
    if ($use_hheight) { $height="HEIGHT=$use_hheight"; }
    $play="Autostart=\"$use_play\"" if ($use_play);
    $loop="Loop=\"$use_loop\"" if ($use_loop);
    &main::write_to_texfile(
      "% *** Master-Generator - $COMMAND fuer HTML ***\n".
      "% BEGIN : $use_local_name.$use_type\n".
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\\end{rawhtml}\n"
    );
    &main::write_to_texfile("<A NAME=$use_label>\n") if ($use_label);
    if ($use_embed) {
      $use_hsign = &main::sign_to_html($use_hsign);
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<P>\n".
        "\t<CENTER>\n".
        "\t<EMBED SRC=\"$new_name\" $width $height $play $loop $use_options>\n".
        "\t<BR>\n".
        "\t$font_begin $use_hsign $font_end\n".
        "\t<\/CENTER>\n".
        "\\end{rawhtml}\n"
      );
    } elsif (($use_imglink)&&($use_alt_name)) {
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<A HREF=\"$new_name\">\n".
        "\\end{rawhtml}\n"
      );
      push @TO_IMAGE,"NO_BOT_SPACE"; 
      unshift @TO_IMAGE,"$COMMAND";
      &main::image_work(@TO_IMAGE);
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<\/A>\n".
        "\t$font_begin\n".
        "\\end{rawhtml}\n"
      );
      &main::write_to_texfile("\\begin{center}\n") if ($use_center);
      if ($hsign_flag) {
        &main::write_to_texfile($use_hsign."\n");
      } else {
        #$use_sign=&main::sign_to_html($use_sign);
        &main::write_to_texfile($use_sign."\n");
      }
      &main::write_to_texfile("\\end{center}\n") if ($use_center);
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t$font_end\n".
        "\\end{rawhtml}\n"
      );
    } else {
      $use_hsign = &main::sign_to_html($use_hsign);
      &main::write_to_texfile("\\begin{rawhtml}\n");
      if ($use_hsign) {
        &main::write_to_texfile("\t<A HREF=\"$new_name\"> ".$use_hsign." <\/A>\n");
      } else {
        &main::write_to_texfile("\t<A HREF==\"$new_name\"> <$use_local_name.$use_type> <\/A>\n");
      }
      &main::write_to_texfile("\\end{rawhtml}\n");
    }
    &main::write_to_texfile("\\begin{rawhtml}\n");
    &main::write_to_texfile("\t<\/A>\n") if ($use_label);
    &main::write_to_texfile( "\t<BR><P>\n");
    &main::write_to_texfile("\\end{rawhtml}\n");
    &main::write_to_texfile("% END : $use_name fuer HTML\n");
  }
  
  sub media_to_html {
    my ($DIR_TARGET,$NW_NAME);
    
    $DIR_TARGET="$main::OUTPUT_DIR/Media/";
    $NW_NAME="Media/$use_local_name.$use_type";
    
    #if ($main::COPY_SOURCE) {&main::get_underdir("Media");}
  
    if (($main::COPY_SOURCE)&&((&ok_to_work)||(!(-f "$main::WORK_DIR/$main::OUTPUT_DIR/$NW_NAME")))) {
      &main::get_underdir("Media");
      &main::my_print("Copy $use_name to $main::OUTPUT_DIR/$NW_NAME\n");
      $status=&main::copy($use_name,"$main::WORK_DIR/$main::OUTPUT_DIR/$NW_NAME");
      if ($status==0) { &main::my_exit("Error copying $use_name."); }

      # Copy additional files
      foreach $add_file (@copy_additional_files) {
        $add_local_file = &main::get_base_name($add_file);
        $add_type = &main::get_file_type($add_file);
        $add_dir  = &main::get_abs_pfad($add_file);
        $add_file = "$add_dir/$add_local_file.$add_type";
        $add_dest = "Media/$add_local_file.$add_type";

        &main::my_print("Copy $add_file to $main::OUTPUT_DIR/$add_dest\n");
        $status=&main::copy($add_file,"$main::WORK_DIR/$main::OUTPUT_DIR/$add_dest");
        if ($status==0) { &main::my_exit("Error copying $add_file."); }
      }
      
      # es hat sich was geaendert 
      $main::SOURCE_IS_CHANGE=1;
    }
  
    if ($main::COPY_SOURCE) { return $NW_NAME;}
    else {
      if ($main::RELATIVE_PFAD) {return "../".$use_relativ_name;}
      else { return $use_name; }
    }
  }
  
}
$MANAGEFUNKTION->{MEDIA} = \&media_work;

1;
