#
# @command MIX
# @title   Bedingter Text
# @index   
# @syntax  \#MIX\{begin\}\\...\\\#MIX-Schalter\\...\\\#MIX\{end\}
# @desc    Der Master-Mechanismus bietet die M"oglichkeit, einzelne
#          Teile des Dokuments bedingt fr die \LaTeXe- oder
#          HTML-Version zu schreiben. Der bedingte Text kann beliebige
#          \LaTeXe-Befehle enthalten und beliebig lang sein. Mit der
#          {\tt \#MIX}-Umgebung wird bedingter Text erzeugt, es sind
#          die Befehle {\tt \#HTMLONLY}, {\tt \#LATEXONLY}, {\tt \#PDFONLY}
#          und {\tt \#COMMON} erlaubt.
#            
# @example_begin
#   #MIX{begin}
#      Dieses Dokument ist f"ur 
#      #LATEXONLY{}
#      \LaTeXe
#      #HTMLONLY{}
#      HTML
#      #PDFONLY{}
#      PDF
#      #COMMON{}
#      optimiert.
#   #MIX{end}
# @example_end
#
# @command HTMLONLY
# @title   Bedingter Text (nur HTML)
# @index   
# @syntax  \#HTMLONLY\{\}
# @desc    Nur innerhalb der {\tt \#MIX}-Umgebung erlaubt. Der Text zwischen diesem
#          Befehl und dem n"achsten Schalter (oder dem Ende der {\tt \#MIX}-Umgebung
#          wird nur bei der Ausgabe nach HTML bearbeitet.
#
# @command LATEXONLY
# @title   Bedingter Text (nur \LaTeXe)
# @index   
# @syntax  \#LATEXONLY\{\}
# @desc    Nur innerhalb der {\tt \#MIX}-Umgebung erlaubt. Der Text zwischen diesem
#          Befehl und dem n"achsten Schalter (oder dem Ende der {\tt \#MIX}-Umgebung
#          wird nur bei der Ausgabe nach \LaTeXe bearbeitet.
#
# @command PDFONLY
# @title   Bedingter Text (nur PDF)
# @index   
# @syntax  \#PDFONLY\{\}
# @desc    Nur innerhalb der {\tt \#MIX}-Umgebung erlaubt. Der Text zwischen diesem
#          Befehl und dem n"achsten Schalter (oder dem Ende der {\tt \#MIX}-Umgebung
#          wird nur bei der Ausgabe nach PDF bearbeitet.
#
# @command COMMON
# @title   Bedingter Text (alle)
# @index   
# @syntax  \#COMMON\{\}
# @desc    Nur innerhalb der {\tt \#MIX}-Umgebung erlaubt. Der Text zwischen diesem
#          Befehl und dem n"achsten Schalter (oder dem Ende der {\tt \#MIX}-Umgebung
#          wird f"ur alle Ausgabearten verwendet.
#
# @end

# MIX-Umgebung mit Befehlen COMMON, LATEXONLY, HTMLONLY, PDFONLY
#
# Beschreibung:
#   Der Master-Mechanismus bietet die Mglichkeit, einzelne Teile
#   des Dokuments bedingt fr die LATEX2e - oder HTML-Version zu
#   schreiben. Der bedingte Text kann beliebige LATEX2e -Befehle
#   enthalten und beliebig lang sein. Mit der #MIX-Umgebung wird
#   bedingter Text erzeugt. 
#
# Syntax:
#
#   #MIX{begin} 
#     ... 
#     #MIX-Schalter 
#     ... 
#   #MIX{end} 
#
#   Die MIX-Schalter drfen nur innerhalb der MIX-Umgebung stehen. Es gibt folgende MIX-Schalter: 
#
#   #LATEXONLY{} 
#       Alles, was nach diesem bis zum nchsten Schalter kommt, ist
#       nur in der LATEX2e -Version sichtbar. 
#   #HTMLONLY{} 
#       Alles, was nach diesem bis zum nchsten Schalter kommt, ist
#       nur in der HTML-Version sichtbar. 
#   #PDFONLY{} 
#       Alles, was nach diesem bis zum nchsten Schalter kommt, ist
#       nur in der PDF-Version sichtbar. 
#   #COMMON{} 
#       Alles, was nach diesem bis zum nchsten Schalter kommt, ist
#       fr alle Versionen relevant.

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{MIX}       = \&mix_work;
$MANAGEFUNKTION->{LATEXONLY} = \&mix_work;
$MANAGEFUNKTION->{HTMLONLY}  = \&mix_work;
$MANAGEFUNKTION->{PDFONLY}   = \&mix_work;
$MANAGEFUNKTION->{COMMON}    = \&mix_work;

$MIX_OK;

# MIX, COMMON, LATEXONLY, HTMLONLY, PDFONLY
sub mix_work {
  package mix;

  my ($flag);
  my $COMMAND = shift @_;
  $flag = shift @_;
  
  $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
  if ($COMMAND eq "MIX") {
    if ($flag eq "begin") {
      if ($main::MIX_OK) { &main::my_exit("\#MIX{begin} without \#MIX{end}\n"); }
      print main::OUT "% *** Master-Generator - MIX-Umgebung ***\n";
      $main::MIX_OK = "1";
    }
    elsif ($flag eq "end") {
      if (!($main::MIX_OK)) { &main::my_exit("\#MIX{end} without \#MIX{begin}\n"); }
      print main::OUT "% *** END - MIX-Umgebung ***\n";
      $main::MIX_OK = "";
      $main::TEXFILE_WRITE_OK = "1";
    }
    else { &main::my_exit("Unknown flag in MIX : $flag\n"); }
  }
  elsif ($COMMAND eq "LATEXONLY") {
    # wenn LATEXONLY ausserhalb der MIX-Umgebung, dann Fehlerausgabe
    if (!($main::MIX_OK)) { &main::my_exit("$COMMAND outside of MIX{}\n");}
    else {
      if ($main::OUTPUTFORMAT eq "latex") {
        $main::TEXFILE_WRITE_OK = "1";
      } else {
        # Alle nachfolgende Zeilen nicht in die Output-Datei schreiben
        $main::TEXFILE_WRITE_OK = "";
      }    
    }
  }
  elsif ($COMMAND eq "HTMLONLY") {
    # wenn HTMLONLY ausserhalb der MIX-Umgebung, dann Fehlerausgabe
    if (!($main::MIX_OK)) { &main::my_exit("$COMMAND outside of MIX{}\n");}
    else {
      if ($main::OUTPUTFORMAT eq "html") {
        $main::TEXFILE_WRITE_OK = "1";
      } else {
        # Alle nachfolgende Zeilen nicht in die Output-Datei schreiben
        $main::TEXFILE_WRITE_OK = "";
      }    
    }
  }
  elsif ($COMMAND eq "PDFONLY") {
    # wenn PDFONLY ausserhalb der MIX-Umgebung, dann Fehlerausgabe
    if (!($main::MIX_OK)) { &main::my_exit("$COMMAND outside of MIX{}\n");}
    else {
      if ($main::OUTPUTFORMAT eq "pdf") {
        $main::TEXFILE_WRITE_OK = "1";
      } else {
        # Alle nachfolgende Zeilen nicht in die Output-Datei schreiben
        $main::TEXFILE_WRITE_OK = "";
      }    
    }
  }
  elsif ($COMMAND eq "COMMON") {
    # wenn COMMON ausserhalb der MIX-Umgebung, dann Fehlerausgabe
    if (!($main::MIX_OK)) { &main::my_exit("$COMMAND outside of MIX{}\n");}
    else {
      # Alle nachfolgende Zeilen in die Output-Datei schreiben
      $main::TEXFILE_WRITE_OK = "1";
    }
  }
}

1;
