#
# @command PERLCODE
# @title   Ausf"uhrung von Perl-Programmcode
# @index   
# @syntax  \#PERLCODE\{begin\}\\perlscript\\\#PERLCODE\{end\}
# @desc    {\tt \#PERLCODE} ist eine Master-Umgebung oder, besser formuliert, eine Schnittstelle,
#          die es erm"oglicht, ein kleines \footnote{Das Programm darf h"ochstens 80 Zeichen enthalten. 
#          Aber das ist nat"urlich ausreichend, um ein externes Programm zu starten.} Programm
#          direkt bei der Konvertierung auszuf"uhren. Man kann es f"ur verschiedene 
#          Zwecke gebrauchen:
#          \begin{itemize}
#          \item dynamisch Index-Dateien erstellen,
#          \item Quell-Dateien in das Output-Verzeichnis kopieren,
#          \item make-Mechanismen f"ur die Teil-Dokumente aufrufen. 
#          \end{itemize}
#          Besonders interessant ist die {\tt \#PERLCODE}-Umgebung in Verbindung mit 
#          Master-Makros (s. Seite \pageref{cmd:NEWCOMMAND} und \pageref{cmd:RENEWCOMMAND}).
#          Dann kann man dem Perl-Programm auch die Argumente "ubergeben. \\
#          Das Perlprogramm {\tt perlskript} darf kein \grq-Zeichen enthalten. Als Kommentar-Zeichen
#          im Programm wird Lattenzaun (\#) benutzt.
# @example_begin
#   % Zu Beginn die Datei vrml.lst loeschen
#   #PERLCODE{begin}             % Perl-begin
#       unlink(vrml.lst)         # l"osche vrml.lst
#           if (-f "vrml.lst");  # falls vorhanden
#   #PERLCODE{end}               % Perl-end
#
#   % Neues Makro definieren, dass bei Einbettung der VRML-Quellen
#   % die Namen der Quellen in der Datei vrlm.lst mitprotokolliert.
#   #NEWMACRO{SETVRML=3}            % Makro mit 3 Argumenten
#      #VRML{                       % Vrml-Datei einbinden
#          file=#1,                 % Dateiname
#          alt=#2,                  % alternatives Bild
#          width=#3                 % Bild-Breite
#      }
#      #PERLCODE{begin}             % Perl-begin
#          open(DP,">>vrml.lst");   # vrml.lst zum Schreiben "offnen 
#          print DP "#1\n";         # Dateinamen eintragen
#          close(DP);               # vrml.lst schliessen 
#      #PERLCODE{end}               % Perl-end
#   #NEWMACRO{end}
# @example_end
#
# @end

sub perl_work {
  package new;

  my $COMMAND=shift @_;    # Master-Befehl
  my (@PARAMS) = @_;      # uebergebene Liste
  my $command;
  my $code;

  $command = shift @PARAMS;    

  if ($command eq "end") {
    
    $code = $main::TEXMEMORY;
      
    $status = &perl_exec("perl -e \'".$code."\'");
    &main::my_msg("Error at Perl-Script (code=$status).\n") if ($status);
    &main::my_print("$COMMAND: *** END *** \n");
    
    # verbiete das Schreiben in Speicher
    $main::WRITE_TO_TEXMEMORY=0;
    # Loesche Speicher
    $main::TEXMEMORY = "";
    
    # Verbiete das Warten
    &main::wait_of_command("");

  } elsif ($command eq "begin") {
    &main::my_print("$COMMAND: *** BEGIN *** (l.$main::INPUTFILE_LINE)\n");
    $main::WRITE_TO_TEXMEMORY = 1;
    # Warte auf PERLCOMMAND
    &main::wait_of_command("$COMMAND");
  } else {
    &main::my_exit("Unknown flag in $COMMAND : $command\n");
  }

  sub perl_exec {
    local($_) = @_;
    local($status);
    if ($child_pid = fork) {
      $status = waitpid($child_pid, 0);
      $child_pid = 0;
      return($?);
    } else {
      exec($_);
      &main::my_print("$_[0]:  $!\n");
      exit($!);
    }
  }
}

$MANAGEFUNKTION->{PERLCODE} = \&perl_work;

1;
