# @command SHELL
# @title   Ausgaben von externen Programmen einbinden
# @index   
# @syntax  \#SHELL\{optionen\}
# @desc    Startet ein externes Programm und schreibt die Ausgabe
#          dieses Programms in das Dokument. Als externe Programme
#          kommen sowohl Shell-Skripte, als auch Programme in Frage,
#          die ohne Eingabe des Benutzers ablaufen und Ausgabe auf
#          der Standardausgabe erzeugen.
# @example_begin
#   #SHELL{date}
# @example_end
#
#   @optname command
#   @optargs \dq cmd\dq
#     {\tt cmd} wird als externes Kommando (inklusive Parameter)
#     ausgef"uhrt. Die von Kommando erzeugte Standardausgabe wird
#     als Text oder Link eingef"ugt.
#
#   @optname (no)center
#     Der Text wird (nicht) zentriert eingef"ugt.
#
#   @optname (no)box
#     Der Text wird mit (ohne) einen umgebenden Rahmen eingef"ugt.
#
#   @optname text
#   @optargs \dq text\dq
#     Der Text {\tt text} wird in der \LaTeXe-Version als Unterschrift
#     unter die Ausgabe gesetzt.\\
#     Ist diese Option in der HTML-Version gesetzt, so wird die Ausgabe
#     als Link eingef"ugt, der Link erh"alt den Text {\tt text}. Ist die
#     Option nicht angegeben, so wird die Ausgabe direkt in den laufenden
#     Text eingef"ugt.
#
# @end

sub shell_work {
  package shell;
  
  my $COMMAND = shift @_;     # Erstes Argument ist der Befehlsname,
                              # in diesem Fall SHELL
  my (@PARAMS) = @_;          # uebergebene Liste (Optionen)
  my $option;                 # Laufende Variable, geht die Optionen durch
  my ($flag,$wert);           # $flag = Optionsname (command)
                              # $wert = Optionswert ("shell")
  my $use_command;            # Shell-Aufruf
  my $shell_out;              # Shell-Ausgabe
  my $center;                 # Zentrieren
  my $text;                   # Unterschrift
  my $nobox;                  # Rahmen
  my $line_count;        # Anzahl der Zeilen
  my $view_font;        # Font
  my $file;            
  my $file_name;

  # gehe die Optionen durch
  foreach $option (@PARAMS) {
    # erfahre den Optionsname und dazugeh"origen Wert  
    ($flag,$wert)=&main::my_quotewords("=",0,$option);

    # loesche die moeglichen Trennzeichen
    $flag =~ s/(\s*)//g;

    # pruefe die Optionen
    SWITCH: {
       if ($flag eq "command") {$use_command=$wert; last SWITCH;}
       if ($flag eq "text") {$text=$wert; last SWITCH;}
       if ($flag eq "nocenter") {$center = 0; last SWITCH;}
       if ($flag eq "center") {$center = 1; last SWITCH;}
       if ($flag eq "box") {$nobox=0; last SWITCH;}
       if ($flag eq "nobox") {$nobox=1; last SWITCH;}
       # falls unerlaubte Option, dann Fehlermeldung
       &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }
  
  # STDOUT
  &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : \"$use_command\"\n");

  # f"uhre das Shell-Programm aus 
  $shell_out = `$use_command`;

  $line_count = ($shell_out =~ s/\n/\n/g);
  if ($line_count > 50) { 
   $nobox = 1; 
  }

  # in der LaTeX/PDF-version als text
  if (($main::OUTPUTFORMAT eq "latex") || ($main::OUTPUTFORMAT eq "pdf")) {
    # Font setzen  
    if ($main::VIEWCODE_FONT) { $view_font = $main::VIEWCODE_FONT; }
    # Default-Font
    else { $view_font = "footnotesize"; }

    $file_name = $use_command;
    $file_name =~ s/\W//g;

    $file = "$main::WORK_DIR/$main::OUTPUT_DIR/$file_name.txt";
    open (CODE,">$file") or &main::my_exit("Error by open : $file\n");
    print CODE "$shell_out";
    close(CODE);

    &main::write_to_texfile(
      "\n".
      "\\fbox{\\parbox{16cm}{\n"
    ) unless ($nobox);
    &main::write_to_texfile("{\\$view_font \\verbatiminput{$main::OUTPUT_DIR/$file_name.txt}}\n");
    unless ($nobox) {
      &main::write_to_texfile("}}\n");
      &main::write_to_texfile("\\begin{center}{\\it $text}\\end{center}\n") if ($text);
      &main::write_to_texfile("\\vspace{3mm}\n\n");
    }

  # HTML-Version 
  } else {
    # als Link
    if ($text) {
      unless (-d "$main::WORK_DIR/$main::OUTPUT_DIR/Sources") {
        &main::get_underdir("Sources");
      }

      $file_name = $use_command;
      $file_name =~ s/\W//g;
      print "VHD = ".$file_name."\n";

      $file = "$main::WORK_DIR/$main::OUTPUT_DIR/Sources/$file_name.txt";
      open (CODE,">$file") or &main::my_exit("Error by open : $file\n");
      print CODE "$shell_out";
      close(CODE);

      $htmllink = "Sources/$file_name.txt";

      if ($center) {
        &main::write_to_texfile("\\begin{center}\n");
      }
      &main::write_to_texfile(
        "\t\\begin{rawhtml}\n".
        "\t\t<A HREF=\"$htmllink\">\n".
        "\t\\end{rawhtml}\n".
        "\t$text"."\n".
        "\t\\begin{rawhtml}\n".
        "\t\t\&nbsp;<\/A>\n".
        "\t\\end{rawhtml}\n"
      );
      if ($center) {
        &main::write_to_texfile("\\end{center}\n");
      }
      # als Text
    } else {
     &main::write_to_texfile("\\begin{verbatim}\n");
     &main::write_to_texfile("$shell_out"); 
     &main::write_to_texfile("\\end{verbatim}\n");
    }
  }
}

$MANAGEFUNKTION->{SHELL} = \&shell_work;

1;

