#
# @command SHOW
# @title   \LaTeXe-Umgebungen als Bild einf"ugen
# @index   
# @syntax  \#SHOW\{begin,optionen\}\\\dots\\\#SHOW\{end\}
# @desc    Die {\tt \#SHOW}-Umgebung erm"oglicht es, einen \LaTeXe-Ausschnitt
#          in ein GIF-Bild umzuwandeln. Diese Umgebung wird benutzt, um die
#          in die HTML-Sprache schwer"ubersetzbaren \LaTeXe-Befehle als
#          GIF-Bilder erscheinen zu lassen.\\
#          Alles, was zwischen {\tt \#SHOW\{begin\}} und {\tt \#SHOW\{end\}}
#          steht, wird in der HTML-Version als GIF-Bild dargestellt. Es empfielt
#          sich der Variable {\tt \$USE\_SHOW\_ANTIALIAS} aus der
#          {\tt mas2tex}-Konfigurationsdatei (s. Abschnitt \ref{initdatei})
#          den Wert {\tt 1} zuzuweisen.
# @example_begin
#   Matrix f"ur Translation um $(-3, -1)$ lautet\\
#   #SHOW{begin,trans,center}             % zenriert und transparent
#     $$
#     A= 
#     \left[
#     \begin{array}{rrc}
#        1 &  0 & 0\\ 
#        0 &  1 & 0\\
#       -3 & -1 & 1
#     \end{array}
#     \right]
#     $$
#   #SHOW{end}
# @example_end
#
#   @optname (no)center
#     Das {\tt \#SHOW}-Bild wird (nicht) zentriert.
#
#   @optname input
#   @optargs \dq <inputliste>\dq
#     Es werden die Daten aus der {\tt <inputliste>} importiert, die f"ur die
#     "Ubersetzung des Ausschnitts notwendig sind.
#     \\Beispiel: {\tt input = \dq pictex,prepictex,postpictex\dq}
#
#   @optname label
#   @optargs <labelbezeichner>
#     Das {\tt \#SHOW}-Bild wird in der HTML-Version mit dem {\tt labelbezeichner} markiert. 
#     Mit dem Master-Befehl {\tt \#SHOWREF\{<labelbezeichner>\}} kann man auf 
#     das Label eine Referenz anlegen.
#
#   @optname leader
#   @optargs \dq pr"aambel\dq
#     Die Pr"aambel, die notwendig ist, um den Ausschnitt zu "ubersetzen. 
#
#   @optname scale
#   @optargs <qualit"atsfaktor>
#     Mit diesem Faktor kann man die Qualit"at des erzeugten Bildes beeinflussen. Der Faktor hat
#     reelle Werte zwischen {\tt 0.0} und {\tt 4.0}.
#     \\Defaultwert : {\tt 1.6}
#
#   @optname texpackage
#   @optargs \dq latexpackage1,latexpackage2,\dots\dq
#     \LaTeXe-Pakete, die notwendig sind, um den Ausschnitt zu "ubersetzen. Diese Pakete erweitern den
#     Defaultwert.
#     \\Defaultwert : {\tt \dq epsfig,german,a4wide,amsfonts\dq}
#
#   @optname (no)trans
#     Erzeugtes Bild ist (nicht) transparent.
# 
#
# @end

# SHOW-Work
sub show_work {
  package show;

  my ($wert,$flag);
  local ($file_basename);
  local $use_name;
  local $use_trans = $main::USE_SHOW_TRANSPARENT;
  local $use_scale = $main::USE_SHOW_SCALE;
  local $use_center = $main::USE_SHOW_CENTER;
  local $antialias = $main::USE_SHOW_ANTIALIAS;
  local $use_package;        # packages fuer LaTeX
  local $use_leader;        # Praeambel fuer LaTeX
  local $use_input;        # Inputs fuer LaTeX
  local $use_label;        # Label fuer die SHOW-Umgebung
  local $COMMAND=shift @_;
  local (@PARAMS) = @_;      # uebergebene Liste
  local $rest;          # Rest der Zeile
  local $length;          # Laenge der Umgebung
  local $status;          # Ergebnis der Konvertierung
  
  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}
  
  # Flags setzen
  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",1,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    $wert =~ s/"//g;
    SWITCH: {
      if ($flag eq "begin") {$use_name=$wert; last SWITCH;}
      if ($flag eq "end") {
        if ($main::OUTPUTFORMAT eq "latex") {
          return;
        } else {
          &main::my_exit("\#SHOW{end} without \#SHOW{begin}\n");
          return;
        }
      }
      if ($flag eq "trans") {$use_trans="1"; last SWITCH;}
      if ($flag eq "notrans") {$use_trans=""; last SWITCH;}
      if ($flag eq "texpackage") {$use_package=$wert; last SWITCH;}
      if ($flag eq "leader") {$use_leader=$wert; last SWITCH;}
      if ($flag eq "input") {$use_input=$wert; last SWITCH;}
      if ($flag eq "label") {$use_label=$wert; last SWITCH;}
      if ($flag eq "center") {$use_center="1"; last SWITCH;}
      if ($flag eq "nocenter") {$use_center=""; last SWITCH;}
      if ($flag eq "scale") {$use_scale=$wert; last SWITCH;}
      &main::my_exit("Unknown flag in SHOW : $flag\n");
    }
  }
  

  &to_flag_check;    # Syntax pruefen und wenn noetig die
            # default-Werte setzen
  
  &main::statistic_count("show");            # fuer Statistik

  # Gif-Dateiname wird generiert
  $file_basename=&generate_name;
  
  if ($main::OUTPUTFORMAT eq "html") {
    &main::my_print("SHOW in $main::INPUTFILE,$main::INPUTFILE_LINE : $main::OUTPUT_DIR/Images/Show/$file_basename.gif\n");} 

  # den Notice-Schluessel bilden
  $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_basename,$use_scale,$use_trans,$antialias);
  
  # Entsprechende Mechanismen aufrufen
  if ($main::OUTPUTFORMAT eq "html") { &html_work; }
  else {
    $main::NOTICE_NEW{$noticy_key}=$main::NOTICE_OLD{$noticy_key};
    return;
  } 
    
  
  # wenn alles geklappt, dann Notice-Eintrag machen
  unless ($status) { #wenn erfolgreich dann eintragen
    $main::NOTICE_NEW{$noticy_key}="$length";
  } 

  return;

  # Pruefe die Flags
  sub to_flag_check {
    if (($use_scale < 0.1)||($use_scale > 10)) {
      &main::my_msg("Not allowed value for \'scale\' : $use_scale.\n");
      $use_scale=1.6;
    }
  }

  # Generiere den Gif-Dateiname
  sub generate_name {
  my $subname=&main::get_base_name("$main::INPUTFILE");
    $main::SHOWCOUNTER{$main::INPUTFILE}=$main::SHOWCOUNTER{$main::INPUTFILE}+1;
    return "show_"."$subname"."_"."$use_name"."$main::SHOWCOUNTER{$main::INPUTFILE}";
  }

  # Pruefe, ob die Konventierungsroutine notwendig ist
  sub ok_to_work {
    if ($main::MAKE_SHOW) {return 1;}
    if ($main::NOT_NTCFILE) {return 0;}
    if (((exists $main::NOTICE_OLD{$noticy_key})
      and($main::NOTICE_OLD{$noticy_key} == "$length"))and
       (-f "$main::WORK_DIR/$main::OUTPUT_DIR/Images/Show/$file_basename.gif")) {return 0;}
    return 1;
  }

  # Diese Subroutine konvertiert Master-Kode in HTML-Kode        #
  # Die Grafiken werden dabei automatisch ins GIF-Format umgewandelt  #
  sub html_work {
    my ($new_name);
    my $report_state = $main::REPORT_WRITE;

    $new_name = &show_to_html;

    $main::REPORT_WRITE = 1;
    &main::write_to_texfile(
      "% *** Master-Generator - $COMMAND fuer HTML ***\n".
      "% BEGIN : $new_name\n"
    );
    &main::write_to_texfile(
      "\\begin{latexonly}\n".
      "\\begin{eqnarray}\n".
      "\t\\label{$use_label}\n".
      "\\end{eqnarray}\n".
      "\\end{latexonly}\n") if ($use_label);
    &main::write_to_texfile("\\begin{rawhtml}\n");
    &main::write_to_texfile("\t<A NAME=$use_label>\n") if ($use_label);
    &main::write_to_texfile("\t<BR><BR><CENTER>\n") if ($use_center);
    &main::write_to_texfile("\t<IMG SRC=\"$new_name\" ALIGN=\"TOP\" BORDER=\"0\">\n");
    &main::write_to_texfile("\t</CENTER><P>\n") if ($use_center);
    &main::write_to_texfile("\t<\/A>\n") if ($use_label);
    &main::write_to_texfile("\\end{rawhtml}\n");
    &main::write_to_texfile("% END : $COMMAND fuer HTML\n");
    $main::REPORT_WRITE = $report_state;
    &main::write_to_texfile("$rest\n") if ($rest);
  }


  # wandle die Latex-Umgebungen ins Gif um
  sub show_to_html {
    my $DIR_TARGET="Images/Show";
    my $NW_NAME="$DIR_TARGET/$file_basename.gif";
    my $DK=$main::SHOWDESKRIPTOR{$main::INPUTFILE};    # Deskriptor von der Master-Datei
    my ($tflag,$antialias_flag);
    my ($line);
    my $ende="";
    my $out="";
    my $lines;
    my $tex_file="$file_basename.tex";
    my $aux_file="$file_basename.aux";
    my $log_file="$file_basename.log";
    my $dvi_file="$file_basename.dvi";
    my $ps_file="$file_basename.ps";

    if (!(-d "$main::WORK_DIR/$main::OUTPUT_DIR/$DIR_TARGET")) {
      if ($DEBUG) {&main::my_print("mkdir $DIR_TARGET\n");}
      &main::get_underdir("$DIR_TARGET");
    }

    package main;
    while (!($ende)&&($line=<$DK>)) {   #lese die Datei Zeilenweise
      if ($line =~ /^(\s*)#SHOW(\s*){(\s*)end(\s*)}(.*)$/) {
        $ende=1;
        $show::rest="$5";
      } else {
        # merke fuer NTC-Datei die gesammte Laenge vom TeX-Teil
        $show::length = ($line =~ s/(.)/$1/g) + $show::length;
        $lines .= $line;
      }
    }
    package show;

    if (&ok_to_work) {
      my @inputs;

      # testen, ob PsToImg installiert ist
      &main::my_exit("No pstoimg detected, can't make image.") if ($main::PSTOIMG eq "");

      #Output-Datei zum schreiben oeffnen
      open(TEX,">$tex_file") or &main::my_exit("Error by open : $TEXFILE\n");
      &main::my_print("Convert SHOW:$use_name to $NW_NAME ...\n");
      print TEX "\\documentclass[11pt]{article}\n";
      print TEX "\\usepackage{epsfig, german, a4wide, amsfonts, $use_package}\n";
      print TEX $use_leader."\n" if ($use_leader);
      if ($use_input) {
        @inputs = split /,/, $use_input;
        foreach $use_input (@inputs) {
          print TEX "\\input $use_input\n";
        }
      }
      print TEX "\\thispagestyle{empty}\n";
      print TEX "\\begin{document}\n";
      print TEX $lines;
      print TEX "\\end{document}\n";
      close(TEX);
      unless ($main::DEBUG) {$out='> /dev/null';}
      if ($use_trans) {
        $tflag = '-transparent';
      }
      if ($antialias) { $antialias_flag = "-antialias"; }
      $status=(&main::my_system("latex $tex_file $out")||
      &main::my_system("dvips $dvi_file")||
      &main::my_system("$main::PSTOIMG -gif $tflag -out $file_basename.gif -scale $use_scale -discard -interlace $antialias_flag $ps_file")||
      move("$file_basename.gif","$main::WORK_DIR/$main::OUTPUT_DIR/$DIR_TARGET"))&&
      &main::my_msg("Convert SHOW:$use_name to $NW_NAME failed.\n");

      unlink "$dvi_file","$ps_file","$aux_file","$log_file","$tex_file";
      close(TEX);
    }
    return $NW_NAME;
  }
}

$MANAGEFUNKTION->{SHOW} = \&show_work;

1;
