#
# @command HSPACE
# @title   Einf"ugen von horizontalen Abst"anden
# @index   
# @syntax  \#HSPACE\{<LaTeX-L"angenangabe>,<HTML-Abstand>\}
# @desc    Durch den {\tt \#HSPACE}-Befehl kann ein horizontaler Abstand eingef"ugt werden. 
# @example_begin
# @example_end
#
#   @optname <LaTeX-L"angenangabe>
#   @optlabel -
#     Feste \TeX-L"angenangabe (optional, nur f"ur \LaTeXe-Version)
#
#   @optname <HTML-Abstand>
#   @optlabel -
#     Anzahl {\tt \&nbsp;\&nbsp;\&nbsp;\&nbsp;} als horizontaler Abstand (optional, nur f"ur HTML-Version)
#
# @command VSPACE
# @title   Einf"ugen von vertikalen Abst"anden
# @index   
# @syntax  \#VSPACE\{<LaTeX-L"angenangabe>,<HTML-Abstand>\}
# @desc    Durch den {\tt \#VSPACE}-Befehl kann ein vertikaler Abstand eingef"ugt werden. 
# @example_begin
#   #VSPACE{1cm,2}   % 1cm vert. Abstand in LaTeX-Version
#                    % <P><P> in der HTML-Version
# @example_end
#
#   @optname <LaTeX-L"angenangabe>
#   @optlabel -
#     Feste \TeX-L"angenangabe (optional, nur f"ur \LaTeXe-Version)
#
#   @optname <HTML-Abstand>
#   @optlabel -
#     Anzahl Paragraphen als vertikaler Abstand (optional, nur f"ur HTML-Version)
#
# @end

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{VSPACE}    = \&vspace_work;
$MANAGEFUNKTION->{HSPACE}    = \&hspace_work;

# Erzeugt ein vertikaler Zwischenraum
sub vspace_work {
  my $html_space;
  my $latex_space;
  my $COMMAND = shift;
  my (@PARAMS) = @_;      # uebergebene Liste
  my $term;

  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    SWITCH: {
      if ($flag =~ /^(\d+)$/) { $html_space = $flag; last SWITCH; }
      elsif ($flag =~ /^(\d+\.?\d*|\.\d+)(pt|pc|in|bp|cm|mm|dd|cc|sp)$/) { $latex_space = $flag }
      else { &main::my_exit("Unknown measure in $COMMAND : $flag\n"); }
    }
  }
  if ($main::OUTPUTFORMAT eq "html") {
    my $space;

    while ($html_space > 0) {
      $space .= "<P>";
      $html_space--;
    }
    &main::write_to_texfile(
      "\\begin\{rawhtml\}\n".
      "\t$space\n".
      "\\end\{rawhtml\}\n"
    ) if ($space);
  } else {
    &main::write_to_texfile("\\vspace*\{$latex_space\}\n") if ($latex_space);
  }
}

# Erzeugt horizontaler Zwischenraum
sub hspace_work {
  my $html_space;
  my $latex_space;
  my $COMMAND = shift;
  my (@PARAMS) = @_;      # uebergebene Liste
  my $term;

  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    SWITCH: {
      if ($flag =~ /^(\d+)$/) { $html_space = $flag; last SWITCH; }
      elsif ($flag =~ /^(\d+\.?\d*|\.\d+)(pt|pc|in|bp|cm|mm|dd|cc|sp)$/) { $latex_space = $flag }
      else { &main::my_exit("Unknown measure in $COMMAND : $flag\n"); }
    }
  }
  if ($main::OUTPUTFORMAT eq "html") {
    my $space;

    while ($html_space > 0) {
      $space .= "&nbsp;&nbsp;&nbsp;&nbsp;";
      $html_space--;
    }
    &main::write_to_texfile(
      "\\begin\{rawhtml\}\n".
      "\t$space\n".
      "\\end\{rawhtml\}\n"
    ) if ($space);
  } else {
    &main::write_to_texfile("\\hspace*\{$latex_space\}\n") if ($latex_space);
  }
}

1;

