#
# @command TABBING
# @title   Tabellen mit Tabulatoren
# @index   
# @syntax  \#TABBING\{begin,optionen\}\\...\\\#TABBING\{end\}
# @desc    Die {\tt \#TABBING}-Umgebung stellt eine einfache Art des Spaltensatzes vor. Die horizontalen
#          Positionen werden mit Tabulatoren markiert, die dann durch den  "`Sprungbefehl"' {\tt \#TAB} oder 
#          ({\tt \textbackslash >}) erreicht werden. F"ur die \LaTeXe-Version wird automatisch folgende Musterzeile
#          generiert:
#          \begin{verbatim}
#                  mmm\=mmm\=mmm\= ... \mmm\=\kill
#          \end{verbatim}
#          Die n"achste Tabulatorposition wird durch den {\tt \textbackslash >}-Befehl erreicht. Zeilen werden
#          wie gew"ohnlich mit \textbackslash\textbackslash\  beendet.\\
#          Eine bequeme M"oglichkeit, auf eine Tabulatorposition zu "`springen"', bietet der 
#          {\tt \#TAB\{<tabposition>\}}-Befehl, wobei {\tt tabposition} eine nat"urliche Zahl ist.
#          Dieser Befehl setzt alle nachfolgenden Zeilen in der {\tt \#TABBING}-Umgebung auf die Tabulatorposition
#          {\tt tabposition}.
# @example_begin
#  #TABBING{begin,font=it}      % Font italic 
#      Zeile 1\\
#      #TAB{1}                  % die naechsten Zeilen beginnen ab
#      Zeile 2\\                % Tabulatorposition 1
#      Zeile 3\\
#      #TAB{0}                  % kein Tabulator vor der Zeile
#      Zeile 4
#  #TABBING{end}
# @example_end
#
#   @optname font
#   @optargs <latexfont>
#       Font {\tt latexfont} f"ur die Umgebung setzen. Bei der HTML-Version wird
#       ein \dq passender\dq\  HTML-Font gesetzt.
#       \\Defaultwert: {\tt font=tt}
#
#   @optname tabstop
#   @optargs <tabgr"o"se>
#       Horizontalen Abstand f"ur die Musterzeile setzen. Der Wert von
#       {\tt tabstop} ({\tt tabgr"o"se}) ist eine nat"urliche Zahl.
#       \\Defaultwert: {\tt tabstop=3} ($\cong$ mmm)
#
# @command TAB
# @title   Tabulatorschritt in {\tt \#TABBING}-Umgebung anspringen
# @index   
# @syntax  \#TAB\{nr\}
# @desc    Sprint innerhalb einer {\tt \#TABBING}-Umgebung
#          an eine Tabulatorposition.
#
# @example_begin
#  #TABBING{begin,font=it}      % Font italic 
#      Zeile 1\\
#      #TAB{1}                  % die naechsten Zeilen beginnen ab
#      Zeile 2\\                % Tabulatorposition 1
#      Zeile 3\\
#      #TAB{0}                  % kein Tabulator vor der Zeile
#      Zeile 4
#  #TABBING{end}
# @example_end
#
#   @optname nr
#     Nummer der Tabulatorposition, 0 entspricht dem Zeilenanfang.
#
# @end

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{TABBING}   = \&tabbing_work;
$MANAGEFUNKTION->{TAB}       = \&tabbing_work;

$TABBING_OK;        # Variable fuer die TABBING-Umgebung

# TABBING/TAB
sub tabbing_work {
  package tabbing;

  my $COMMAND = shift @_;
  my $flag    = shift @_;
  $font;
  my $wert;
  my $tabstop;
  my $tab;
  my $space;
  
  $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
  if ($COMMAND eq "TABBING") {
    if ($flag eq "begin") {
      if ($main::TABBING_OK) { &main::my_exit("\#TABBING{begin} without \#TABBING{end}\n"); }
      # Flags setzen
      foreach $term (@_) {    
        ($flag,$wert)=&main::my_quotewords("=",0,$term);
        $flag =~ s/(\s*)//;      # Trennzeichen loeschen
        SWITCH: {
          if ($flag eq "font") {$font=$wert; last SWITCH;}
          if ($flag eq "tabstop") {
            $tabstop=$wert; 
            if ($tabstop =~ /(\D+)/) {
              &main::my_msg("Flag $flag in $COMMAND is not positiv digit, set $flag=3\n");
              $tabstop=3;
            }
            last SWITCH;
          }
          &main::my_exit("Unknown flag in $COMMAND : $flag\n");
        }
      }
      if (!($tabstop)) { $tabstop=3; }
      while ($tabstop > 0) {
        $tab .= "m";
        $tabstop--;
      }
      print main::OUT "% *** Master-Generator - TABBING-Umgebung ***\n";
      $main::TABBING_OK = "1";
      if ($main::OUTPUTFORMAT eq "html") {
        &main::replace_in_lines("\\\\>",
          "\\begin{rawhtml} &nbsp;&nbsp;&nbsp;&nbsp; \\end{rawhtml}\n");
        $font = &main::font_to_html("$font");
        print main::OUT "\\begin{rawhtml}\n";
        print main::OUT "<BR><P>";
        print main::OUT "<$font>" if ($font);
        print main::OUT "\n";
        print main::OUT "\\end{rawhtml}\n";
      } else {
        if ($font) {
          print main::OUT "{\\$font\n";
        } else {
          print main::OUT "{\\tt\n";
        }
        print main::OUT "\\begin{tabbing}\n";
        print main::OUT "$tab\\=$tab\\=$tab\\=$tab\\=$tab\\=$tab\\=$tab\\=\\kill\n";
      }
    } elsif ($flag eq "end") {
      if (!($main::TABBING_OK)) { &main::my_exit("\#TABBING{end} without \#TABBING{begin}\n"); }
      $main::TABBING_OK = "";
      if ($main::OUTPUTFORMAT eq "html") {
        &main::replace_in_lines("","");
        print main::OUT "\\begin{rawhtml}\n";
        print main::OUT "<\/$font>" if ($font);
        print main::OUT "<BR>\n";
        print main::OUT "\\end{rawhtml}\n";
      } else {
        print main::OUT "\\end{tabbing}\n";
        print main::OUT "}\n";
      }
      # setze kein Eintrag vor der Zeile
      &main::set_in_either_line("");
    } else { &main::my_exit("Unknown flag in TABBING : $flag\n"); }
  } elsif ($COMMAND eq "TAB") {
    # wenn TAB ausserhalb der TABBING-Umgebung, dann Fehlerausgabe
    if (!($main::TABBING_OK)) { &main::my_exit("$COMMAND outside of TABBING{}\n");}
    # wenn Wert in TAB keine Zahl, oder kleiner als 0
    if (($flag =~ /(\D+)/)||($flag < 0)) {
      &main::my_exit("Not allowed value for \#TAB : $flag\n");
    }
    # HTML
    if ($main::OUTPUTFORMAT eq "html") { 
      $space .= "\\begin{rawhtml}\n";
      while ($flag > 0) {
        $space .= '&nbsp;&nbsp;&nbsp;&nbsp;';
        $flag--;
      }
      $space .= "\n\\end{rawhtml}\n";
      # LaTeX/PDF
    } else {    
      while ($flag > 0) {
        $space .= '\\> ';
        $flag--;
      }
    }
    # setze space vor jeder weiterer Zeile in die OUTPUT-Datei
    &main::set_in_either_line($space);
  }  
}

1;

