#
# @command TABBEGIN
# @title   Tabellen beginnen
# @index   
# @syntax  \#TABBEGIN\{optionen\}\\\%LaTeX-Tabelle\\$\backslash$begin\{tabular\}\\...\\$\backslash$end\{tabular\}\\\#TABEND\{\}
# @desc    Die {\tt \#TABBEGIN}-{\tt \#TABEND}-Umgebung erm"oglicht es,
#          eine Tabelle in das Masterdokument einzuf"ugen.
# @example_begin
#   #TABBEGIN{text="Bemerkenswerte Geschwindigkeiten"}
#   \begin{tabular}{lrr}
#   \hline
#   &{\tt m/s}&{\tt km/h}\\
#   \hline
#   Haarwachstum&{\tt 0.000 000 003}&{\tt 0.000 000 011}\\
#   Schnecke&{\tt 0.002}&{\tt 0.007}\\
#   Fu"sg"anger&{\tt 1.4}&{\tt 5}\\
#   Radfahrer&{\tt 10}&{\tt 36}\\
#   Orkan&{\tt 45}&{\tt 162}\\
#   D"usenverkehrsflugzeug&{\tt 250}&{\tt 900}\\
#   Schall in Luft&{\tt 332}&{\tt 1195}\\
#   Mondrakete&{\tt 11 084}&{\tt 39 903}\\
#   Erde um die Sonne&{\tt 30 000}&{\tt 108 000}\\
#   Licht im Vakuum&{\tt 299 792 458}&{\tt 1 079 252 849}\\
#   \hline
#   \end{tabular}
#   #TABEND{}
# @example_end
#
#   @optname text
#   @optargs \dq tabellenunterschrift\dq
#       Unterschrift der Tabelle. Die Sonderzeichen (s. Abschnitt \ref{sonderzeichen}) 
#       m"ussen mit \textbackslash (backslash) gesch"utzt werden. 
#
#   @optname (no)figure
#       Die Tabelle (nicht) als Gleitobjekt setzen.
#
#   @optname label
#   @optargs <labelbezeichner>
#       Die Tabelle wird im Dokument mit {\tt labelbezeichner} markiert. Mit dem
#       \LaTeXe-Befehl {\tt \textbackslash ref\{labelbezeichner\}} kann man die
#       Tabelle referenzieren. 
#
# @command TABEND
# @title   Tabelle beenden
# @index   
# @syntax  \#TABBEGIN\{optionen\}\\\%LaTeX-Tabelle\\$\backslash$begin\{tabular\}\\...\\$\backslash$end\{tabular\}\\\#TABEND\{\}
# @desc    Beendet die mit {\tt \#TABBEGIN} begonnene Tabellenumgebung.
# @example_begin
# @example_end
#
# @end

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{TABBEGIN}  = \&tabelle_work;
$MANAGEFUNKTION->{TABEND}    = \&tabelle_work;

$TABELLE_OK;

# TABBEGIN-TABEND-Umgebung
sub tabelle_work {
  package tabelle;

  my $COMMAND = shift @_;
  $text;
  $tab_figure;
  $tab_label;
  my $tab_options = $main::FIGURE_OPTIONS;
  my $flag;
  my $wert;
  my $term;
  my $label;
  
  if ($COMMAND eq "TABBEGIN") {
    if ($main::TABELLE_OK) { &main::my_exit("\#TABBEGIN{} without \#TABEND{}\n"); }
    $main::TABELLE_OK = 1;
    $tab_figure = $main::USE_TABLE;

    # Flags setzen
    foreach $term (@_) {    
      ($flag,$wert)=&main::my_quotewords("=",0,$term);
      $flag =~ s/(\s*)//;      # Trennzeichen loeschen
      SWITCH: {
        if ($flag eq "text") {$text=$wert; last SWITCH;}
        if ($flag eq "figure") { $tab_figure = 1; last SWITCH; }
        if ($flag eq "label") { $tab_label = $wert; last SWITCH; }
        if ($flag eq "nofigure") { $tab_figure = 0; last SWITCH; }
        &main::my_exit("Unknown flag in $COMMAND : $flag\n");
      }
    }
    $tab_figure = 1 if ($tab_label);
    $tab_options = "htb" unless ($tab_options);
    
    # TABBEGIN fuer die HTML-Version
    if ($main::OUTPUTFORMAT eq "html") {
      &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer HTML ***\n");
      
      # Dieses Teil braucht man fuer die Zaehlung von
      # Tabellen in der HTML-Version.
      &main::write_to_texfile(
        "\\begin{latexonly}\n".
        "\\begin{table}[$tab_options]\n") if ($tab_figure);
      &main::write_to_texfile("\t\\caption{\\label{$tab_label}}\n") if ($tab_label);
      &main::write_to_texfile(
        "\\end{table}\n".
        "\\end{latexonly}\n") if ($tab_figure);
      &main::write_to_texfile("\\begin{center}\n") if ($main::USE_CENTER);

    # TABBEGIN fuer die PDF/LaTeX-Version
    } else {
      &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer LaTeX ***\n");
      &main::write_to_texfile("\\begin{table}[$tab_options]\n") if ($tab_figure);
      &main::write_to_texfile("\\begin{center}\n") if ($main::USE_CENTER);
    }
  } else {
    unless ($main::TABELLE_OK) { &main::my_exit("\#TABEND{} without \#TABBEGIN{}\n"); }
    $main::TABELLE_OK = 0;
    
    # TABEND fuer die HTML-Version
    if ($main::OUTPUTFORMAT eq "html") {
      &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer HTML ***\n");
      if ($text) {
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t<$main::HTML_SIGN_FONT>\n".
          "\\end{rawhtml}\n"
        ) if ($main::HTML_SIGN_FONT);
        &main::write_to_texfile($text."\n");
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t<\/$main::HTML_SIGN_FONT>\n".
          "\\end{rawhtml}\n"
        ) if ($main::HTML_SIGN_FONT);
      &main::write_to_texfile("\\end{center}\n") if ($main::USE_CENTER);
      }

    # TABEND fuer die PDF/LaTeX-Version
    } else {
      &main::write_to_texfile("% *** Master-Generator - $COMMAND fuer LaTeX ***\n");
      if ($tab_figure) {
        if ($text) {
          # bei der Caption-Umgebung einige Befehle
          # mit \protect-Befehl schuetzen
          $text =~ s/\\(begin|end|\\)/\\protect\\$1/g;
          
          if ($tab_label) {
            $label="\\label{$tab_label}"; 
          }

          &main::write_to_texfile("\t\\caption{$label \\small ".$text."}\n");
        } else {
          &main::write_to_texfile("\t\\caption{$label}\n");
        }
      } else {
        &main::write_to_texfile("\n\t{\\small \\rm ".$text."}\n") if ($text);
      }
      &main::write_to_texfile("\\end{center}\n") if ($main::USE_CENTER);
      &main::write_to_texfile("\\end{table}\n") if ($tab_figure);
    }
  }
}

1;
