#
# @command VIEWCODE
# @title   Einbinden von Textdateien
# @index   
# @syntax  \#VIEWCODE\{optionen\}
# @desc    Mit dem Master-Befehl {\tt \#VIEWCODE} kann der Inhalt einer Datei 
#          als {\tt verbatim}-Umgebung in das Master-Dokument eingef"ugt werden. Dieser Befehl wird benutzt, um
#          z.B. Programmcode im Dokument anzuzeigen. In der HTML-Version kann auch ein Link
#          auf die Datei erzeugt werden.
# @example_begin
#   ...
#   Inhalt der Datei space.wrl:
#   #VIEWCODE{file="./Source/space.wrl"}
#   ...
# @example_end
#
#   @optname count
#   @optargs <zeilen>
#       Anzahl der Zeilen, die angezeigt werden.
#
#   @optname file
#   @optargs <datei>
#       Die Datei, deren Inhalt in das Dokument eingef"ugt werden soll.
#
#   @optname htmlonly
#       Die Datei oder ein Link erscheint nur in der HTML-Version.
#
#   @optname sign
#   @optargs \dq linkname\dq
#       Bei dieser Option wird in der HTML-Version ein Link mit dem Namen {\tt sign} 
#       auf die Datei {\tt file} erzeugt. In der \LaTeXe-Version erscheint {\tt sign}
#       als Unterschrift zu dem Rahmen mit dem Dateiinhalt.
#
#   @optname (no)inline
#       In der HTML-Version wird mit dem Schalter inline die Quelle direkt
#       in die HTML-Seite eingebunden. Mit noinline (Standardeinstellung)
#       wird nur ein Link auf die Quelle eingebaut.
#
#   @optname (no)box
#       Diese Option schaltet in der \LaTeX-Version den Rahmen um die Datei {\tt file} ab (aus).
#       Falls die Datei {\tt file} in der \LaTeXe-Version nicht auf eine Seite pa"st,
#       wird die Umrahmung automatisch abgeschaltet, auch wenn die Option {\tt box}
#       angegeben wurde.
#
# @end

# Mas2TeX-Befehle definieren
$MANAGEFUNKTION->{VIEWCODE}  = \&viewcode_work;

# Bindet eine Textdatei in das Dokument ein
# VIEWCODE
sub viewcode_work {
  my $COMMAND = shift @_;     # Erstes Argument ist der Befehlsname,
                              # in diesem Fall SHELL
  my (@PARAMS) = @_;          # uebergebene Liste (Optionen)
  my $file;
  my $relativ_name;
  my $text;
  my $box;
  my $inline = 0;
  my $nobox;
  my $htmllink;
  my $vc_count=-1;
  my $line;
  my $line_count;
  my $view_font;
  my $view_dir;
  my $view_tmp;
  my $viewcode_center;
  
  my $option;                 # Laufende Variable, geht die Optionen durch
  my ($flag,$wert);           # $flag = Optionsname (command)

  # gehe die Optionen durch
  foreach $option (@PARAMS) {
     # erfahre den Optionsname und dazugeh"origen Wert  
    ($flag,$wert)=&main::my_quotewords("=",0,$option);

    # loesche die moeglichen Trennzeichen
    $flag =~ s/(\s*)//g;

    # pruefe die Optionen
    SWITCH: {
      if ($flag eq "file") {$file=$wert; last SWITCH;}
      if (($flag eq "text")||($flag eq "sign")) {$text=$wert; last SWITCH;}
      if ($flag eq "count") {$vc_count=$wert; last SWITCH;}
      if ($flag eq "center") {$viewcode_center = 1; last SWITCH;}
      if ($flag eq "box") {$box=1; last SWITCH;}
      if ($flag eq "nobox") {$nobox=1; last SWITCH;}
      if ($flag eq "inline") {$inline=1; last SWITCH;}
      if ($flag eq "noinline") {$inline=0; last SWITCH;}
      if ($flag eq "htmlonly") {
        return if ($main::OUTPUTFORMAT ne "html"); 
        last SWITCH;
      }
      # falls unerlaubte Option, dann Fehlermeldung
      &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }
  
  if ($vc_count==0) { $vc_count=-1 };

  # loesche die moeglichen Trennzeichen
  $file =~ s/(\s*)//g;

  $relativ_name = &main::get_relativ_name($file); 

  &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : File=$relativ_name\n");

  unless (-f "$file") {
    &main::my_msg("File $relativ_name not found in $COMMAND.\n");
    return;
  } 

  # Dateiname fr Ausgabedatei setzen
  $view_dir = &main::get_dir_name($file);
  if ($vc_count > 0) {
    $view_tmp = &main::get_base_name($file)."_".$vc_count.".".&main::get_file_type($file);
  } else { 
    $view_tmp = &main::get_base_name($file).".".&main::get_file_type($file);
  }

  &main::write_to_texfile("% *** Master-Generator - $COMMAND ($relativ_name) ***\n");

  # in der PDF/LaTeX-Version als Text
  if (($main::OUTPUTFORMAT eq "latex") || ($main::OUTPUTFORMAT eq "pdf")) {
    # wenn die Datei groesser als die Seite ist,
    # dann wird die Umramung in der LaTeXe-Version 
    # abgeschaltet
    &main::get_underdir($view_dir);
    open (VIEWFILE,"$file") or &main::my_exit("Error by open: $file\n");
    open (TARGETFILE,">$main::OUTPUT_DIR/$view_dir/$view_tmp") 
        or &main::my_exit("Error by open: $main::OUTPUT_DIR/$view_dir/$view_tmp\n");
    while (($line=<VIEWFILE>)&&($line_count != $vc_count)) {
      print TARGETFILE $line; 
      $line_count++; 
    }
    close TARGETFILE;
    close VIEWFILE;
    if ($line_count > 50) { 
      $nobox = 1; 
    }

    # Font setzen  
    if ($main::VIEWCODE_FONT) { $view_font = $main::VIEWCODE_FONT; }
    # Default-Font
    else { $view_font = "footnotesize"; }
    &main::write_to_texfile(
      "\n".
      "\\fbox{\\parbox{16cm}{\n"
    ) unless ($nobox);
    &main::write_to_texfile("{\\$view_font \\verbatiminput{$main::OUTPUT_DIR/$view_dir/$view_tmp}}\n");
    unless ($nobox) {
     &main::write_to_texfile("}}\n");
     &main::write_to_texfile("\\begin{center}{\\it $text}\\end{center}\n") if ($text);
     &main::write_to_texfile("\\vspace{3mm}\n\n");
    }

  # HTML-Version 
  } else {
    # Html-Unterschrift anpasen
    if ($text) {
      $text=&main::sign_to_html($text);
    }

    # als Link
    if ($text && ($inline == 0)) {
      # Datei in HTML umwandeln
      &main::get_underdir($view_dir);
      open (TARGETFILE,">$main::OUTPUT_DIR/$view_dir/$view_tmp.html") 
          or &main::my_exit("Error by open: $main::OUTPUT_DIR/$view_dir/$view_tmp.html\n");
      print TARGETFILE &main::txt_to_html($file,1);
      close TARGETFILE;

      if ($file=~/^\//) { $htmllink = $file; }
      #else { $htmllink = "/"."$relativ_name"; }
      else { $htmllink = "$relativ_name"; }
      if ($viewcode_center) {
        &main::write_to_texfile("\\begin{center}\n");
      }
      &main::write_to_texfile(
        "\t\\begin{rawhtml}".
        "<A HREF=\"$htmllink.html\">".
        "\\end{rawhtml}".
        "$text".
        "\\begin{rawhtml}".
        #"\&nbsp;".
        "<\/A>".
        "\\end{rawhtml}\n"
      );
      if ($viewcode_center) {
        &main::write_to_texfile("\\end{center}\n");
      }
    # als Text
    } else {
      if ($box) {
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t<TABLE WIDTH=100\% BORDER>\n"
        );
        if ($text) {
          &main::write_to_texfile("\t<CAPTION align=bottom>$text</CAPTION>\n");
        }
        &main::write_to_texfile( "\t<TR><TD><PRE>\n" );
        &main::write_to_texfile( &main::txt_to_html($file) );
        &main::write_to_texfile(
          "</PRE></TD></TR>\n".
          "\t</TABLE>\n".
          "\\end{rawhtml}\n"
        );
      } else {
        &main::write_to_texfile("\\begin{rawhtml}\n<PRE>\n");
        &main::write_to_texfile( &main::txt_to_html($file) );
        &main::write_to_texfile("</PRE>\n\\end{rawhtml}\n");
      }
    }
  }
}

1;
